@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This application is open source and may be redistributed and/or modified   
  * freely and without restriction, both in commericial and non commericial applications,
@@ -9,6 +9,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
 
+#ifdef WIN32
+    #pragma warning(disable : 4503)
+#endif // WIN32
+
 #include <osg/Texture2D>
 #include <osg/Geometry>
 #include <osg/State>
@@ -85,37 +89,37 @@ void ellipsodeTransformTest(double latitude, double longitude, double height)
 }
 
 void processFile(std::string filename,
-		   osgTerrain::DataSet::Source::Type type,
-		   std::string currentCS, 
-		   osg::Matrixd &geoTransform,
-		   bool geoTransformSet,
-		   bool geoTransformScale,
+                   osgTerrain::DataSet::Source::Type type,
+                   std::string currentCS, 
+                   osg::Matrixd &geoTransform,
+                   bool geoTransformSet,
+                   bool geoTransformScale,
                    bool minmaxLevelSet, unsigned int min_level, unsigned int max_level,
                    unsigned int layerNum,
-		   osg::ref_ptr<osgTerrain::DataSet> dataset) {
+                   osg::ref_ptr<osgTerrain::DataSet> dataset) {
 
     if(filename.empty()) return;
 
     if(osgDB::fileType(filename) == osgDB::REGULAR_FILE) {
-	
-	osgTerrain::DataSet::Source* source = new osgTerrain::DataSet::Source(type, filename);                
-	if (source)
-	{
-	    if (!currentCS.empty())
-	    {
-		std::cout<<"source->setCoordySystem "<<currentCS<<std::endl;
-		source->setCoordinateSystemPolicy(osgTerrain::DataSet::Source::PREFER_CONFIG_SETTINGS);
-		source->setCoordinateSystem(currentCS);
-	    } 
-	    
-	    if (geoTransformSet)
-	    {
-		std::cout<<"source->setGeoTransform "<<geoTransform<<std::endl;
-		source->setGeoTransformPolicy(geoTransformScale ? 
-					      osgTerrain::DataSet::Source::PREFER_CONFIG_SETTINGS_BUT_SCALE_BY_FILE_RESOLUTION : 
-					      osgTerrain::DataSet::Source::PREFER_CONFIG_SETTINGS);
-		source->setGeoTransform(geoTransform);
-	    }
+        
+        osgTerrain::DataSet::Source* source = new osgTerrain::DataSet::Source(type, filename);                
+        if (source)
+        {
+            if (!currentCS.empty())
+            {
+                std::cout<<"source->setCoordySystem "<<currentCS<<std::endl;
+                source->setCoordinateSystemPolicy(osgTerrain::DataSet::Source::PREFER_CONFIG_SETTINGS);
+                source->setCoordinateSystem(currentCS);
+            } 
+            
+            if (geoTransformSet)
+            {
+                std::cout<<"source->setGeoTransform "<<geoTransform<<std::endl;
+                source->setGeoTransformPolicy(geoTransformScale ? 
+                                              osgTerrain::DataSet::Source::PREFER_CONFIG_SETTINGS_BUT_SCALE_BY_FILE_RESOLUTION : 
+                                              osgTerrain::DataSet::Source::PREFER_CONFIG_SETTINGS);
+                source->setGeoTransform(geoTransform);
+            }
             if (minmaxLevelSet) 
             {
                 source->setMinMaxLevel(min_level, max_level);
@@ -124,26 +128,26 @@ void processFile(std::string filename,
             source->setLayer(layerNum);
             
             std::cout<<"Reading as layer = "<<layerNum<<std::endl;
-	    
-	    dataset->addSource(source);
-	}
+            
+            dataset->addSource(source);
+        }
     } else if (osgDB::fileType(filename) == osgDB::DIRECTORY) {
 
-	osgDB::DirectoryContents dirContents= osgDB::getDirectoryContents(filename);
-	
-	// loop through directory contents and call processFile
-	std::vector<std::string>::iterator i;
-	std::string fullfilename;
-	for(i = dirContents.begin(); i != dirContents.end(); ++i) {
-	    if((*i != ".") && (*i != "..")) {
-		fullfilename = filename + '/' + *i;
-		processFile(fullfilename, type, currentCS, 
+        osgDB::DirectoryContents dirContents= osgDB::getDirectoryContents(filename);
+        
+        // loop through directory contents and call processFile
+        std::vector<std::string>::iterator i;
+        std::string fullfilename;
+        for(i = dirContents.begin(); i != dirContents.end(); ++i) {
+            if((*i != ".") && (*i != "..")) {
+                fullfilename = filename + '/' + *i;
+                processFile(fullfilename, type, currentCS, 
                             geoTransform, geoTransformSet, geoTransformScale, 
                             minmaxLevelSet, min_level, max_level,
                             layerNum,
                             dataset);
-	    }
-	}
+            }
+        }
     }
 }
 
@@ -156,7 +160,7 @@ int main( int argc, char **argv )
     
     // set up the usage document, in case we need to print out how to use this program.
     arguments.getApplicationUsage()->setApplicationName(arguments.getApplicationName());
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the standard OpenSceneGraph example which loads and visualises 3d models.");
+    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" application is utility tools which can be used to generate paged geospatial terrain databases.");
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
     arguments.getApplicationUsage()->addCommandLineOption("-d <filename>","Specify the digital elevation map input file to process");
     arguments.getApplicationUsage()->addCommandLineOption("-t <filename>","Specify the texture map input file to process");
@@ -164,6 +168,7 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->addCommandLineOption("-a <archivename>","Specify the archive to place the generated database");
     arguments.getApplicationUsage()->addCommandLineOption("-o <outputfile>","Specify the output master file to generate");
     arguments.getApplicationUsage()->addCommandLineOption("-l <numOfLevels>","Specify the number of PagedLOD levels to generate");
+    arguments.getApplicationUsage()->addCommandLineOption("--image-ext <ext>","Specify the Image format to output to via its plugin name, i.e. rgb, dds, jp2, jpeg.");
     arguments.getApplicationUsage()->addCommandLineOption("--levels <begin_level> <end_level>","Specify the range of lavels that the next source Texture or DEM will contribute to.");
     arguments.getApplicationUsage()->addCommandLineOption("--layer <layer_num>","Specify the layer that the next source Texture will contribute to..");
     arguments.getApplicationUsage()->addCommandLineOption("-e <x> <y> <w> <h>","Extents of the model to generate");
@@ -184,14 +189,16 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->addCommandLineOption("--RGB-16","Use 16bit RGBA destination imagery");     
     arguments.getApplicationUsage()->addCommandLineOption("--RGBA","Use 32bit RGBA destination imagery");     
     arguments.getApplicationUsage()->addCommandLineOption("--max-visible-distance-of-top-level","Set the maximum visible distance that the top most tile can be viewed at");     
+    arguments.getApplicationUsage()->addCommandLineOption("--no-terrain-simplification","Switch off terrain simplification.");
+    arguments.getApplicationUsage()->addCommandLineOption("--default-color <r,g,b,a>","Sets the default color of the terrain.");
     arguments.getApplicationUsage()->addCommandLineOption("--radius-to-max-visible-distance-ratio","Set the maximum visible distance ratio for all tiles apart from the top most tile. The maximum visuble distance is computed from the ratio * tile radius.");     
     arguments.getApplicationUsage()->addCommandLineOption("--no-mip-mapping","Disable mip mapping of textures");     
     arguments.getApplicationUsage()->addCommandLineOption("--mip-mapping-hardware","Use mip mapped textures, and generate the mipmaps in hardware when available.");     
     arguments.getApplicationUsage()->addCommandLineOption("--mip-mapping-imagery","Use mip mapped textures, and generate the mipmaps in imagery.");     
     arguments.getApplicationUsage()->addCommandLineOption("--max-anisotropy","Max anisotropy level to use when texturing, defaults to 1.0.");
-    arguments.getApplicationUsage()->addCommandLineOption("--bluemarble-east","");     
-    arguments.getApplicationUsage()->addCommandLineOption("--bluemarble-west","");     
-    arguments.getApplicationUsage()->addCommandLineOption("--whole-globe","");
+    arguments.getApplicationUsage()->addCommandLineOption("--bluemarble-east","Set the coordinates system for next texture or dem to represent the eastern hemisphere of the earth.");     
+    arguments.getApplicationUsage()->addCommandLineOption("--bluemarble-west","Set the coordinates system for next texture or dem to represent the western hemisphere of the earth.");     
+    arguments.getApplicationUsage()->addCommandLineOption("--whole-globe","Set the coordinates system for next texture or dem to represent the whole hemisphere of the earth.");
     arguments.getApplicationUsage()->addCommandLineOption("--geocentric","");
     arguments.getApplicationUsage()->addCommandLineOption("--range","");     
     arguments.getApplicationUsage()->addCommandLineOption("--xx","");     
@@ -211,7 +218,7 @@ int main( int argc, char **argv )
     float x,y,w,h;
     while (arguments.read("-e",x,y,w,h))
     {
-        dataset->setDestinationExtents(osg::BoundingBox(x,y,0.0f,x+w,y+h,0.0f));
+        dataset->setDestinationExtents(osgTerrain::GeospatialExtents(x,y,x+w,y+h));
     }
     
     while (arguments.read("--HEIGHT_FIELD"))
@@ -263,8 +270,7 @@ int main( int argc, char **argv )
     std::string archiveName;
     while (arguments.read("-a",archiveName)) { dataset->setArchiveName(archiveName); }
 
-    dataset->setDestinationTileBaseName("output");
-    dataset->setDestinationTileExtension(".ive");
+    dataset->setDestinationName("output.ive");
 
 
     unsigned int numLevels = 10;
@@ -296,6 +302,46 @@ int main( int argc, char **argv )
         dataset->setRadiusToMaxVisibleDistanceRatio(radiusToMaxVisibleDistanceRatio);
     }
 
+    while (arguments.read("--no_terrain_simplification") ||
+           arguments.read("--no-terrain-simplification"))
+    {
+        dataset->setSimplifyTerrain(false);
+    }
+
+    std::string str;
+    while (arguments.read("--default-color",str) ||
+           arguments.read("--default_color",str))
+    {
+        osg::Vec4 defaultColor;
+        if( sscanf( str.c_str(), "%f,%f,%f,%f",
+                &defaultColor[0], &defaultColor[1], &defaultColor[2], &defaultColor[3] ) != 4 )
+        {
+            std::cout<<"Color argument format incorrect."<<std::endl;
+            return 1;
+        }
+        dataset->setDefaultColor(defaultColor);
+    }
+
+    std::string image_ext;
+    while (arguments.read("--image-ext",image_ext))
+    {
+        std::string::size_type dot = image_ext.find_last_of('.');
+        if (dot!=std::string::npos) image_ext.erase(0,dot+1);
+
+        osgDB::ReaderWriter* rw = osgDB::Registry::instance()->getReaderWriterForExtension(image_ext);
+        if (rw) 
+        {
+            image_ext.insert(0,".");
+            dataset->setDestinationImageExtension(image_ext);
+            return 1;
+        }
+        else
+        {
+            std::cout<<"Error: can not find plugin to write out image with extension '"<<image_ext<<"'"<<std::endl;
+            return 1;
+        }
+    }
+
 
     // if user request help write it out to cout.
     if (arguments.read("-h") || arguments.read("--help"))
@@ -494,8 +540,8 @@ int main( int argc, char **argv )
 
         else if (arguments.read(pos, "-d",filename))
         {
-	    std::cout<<"-d "<<filename<<std::endl;
-	    processFile(filename, osgTerrain::DataSet::Source::HEIGHT_FIELD, currentCS, 
+            std::cout<<"-d "<<filename<<std::endl;
+            processFile(filename, osgTerrain::DataSet::Source::HEIGHT_FIELD, currentCS, 
                         geoTransform, geoTransformSet, geoTransformScale,
                         minmaxLevelSet, min_level, max_level,
                         currentLayerNum,
@@ -513,8 +559,8 @@ int main( int argc, char **argv )
         }
         else if (arguments.read(pos, "-t",filename))
         {
-	    std::cout<<"-t "<<filename<<std::endl;
-	    processFile(filename, osgTerrain::DataSet::Source::IMAGE, currentCS, 
+            std::cout<<"-t "<<filename<<std::endl;
+            processFile(filename, osgTerrain::DataSet::Source::IMAGE, currentCS, 
                         geoTransform, geoTransformSet, geoTransformScale, 
                         minmaxLevelSet, min_level, max_level, 
                         currentLayerNum,
@@ -532,8 +578,8 @@ int main( int argc, char **argv )
 /*        
         else if (arguments.read(pos, "-m",filename))
         {
-	    std::cout<<"-m "<<filename<<std::endl;
-	    processFile(filename, osgTerrain::DataSet::Source::MODEL, currentCS, 
+            std::cout<<"-m "<<filename<<std::endl;
+            processFile(filename, osgTerrain::DataSet::Source::MODEL, currentCS, 
                         geoTransform, geoTransformSet, geoTransformScale, 
                         minmaxLevelSet, min_level, max_level, 
                         currentLayerNum, 
@@ -552,14 +598,7 @@ int main( int argc, char **argv )
         else if (arguments.read(pos, "-o",filename)) 
         {
             std::cout<<"-o "<<filename<<std::endl;
-
-            std::string path = osgDB::getFilePath(filename);
-            std::string base = path.empty()?osgDB::getStrippedName(filename):
-                                            path +'/'+ osgDB::getStrippedName(filename);
-            std::string extension = '.'+osgDB::getLowerCaseFileExtension(filename);
-
-            dataset->setDestinationTileBaseName(base);
-            dataset->setDestinationTileExtension(extension);
+            dataset->setDestinationName(filename);
             
             if (!currentCS.empty()) dataset->setDestinationCoordinateSystem(currentCS);
 
