@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This application is open source and may be redistributed and/or modified   
  * freely and without restriction, both in commericial and non commericial applications,
@@ -81,14 +81,14 @@ osg::Node* createTile(const std::string& filename, bool leftHemisphere, double x
     {
         texCoordRange = dynamic_cast<osgDB::ImageOptions::TexCoordRange*>(image->getUserData());
     
-	osg::Texture2D* texture = new osg::Texture2D;
-	texture->setImage(image);
+        osg::Texture2D* texture = new osg::Texture2D;
+        texture->setImage(image);
         texture->setWrap(osg::Texture::WRAP_S,osg::Texture::CLAMP_TO_EDGE);
         texture->setWrap(osg::Texture::WRAP_T,osg::Texture::CLAMP_TO_EDGE);
         texture->setFilter(osg::Texture::MIN_FILTER,osg::Texture::LINEAR);
         texture->setFilter(osg::Texture::MAG_FILTER,osg::Texture::LINEAR);
         texture->setMaxAnisotropy(8);
-	stateset->setTextureAttributeAndModes(0,texture,osg::StateAttribute::ON);
+        stateset->setTextureAttributeAndModes(0,texture,osg::StateAttribute::ON);
         
         if (useCompressedTextures)
         {
@@ -124,7 +124,7 @@ osg::Node* createTile(const std::string& filename, bool leftHemisphere, double x
     osg::Vec3Array& v = *(new osg::Vec3Array(numColumns*numRows));
     osg::Vec3Array& n = *(new osg::Vec3Array(numColumns*numRows));
     osg::Vec2Array& t = *(new osg::Vec2Array(numColumns*numRows));
-    osg::UByte4Array& color = *(new osg::UByte4Array(1));
+    osg::Vec4ubArray& color = *(new osg::Vec4ubArray(1));
 
     color[0].set(255,255,255,255);
 
@@ -163,8 +163,8 @@ osg::Node* createTile(const std::string& filename, bool leftHemisphere, double x
         double latitude = orig_latitude;
         
         tex.x() = tex_orig.x();
-	for(c=0;c<numColumns;++c)
-	{
+        for(c=0;c<numColumns;++c)
+        {
             double sin_longitude = sin(longitude);
             if (leftHemisphere)
             {
@@ -174,14 +174,14 @@ osg::Node* createTile(const std::string& filename, bool leftHemisphere, double x
             {
                 normal.set(-cos(latitude)*sin_longitude,-sin(latitude)*sin_longitude,-cos(longitude));
             }
-	    v[vi] = normal;
-	    n[vi] = normal;
+            v[vi] = normal;
+            n[vi] = normal;
             
-	    t[vi].set(tex.x(),tex.y());
+            t[vi].set(tex.x(),tex.y());
             latitude+=delta_latitude;
             tex.x()+=columnTexDelta;
             ++vi;
-	}
+        }
         longitude += delta_longitude;
         tex.y() += rowTexDelta;
     }
@@ -198,11 +198,11 @@ osg::Node* createTile(const std::string& filename, bool leftHemisphere, double x
         osg::DrawElementsUShort& drawElements = *(new osg::DrawElementsUShort(GL_QUAD_STRIP,2*numColumns));
         geometry->addPrimitiveSet(&drawElements);
         int ei=0;
-	for(c=0;c<numColumns;++c)
-	{
-	    drawElements[ei++] = (r+1)*numColumns+c;
-	    drawElements[ei++] = (r)*numColumns+c;
-	}
+        for(c=0;c<numColumns;++c)
+        {
+            drawElements[ei++] = (r+1)*numColumns+c;
+            drawElements[ei++] = (r)*numColumns+c;
+        }
     }
 
     osgUtil::TriStripVisitor tsv;
@@ -233,7 +233,7 @@ osg::Node* createTile(const std::string& filename, bool leftHemisphere, double x
         min_dot = osg::minimum(normal*n01,min_dot);
 
         float angle = acosf(min_dot)+osg::PI*0.5f;
-        float deviation = (angle<osg::PI) ? deviation = cosf(angle) : -1.0f;
+        float deviation = (angle<osg::PI) ? cosf(angle) : -1.0f;
 
         osg::ClusterCullingCallback* ccc = new osg::ClusterCullingCallback;
         ccc->setControlPoint(center);
@@ -367,7 +367,7 @@ int main( int argc, char **argv )
     
     // set up the usage document, in case we need to print out how to use this program.
     arguments.getApplicationUsage()->setApplicationName(arguments.getApplicationName());
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the standard OpenSceneGraph example which loads and visualises 3d models.");
+    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is an example which creates and paged database from the Nase blue marble hemisphere's.");
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
     arguments.getApplicationUsage()->addCommandLineOption("-e <filename>","Specify the east hemisphere input file to process");
     arguments.getApplicationUsage()->addCommandLineOption("-w <filename>","Specify the west hemisphere input file to process");
