@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -862,9 +862,9 @@ bool GameEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionA
                     if (_numberOfPlayers>=2) _players[1].moveRight();
                 }
 
-                static double previous_time = ea.time();
-                double dt = ea.time()-previous_time;
-                previous_time = ea.time();
+                static double previous_time = ea.getTime();
+                double dt = ea.getTime()-previous_time;
+                previous_time = ea.getTime();
 
                 // move objects
                 for(CatchableObjectList::iterator itr=_catchableObjects.begin();
@@ -921,7 +921,7 @@ bool GameEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionA
                     }
 
                     if (!(*itr)->anyInside(_origin, _origin+_width+_height) || 
-                        (*itr)->needToRemove(ea.time()) ||
+                        (*itr)->needToRemove(ea.getTime()) ||
                         removeEntry)
                     {
                         // need to remove
@@ -944,16 +944,16 @@ bool GameEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionA
                         // hit base line
                         (*itr)->explode();
                         (*itr)->stop();
-                        (*itr)->setTimeToRemove(ea.time()+3.0);
+                        (*itr)->setTimeToRemove(ea.getTime()+3.0);
                     }
 
                 }
 
 
                 // create new catchable objects
-                static double previousTime = ea.time();
-                double deltaTime = ea.time()-previousTime;
-                previousTime = ea.time();
+                static double previousTime = ea.getTime();
+                double deltaTime = ea.getTime()-previousTime;
+                previousTime = ea.getTime();
 
                 float numDropsPerSecond = _initialNumDropsPerSecond * (_level+1);
                 float r = (float)rand()/(float)RAND_MAX;
@@ -1359,14 +1359,19 @@ class CompileStateCallback : public osgProducer::OsgCameraGroup::RealizeCallback
         { 
             // now safe to construct
             sh.init();
-                        
-            if (_gameEventHandler)
+          
             {
-                _gameEventHandler->compileGLObjects(*(sh.getSceneView()->getState()));
+                OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+
+                if (_gameEventHandler)
+                {
+                    _gameEventHandler->compileGLObjects(*(sh.getSceneView()->getState()));
+                }
             }
         }
         
         
+        OpenThreads::Mutex  _mutex;
         GameEventHandler*  _gameEventHandler;
         
 };
@@ -1425,9 +1430,9 @@ int main( int argc, char **argv )
     // now the windows have been realized we switch off the cursor to prevent it
     // distracting the people seeing the stereo images.
     float fovy = 1.0f;
-    for( unsigned int i = 0; i < viewer.getCameraConfig()->getNumberOfCameras(); i++ )
+    for( unsigned int i = 0; i < viewer.getNumberOfCameras(); i++ )
     {
-        Producer::Camera* cam = viewer.getCameraConfig()->getCamera(i);
+        Producer::Camera* cam = viewer.getCamera(i);
         Producer::RenderSurface* rs = cam->getRenderSurface();
         rs->useCursor(false);
         fovy = osg::DegreesToRadians(cam->getLensVerticalFov());
