@@ -15,6 +15,7 @@
 #include <osg/Texture2D>
 
 #include <osgDB/ReadFile>
+#include <osgDB/FileUtils>
 
 #include <osgUtil/IntersectVisitor>
 #include <osgUtil/SmoothingVisitor>
@@ -43,7 +44,7 @@ public:
             _height(1.0f),
             _type(0) {}
 
-        Tree(const osg::Vec3& position, const osg::UByte4& color, float width, float height, unsigned int type):
+        Tree(const osg::Vec3& position, const osg::Vec4ub& color, float width, float height, unsigned int type):
             _position(position),
             _color(color),
             _width(width),
@@ -51,7 +52,7 @@ public:
             _type(type) {}
 
         osg::Vec3       _position;
-        osg::UByte4     _color;
+        osg::Vec4ub     _color;
         float           _width;
         float           _height;
         unsigned int    _type;
@@ -98,15 +99,18 @@ public:
 
     void createTreeList(osg::Node* terrain,const osg::Vec3& origin, const osg::Vec3& size,unsigned int numTreesToCreate,TreeList& trees);
 
-    osg::Geometry* createSprite( float w, float h, osg::UByte4 color );
+    osg::Geometry* createSprite( float w, float h, osg::Vec4ub color );
 
-    osg::Geometry* createOrthogonalQuads( const osg::Vec3& pos, float w, float h, osg::UByte4 color );
+    osg::Geometry* createOrthogonalQuads( const osg::Vec3& pos, float w, float h, osg::Vec4ub color );
+    osg::Geometry* createOrthogonalQuadsNoColor( const osg::Vec3& pos, float w, float h );
 
     osg::Node* createBillboardGraph(Cell* cell,osg::StateSet* stateset);
 
     osg::Node* createXGraph(Cell* cell,osg::StateSet* stateset);
 
     osg::Node* createTransformGraph(Cell* cell,osg::StateSet* stateset);
+
+    osg::Node* createShaderGraph(Cell* cell,osg::StateSet* stateset);
     
     osg::Node* createHUDWithText(const std::string& text);
 
@@ -353,9 +357,9 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
     osg::Image* image = osgDB::readImageFile("Images/lz.rgb");
     if (image)
     {
-	osg::Texture2D* texture = new osg::Texture2D;
-	texture->setImage(image);
-	stateset->setTextureAttributeAndModes(0,texture,osg::StateAttribute::ON);
+        osg::Texture2D* texture = new osg::Texture2D;
+        texture->setImage(image);
+        stateset->setTextureAttributeAndModes(0,texture,osg::StateAttribute::ON);
     }
     
     geode->setStateSet( stateset );
@@ -370,11 +374,11 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
     float max_z = -FLT_MAX;
     for(r=0;r<numRows;++r)
     {
-	for(c=0;c<numColumns;++c)
-	{
-	    min_z = osg::minimum(min_z,vertex[r+c*numRows][2]);
-	    max_z = osg::maximum(max_z,vertex[r+c*numRows][2]);
-	}
+        for(c=0;c<numColumns;++c)
+        {
+            min_z = osg::minimum(min_z,vertex[r+c*numRows][2]);
+            max_z = osg::maximum(max_z,vertex[r+c*numRows][2]);
+        }
     }
     
     float scale_z = size.z()/(max_z-min_z);
@@ -392,10 +396,10 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
 
         for(r=0;r<numRows;++r)
         {
-	    for(c=0;c<numColumns;++c)
-	    {
-	        grid->setHeight(c,r,(vertex[r+c*numRows][2]-min_z)*scale_z);
-	    }
+            for(c=0;c<numColumns;++c)
+            {
+                grid->setHeight(c,r,(vertex[r+c*numRows][2]-min_z)*scale_z);
+            }
         }
         
         geode->addDrawable(new osg::ShapeDrawable(grid));
@@ -406,7 +410,7 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
         
         osg::Vec3Array& v = *(new osg::Vec3Array(numColumns*numRows));
         osg::Vec2Array& t = *(new osg::Vec2Array(numColumns*numRows));
-        osg::UByte4Array& color = *(new osg::UByte4Array(1));
+        osg::Vec4ubArray& color = *(new osg::Vec4ubArray(1));
         
         color[0].set(255,255,255,255);
 
@@ -423,14 +427,14 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
         {
             pos.x() = origin.x();
             tex.x() = 0.0f;
-	    for(c=0;c<numColumns;++c)
-	    {
-	        v[vi].set(pos.x(),pos.y(),pos.z()+(vertex[r+c*numRows][2]-min_z)*scale_z);
-	        t[vi].set(tex.x(),tex.y());
+            for(c=0;c<numColumns;++c)
+            {
+                v[vi].set(pos.x(),pos.y(),pos.z()+(vertex[r+c*numRows][2]-min_z)*scale_z);
+                t[vi].set(tex.x(),tex.y());
                 pos.x()+=columnCoordDelta;
                 tex.x()+=columnTexDelta;
                 ++vi;
-	    }
+            }
             pos.y() += rowCoordDelta;
             tex.y() += rowTexDelta;
         }
@@ -445,11 +449,11 @@ osg::Geode* ForestTechniqueManager::createTerrain(const osg::Vec3& origin, const
             osg::DrawElementsUShort& drawElements = *(new osg::DrawElementsUShort(GL_QUAD_STRIP,2*numColumns));
             geometry->addPrimitiveSet(&drawElements);
             int ei=0;
-	    for(c=0;c<numColumns;++c)
-	    {
-	        drawElements[ei++] = (r+1)*numColumns+c;
-	        drawElements[ei++] = (r)*numColumns+c;
-	    }
+            for(c=0;c<numColumns;++c)
+            {
+                drawElements[ei++] = (r+1)*numColumns+c;
+                drawElements[ei++] = (r)*numColumns+c;
+            }
         }
         
         geode->addDrawable(geometry);
@@ -508,12 +512,12 @@ void ForestTechniqueManager::createTreeList(osg::Node* terrain,const osg::Vec3&
     }
 }
 
-osg::Geometry* ForestTechniqueManager::createSprite( float w, float h, osg::UByte4 color )
+osg::Geometry* ForestTechniqueManager::createSprite( float w, float h, osg::Vec4ub color )
 {
     // set up the coords
     osg::Vec3Array& v = *(new osg::Vec3Array(4));
     osg::Vec2Array& t = *(new osg::Vec2Array(4));
-    osg::UByte4Array& c = *(new osg::UByte4Array(1));
+    osg::Vec4ubArray& c = *(new osg::Vec4ubArray(1));
 
     v[0].set(-w*0.5f,0.0f,0.0f);
     v[1].set( w*0.5f,0.0f,0.0f);
@@ -541,12 +545,12 @@ osg::Geometry* ForestTechniqueManager::createSprite( float w, float h, osg::UByt
     return geom;
 }
 
-osg::Geometry* ForestTechniqueManager::createOrthogonalQuads( const osg::Vec3& pos, float w, float h, osg::UByte4 color )
+osg::Geometry* ForestTechniqueManager::createOrthogonalQuads( const osg::Vec3& pos, float w, float h, osg::Vec4ub color )
 {
     // set up the coords
     osg::Vec3Array& v = *(new osg::Vec3Array(8));
     osg::Vec2Array& t = *(new osg::Vec2Array(8));
-    osg::UByte4Array& c = *(new osg::UByte4Array(1));
+    osg::Vec4ubArray& c = *(new osg::Vec4ubArray(1));
     
     float rotation = random(0.0f,osg::PI/2.0f);
     float sw = sinf(rotation)*w*0.5f;
@@ -677,7 +681,7 @@ osg::Node* ForestTechniqueManager::createTransformGraph(Cell* cell,osg::StateSet
     {
         transform_group = new osg::Group;
         
-        osg::Geometry* geometry = createOrthogonalQuads(osg::Vec3(0.0f,0.0f,0.0f),1.0f,1.0f,osg::UByte4(255,255,255,255));
+        osg::Geometry* geometry = createOrthogonalQuads(osg::Vec3(0.0f,0.0f,0.0f),1.0f,1.0f,osg::Vec4ub(255,255,255,255));
         
         for(TreeList::iterator itr=cell->_trees.begin();
             itr!=cell->_trees.end();
@@ -712,6 +716,162 @@ osg::Node* ForestTechniqueManager::createTransformGraph(Cell* cell,osg::StateSet
     else return transform_group;
 }
 
+osg::Geometry* ForestTechniqueManager::createOrthogonalQuadsNoColor( const osg::Vec3& pos, float w, float h)
+{
+    // set up the coords
+    osg::Vec3Array& v = *(new osg::Vec3Array(8));
+    osg::Vec2Array& t = *(new osg::Vec2Array(8));
+    
+    float rotation = random(0.0f,osg::PI/2.0f);
+    float sw = sinf(rotation)*w*0.5f;
+    float cw = cosf(rotation)*w*0.5f;
+
+    v[0].set(pos.x()-sw,pos.y()-cw,pos.z()+0.0f);
+    v[1].set(pos.x()+sw,pos.y()+cw,pos.z()+0.0f);
+    v[2].set(pos.x()+sw,pos.y()+cw,pos.z()+h);
+    v[3].set(pos.x()-sw,pos.y()-cw,pos.z()+h);
+
+    v[4].set(pos.x()-cw,pos.y()+sw,pos.z()+0.0f);
+    v[5].set(pos.x()+cw,pos.y()-sw,pos.z()+0.0f);
+    v[6].set(pos.x()+cw,pos.y()-sw,pos.z()+h);
+    v[7].set(pos.x()-cw,pos.y()+sw,pos.z()+h);
+
+    t[0].set(0.0f,0.0f);
+    t[1].set(1.0f,0.0f);
+    t[2].set(1.0f,1.0f);
+    t[3].set(0.0f,1.0f);
+
+    t[4].set(0.0f,0.0f);
+    t[5].set(1.0f,0.0f);
+    t[6].set(1.0f,1.0f);
+    t[7].set(0.0f,1.0f);
+
+    osg::Geometry *geom = new osg::Geometry;
+
+    geom->setVertexArray( &v );
+
+    geom->setTexCoordArray( 0, &t );
+
+    geom->addPrimitiveSet( new osg::DrawArrays(osg::PrimitiveSet::QUADS,0,8) );
+
+    return geom;
+}
+
+class ShaderGeometry : public osg::Drawable
+{
+    public:
+        ShaderGeometry() { setUseDisplayList(false); }
+
+        /** Copy constructor using CopyOp to manage deep vs shallow copy.*/
+        ShaderGeometry(const ShaderGeometry& ShaderGeometry,const osg::CopyOp& copyop=osg::CopyOp::SHALLOW_COPY):
+            osg::Drawable(ShaderGeometry,copyop) {}
+
+        META_Object(osg,ShaderGeometry)
+
+        typedef std::vector<osg::Vec4> PositionSizeList;
+        
+        virtual void drawImplementation(osg::State& state) const
+        {
+            for(PositionSizeList::const_iterator itr = _trees.begin();
+                itr != _trees.end();
+                ++itr)
+            {
+                glColor4fv(itr->ptr());
+                _geometry->draw(state);
+            }
+        }
+
+        virtual osg::BoundingBox computeBound() const
+        {
+            osg::BoundingBox geom_box = _geometry->getBound();
+            osg::BoundingBox bb;
+            for(PositionSizeList::const_iterator itr = _trees.begin();
+                itr != _trees.end();
+                ++itr)
+            {
+                bb.expandBy(geom_box.corner(0)*(*itr)[3] +
+                            osg::Vec3( (*itr)[0], (*itr)[1], (*itr)[2] ));
+                bb.expandBy(geom_box.corner(7)*(*itr)[3] +
+                            osg::Vec3( (*itr)[0], (*itr)[1], (*itr)[2] ));
+            }
+            return bb;
+        }
+        
+        void setGeometry(osg::Geometry* geometry)
+        {
+            _geometry = geometry;
+        }
+        
+        void addTree(ForestTechniqueManager::Tree& tree)
+        {
+            _trees.push_back(osg::Vec4(tree._position.x(), tree._position.y(), tree._position.z(), tree._height));
+        }
+        
+        osg::ref_ptr<osg::Geometry> _geometry;
+
+        PositionSizeList _trees;
+
+    protected:
+    
+        virtual ~ShaderGeometry() {}
+        
+};
+
+osg::Geometry* shared_geometry = 0;
+
+osg::Node* ForestTechniqueManager::createShaderGraph(Cell* cell,osg::StateSet* stateset)
+{
+    if (shared_geometry==0)
+    {
+        shared_geometry = createOrthogonalQuadsNoColor(osg::Vec3(0.0f,0.0f,0.0f),1.0f,1.0f);
+        //shared_geometry->setUseDisplayList(false);
+    }
+
+
+    bool needGroup = !(cell->_cells.empty());
+    bool needTrees = !(cell->_trees.empty());
+    
+    osg::Geode* geode = 0;
+    osg::Group* group = 0;
+    
+    if (needTrees)
+    {
+        geode = new osg::Geode;
+        
+        ShaderGeometry* shader_geometry = new ShaderGeometry;
+        shader_geometry->setGeometry(shared_geometry);
+        
+        
+        for(TreeList::iterator itr=cell->_trees.begin();
+            itr!=cell->_trees.end();
+            ++itr)
+        {
+            Tree& tree = **itr;
+            shader_geometry->addTree(tree);
+
+        }
+
+        geode->setStateSet(stateset);
+        geode->addDrawable(shader_geometry);
+    }
+    
+    if (needGroup)
+    {
+        group = new osg::Group;
+        for(Cell::CellList::iterator itr=cell->_cells.begin();
+            itr!=cell->_cells.end();
+            ++itr)
+        {
+            group->addChild(createShaderGraph(itr->get(),stateset));
+        }
+        
+        if (geode) group->addChild(geode);
+        
+    }
+    if (group) return group;
+    else return geode;
+}
+
 osg::Node* ForestTechniqueManager::createHUDWithText(const std::string& str)
 {
     osg::Geode* geode = new osg::Geode();
@@ -799,12 +959,12 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
     
 
     _techniqueSwitch = new osg::Switch;
-    
+
     {
         std::cout<<"Creating billboard based forest...";
         osg::Group* group = new osg::Group;
         group->addChild(createBillboardGraph(cell.get(),dstate));
-        group->addChild(createHUDWithText("Using osg::Billboard's to create a forest\n\nPress left cursor key to select osg::MatrixTransform based forest\nPress right cursor key to select double quad based forest"));
+        group->addChild(createHUDWithText("Using osg::Billboard's to create a forest\n\nPress left cursor key to select OpenGL shader based forest\nPress right cursor key to select double quad based forest"));
         _techniqueSwitch->addChild(group);
         std::cout<<"done."<<std::endl;
     }
@@ -822,11 +982,77 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int numTreesToCreates)
         std::cout<<"Creating billboard based forest...";
         osg::Group* group = new osg::Group;
         group->addChild(createTransformGraph(cell.get(),dstate));
-        group->addChild(createHUDWithText("Using osg::MatrixTransform's to create a forest\n\nPress left cursor key to select double quad based forest\nPress right cursor key to select osg::Billboard based forest"));
+        group->addChild(createHUDWithText("Using osg::MatrixTransform's to create a forest\n\nPress left cursor key to select double quad based forest\nPress right cursor key to select OpenGL shder based forest"));
         _techniqueSwitch->addChild(group);
         std::cout<<"done."<<std::endl;
     }
-    
+
+    {
+        osg::Group* group = new osg::Group;
+        
+
+        osg::StateSet* stateset = new osg::StateSet;
+
+        stateset->setTextureAttributeAndModes(0, tex, osg::StateAttribute::ON );
+        stateset->setRenderingHint( osg::StateSet::TRANSPARENT_BIN );
+
+        {
+            osg::Program* program = new osg::Program;
+            stateset->setAttribute(program);
+
+#if 1
+            // use inline shaders
+            
+            ///////////////////////////////////////////////////////////////////
+            // vertex shader using just Vec4 coefficients
+            char vertexShaderSource[] = 
+                "varying vec2 texcoord;\n"
+                "\n"
+                "void main(void)\n"
+                "{\n"
+                "    vec3 position = gl_Vertex.xyz * gl_Color.w + gl_Color.xyz;\n"
+                "    gl_Position     = gl_ModelViewProjectionMatrix * vec4(position,1.0);\n"
+                "    gl_FrontColor = vec4(1.0,1.0,1.0,1.0);\n"
+                "    texcoord = gl_MultiTexCoord0.st;\n"
+                "}\n";
+
+            //////////////////////////////////////////////////////////////////
+            // fragment shader
+            //
+            char fragmentShaderSource[] = 
+                "uniform sampler2D baseTexture; \n"
+                "varying vec2 texcoord; \n"
+                "\n"
+                "void main(void) \n"
+                "{ \n"
+                "    gl_FragColor = texture2D( baseTexture, texcoord); \n"
+                "}\n";
+
+            osg::Shader* vertex_shader = new osg::Shader(osg::Shader::VERTEX, vertexShaderSource);
+            program->addShader(vertex_shader);
+
+            osg::Shader* fragment_shader = new osg::Shader(osg::Shader::FRAGMENT, fragmentShaderSource);
+            program->addShader(fragment_shader);
+            
+#else
+
+            // get shaders from source
+            program->addShader(osg::Shader::readShaderFile(osg::Shader::VERTEX, osgDB::findDataFile("shaders/forest.vert")));
+            program->addShader(osg::Shader::readShaderFile(osg::Shader::FRAGMENT, osgDB::findDataFile("shaders/forest.frag")));
+
+#endif
+
+            osg::Uniform* baseTextureSampler = new osg::Uniform("baseTexture",0);
+            stateset->addUniform(baseTextureSampler);
+        }
+
+        std::cout<<"Creating billboard based forest...";
+        group->addChild(createShaderGraph(cell.get(),stateset));
+        group->addChild(createHUDWithText("Using OpenGL Shader to create a forest\n\nPress left cursor key to select osg::MatrixTransform based forest\nPress right cursor key to select osg::Billboard based forest"));
+        _techniqueSwitch->addChild(group);
+        std::cout<<"done."<<std::endl;
+    }
+
     _currentTechnique = 0;
     _techniqueSwitch->setSingleChildOn(_currentTechnique);
     
