@@ -227,7 +227,7 @@ bool GliderManipulator::calcMovement()
     if (_ga_t0.get()==NULL || _ga_t1.get()==NULL) return false;
 
 
-    double dt = _ga_t0->time()-_ga_t1->time();
+    double dt = _ga_t0->getTime()-_ga_t1->getTime();
 
     if (dt<0.0f)
     {
@@ -260,6 +260,12 @@ bool GliderManipulator::calcMovement()
     float dx = _ga_t0->getXnormalized();
     float dy = _ga_t0->getYnormalized();
 
+    // mew - flag to reverse mouse-control mapping
+    if( getenv( "OSGHANGGLIDE_REVERSE_CONTROLS" ) )
+    {
+        dx = -dx;
+        dy = -dy;
+    }
 
     osg::Matrixd rotation_matrix;
     rotation_matrix.makeRotate(_rotation);
