@@ -6,6 +6,7 @@
 #include <osg/Texture2D>
 #include <osg/PositionAttitudeTransform>
 #include <osg/MatrixTransform>
+#include <osg/io_utils>
 
 #include <osgUtil/Optimizer>
 
@@ -16,6 +17,7 @@
 #include <osgParticle/ExplosionEffect>
 #include <osgParticle/ExplosionDebrisEffect>
 #include <osgParticle/SmokeEffect>
+#include <osgParticle/SmokeTrailEffect>
 #include <osgParticle/FireEffect>
 
 // for the grid data..
@@ -62,7 +64,7 @@ osg::Node* createMovingModel(const osg::Vec3& center, float radius)
     if (glider)
     {
         const osg::BoundingSphere& bs = glider->getBound();
-        float size = radius/bs.radius()*0.3f;
+        float size = radius/bs.radius()*0.15f;
 
         osg::MatrixTransform* positioned = new osg::MatrixTransform;
         positioned->setDataVariance(osg::Object::STATIC);
@@ -84,7 +86,7 @@ osg::Node* createMovingModel(const osg::Vec3& center, float radius)
     if (cessna)
     {
         const osg::BoundingSphere& bs = cessna->getBound();
-        float size = radius/bs.radius()*0.3f;
+        float size = radius/bs.radius()*0.15f;
 
         osg::MatrixTransform* positioned = new osg::MatrixTransform;
         positioned->getOrCreateStateSet()->setMode(GL_NORMALIZE, osg::StateAttribute::ON);
@@ -149,9 +151,9 @@ void build_world(osg::Group *root)
         osg::Image* image = osgDB::readImageFile("Images/lz.rgb");
         if (image)
         {
-	    osg::Texture2D* texture = new osg::Texture2D;
-	    texture->setImage(image);
-	    stateset->setTextureAttributeAndModes(0,texture,osg::StateAttribute::ON);
+            osg::Texture2D* texture = new osg::Texture2D;
+            texture->setImage(image);
+            stateset->setTextureAttributeAndModes(0,texture,osg::StateAttribute::ON);
         }
 
         terrainGeode->setStateSet( stateset );
@@ -167,10 +169,10 @@ void build_world(osg::Group *root)
 
         for(unsigned int r=0;r<39;++r)
         {
-	    for(unsigned int c=0;c<38;++c)
-	    {
-	        grid->setHeight(c,r,z_scale*vertex[r+c*39][2]);
-	    }
+            for(unsigned int c=0;c<38;++c)
+            {
+                grid->setHeight(c,r,z_scale*vertex[r+c*39][2]);
+            }
         }
         terrainGeode->addDrawable(new osg::ShapeDrawable(grid));
         
@@ -220,7 +222,7 @@ void build_world(osg::Group *root)
 
     // create the moving models.
     {
-        root->addChild(createMovingModel(osg::Vec3(500.0f,500.0f,500.0f),100.0f));
+        root->addChild(createMovingModel(osg::Vec3(500.0f,500.0f,500.0f),300.0f));
     }
 }
 
@@ -271,13 +273,17 @@ public:
             }
             
             osg::Vec3 position = handleMovingModels ? hit.getLocalIntersectPoint() : hit.getWorldIntersectPoint();
-            float scale = 20.0f * ((float)rand() / (float)RAND_MAX);
-            float intensity = handleMovingModels ? 5.0f : 1.0f;
+            float scale = 10.0f * ((float)rand() / (float)RAND_MAX);
+            float intensity = 1.0f;
 
             osgParticle::ExplosionEffect* explosion = new osgParticle::ExplosionEffect(position, scale, intensity);
             osgParticle::ExplosionDebrisEffect* explosionDebri = new osgParticle::ExplosionDebrisEffect(position, scale, intensity);
-            osgParticle::SmokeEffect* smoke = new osgParticle::SmokeEffect(position, scale, intensity);
             osgParticle::FireEffect* fire = new osgParticle::FireEffect(position, scale, intensity);
+            osgParticle::ParticleEffect* smoke = 0;
+            if (handleMovingModels)
+                smoke =  new osgParticle::SmokeTrailEffect(position, scale, intensity);
+            else
+                smoke =  new osgParticle::SmokeEffect(position, scale, intensity);
             
             explosion->setWind(wind);
             explosionDebri->setWind(wind);
@@ -377,11 +383,46 @@ protected:
     virtual ~PickHandler() {}
 };
 
+// function used in debugging
+void insertParticle(osg::Group* root, const osg::Vec3& center, float radius)
+{
+    bool handleMovingModels = false;
+
+    osg::Vec3 position = center + 
+               osg::Vec3( radius * (((float)rand() / (float)RAND_MAX)-0.5)*2.0,
+                          radius * (((float)rand() / (float)RAND_MAX)-0.5)*2.0,
+                          0.0f);
+
+    float scale = 10.0f * ((float)rand() / (float)RAND_MAX);
+    float intensity = 1.0f;
+
+    osgParticle::ExplosionEffect* explosion = new osgParticle::ExplosionEffect(position, scale, intensity);
+    osgParticle::ExplosionDebrisEffect* explosionDebri = new osgParticle::ExplosionDebrisEffect(position, scale, intensity);
+    osgParticle::FireEffect* fire = new osgParticle::FireEffect(position, scale, intensity);
+    osgParticle::ParticleEffect* smoke = 0;
+    if (handleMovingModels)
+        smoke =  new osgParticle::SmokeTrailEffect(position, scale, intensity);
+    else
+        smoke =  new osgParticle::SmokeEffect(position, scale, intensity);
+
+    explosion->setWind(wind);
+    explosionDebri->setWind(wind);
+    smoke->setWind(wind);
+    fire->setWind(wind);
+
+    osg::Group* effectsGroup = new osg::Group;
+    effectsGroup->addChild(explosion);
+    effectsGroup->addChild(explosionDebri);
+    effectsGroup->addChild(smoke);
+    effectsGroup->addChild(fire);
+
+    root->addChild(effectsGroup);
+}
+
 //////////////////////////////////////////////////////////////////////////////
 // main()
 //////////////////////////////////////////////////////////////////////////////
 
-
 int main(int argc, char **argv)
 {
     // use an ArgumentParser object to manage the program arguments.
@@ -435,11 +476,16 @@ int main(int argc, char **argv)
     // create the windows and run the threads.
     viewer.realize();
 
+    // osg::Vec3 center = root->getBound().center();
+    // float radius = root->getBound().radius();
+
     while( !viewer.done() )
     {
         // wait for all cull and draw threads to complete.
         viewer.sync();
 
+        // insertParticle(root, center, radius);
+
         // update the scene by traversing it with the the update visitor which will
         // call all node update callbacks and animations.
         viewer.update();
