@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
  * This application is open source and may be redistributed and/or modified   
  * freely and without restriction, both in commericial and non commericial applications,
@@ -18,8 +18,8 @@
 #include <osg/MatrixTransform>
 
 #include <osgUtil/SmoothingVisitor>
-#include <osgUtil/TransformCallback>
 
+#include <Producer/CameraConfig>
 
 class PBufferTexture2D : public osg::Texture2D
 {
@@ -53,7 +53,13 @@ class PBufferTexture2D : public osg::Texture2D
             else
             {
                 textureObject->bind();
+                
+                static unsigned int frameNum = 0;
+                
                 _pbuffer->bindPBufferToTexture( Producer::RenderSurface::FrontBuffer );
+                
+                ++frameNum;
+                
             }
         }
 
@@ -233,7 +239,7 @@ osg::ref_ptr<osg::Node> buildSceneGraphAndSetCameraViews(osg::Node *loadedModel,
     osg::ref_ptr<osg::MatrixTransform> loadedModelTransform = new osg::MatrixTransform;
     loadedModelTransform->addChild(loadedModel);
 
-    osg::ref_ptr<osg::NodeCallback> nc = new osgUtil::TransformCallback(
+    osg::ref_ptr<osg::NodeCallback> nc = new osg::AnimationPathCallback(
             loadedModelTransform->getBound().center(),osg::Vec3(0.0f,0.0f,1.0f),osg::inDegrees(45.0f));
     loadedModelTransform->setUpdateCallback(nc.get());
 
@@ -263,7 +269,7 @@ osg::ref_ptr<osg::Node> buildSceneGraph(osg::Node *loadedModel, Producer::Render
     osg::ref_ptr<osg::MatrixTransform> loadedModelTransform = new osg::MatrixTransform;
     loadedModelTransform->addChild(loadedModel);
 
-    osg::ref_ptr<osg::NodeCallback> nc = new osgUtil::TransformCallback(
+    osg::ref_ptr<osg::NodeCallback> nc = new osg::AnimationPathCallback(
             loadedModelTransform->getBound().center(),osg::Vec3(0.0f,0.0f,1.0f),osg::inDegrees(45.0f));
     loadedModelTransform->setUpdateCallback(nc.get());
 
@@ -332,14 +338,14 @@ int main( int argc, char **argv )
     
     // set up the usage document, in case we need to print out how to use this program.
     arguments.getApplicationUsage()->setApplicationName(arguments.getApplicationName());
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the standard OpenSceneGraph example which loads and visualises 3d models.");
+    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" example shows how to set up a Producer pbuffer for render to texture effects.");
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
     arguments.getApplicationUsage()->addCommandLineOption("--image <filename>","Load an image and render it on a quad");
     arguments.getApplicationUsage()->addCommandLineOption("--dem <filename>","Load an image/DEM and render it on a HeightField");
-    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display command line paramters");
-    arguments.getApplicationUsage()->addCommandLineOption("--help-env","Display environmental variables available");
+    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display command line parameters");
+    arguments.getApplicationUsage()->addCommandLineOption("--help-env","Display environment variables available");
     arguments.getApplicationUsage()->addCommandLineOption("--help-keys","Display keyboard & mouse bindings available");
-    arguments.getApplicationUsage()->addCommandLineOption("--help-all","Display all command line, env vars and keyboard & mouse bindigs.");
+    arguments.getApplicationUsage()->addCommandLineOption("--help-all","Display all command line, env vars and keyboard & mouse bindings");
 
     /// Load models from command line and build scene graph
     osg::Timer_t start_tick = osg::Timer::instance()->tick();
@@ -437,7 +443,7 @@ int main( int argc, char **argv )
         }
     }
 
-printf( "PBuffer window: 0x%x\n", pbuffer->getWindow() );
+    // printf( "PBuffer window: 0x%x\n", pbuffer->getWindow() );
 
     viewer.getCamera(0)->setClearColor( 0.1f,0.9f,0.3f,1.0f );
 
