@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This application is open source and may be redistributed and/or modified under  
  * the terms of the GNU Public License (GPL) version 1.0 or 
@@ -612,17 +612,17 @@ bool SlideEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIAction
             if (ea.getKey()=='a')
             {
                 _autoSteppingActive = !_autoSteppingActive;
-                _previousTime = ea.time();
+                _previousTime = ea.getTime();
                 return true;
             }
             else if (ea.getKey()=='n')
             {
-                _album->nextPage(ea.time()+1.0f);
+                _album->nextPage(ea.getTime()+1.0f);
                 return true;
             }
             else if (ea.getKey()=='p')
             {
-                _album->previousPage(ea.time()+1.0f);
+                _album->previousPage(ea.getTime()+1.0f);
                 return true;
             }
             return false;
@@ -634,13 +634,13 @@ bool SlideEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIAction
                 if (_firstTraversal)
                 {
                     _firstTraversal = false;
-                    _previousTime = ea.time();
+                    _previousTime = ea.getTime();
                 }
-                else if (ea.time()-_previousTime>_timePerSlide)
+                else if (ea.getTime()-_previousTime>_timePerSlide)
                 {
-                    _previousTime = ea.time();
+                    _previousTime = ea.getTime();
 
-                    _album->nextPage(ea.time()+1.0f);
+                    _album->nextPage(ea.getTime()+1.0f);
                 }
             }
             
@@ -744,9 +744,9 @@ int main( int argc, char **argv )
     // distracting the people seeing the stereo images.
     float fovx = 1.25f;
     float fovy = 1.0f;
-    for( unsigned int i = 0; i < viewer.getCameraConfig()->getNumberOfCameras(); i++ )
+    for( unsigned int i = 0; i < viewer.getNumberOfCameras(); i++ )
     {
-        Producer::Camera* cam = viewer.getCameraConfig()->getCamera(i);
+        Producer::Camera* cam = viewer.getCamera(i);
         //Producer::RenderSurface* rs = cam->getRenderSurface();
         //rs->useCursor(false);
         fovx = cam->getLensHorizontalFov();
