@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This application is open source and may be redistributed and/or modified   
  * freely and without restriction, both in commericial and non commericial applications,
@@ -20,8 +20,7 @@
 #include <osg/ColorMask>
 #include <osg/Depth>
 #include <osg/ClipNode>
-
-#include <osgUtil/TransformCallback>
+#include <osg/AnimationPath>
 
 #include <osgDB/ReadFile>
 #include <osgUtil/Optimizer>
@@ -150,7 +149,7 @@ osg::Node* createMirroredScene(osg::Node* model)
         // set up the stencil ops so that the stencil buffer get set at
         // the mirror plane 
         osg::Stencil* stencil = new osg::Stencil;
-        stencil->setFunction(osg::Stencil::ALWAYS,1,~0);
+        stencil->setFunction(osg::Stencil::ALWAYS,1,~0u);
         stencil->setOperation(osg::Stencil::KEEP, osg::Stencil::KEEP, osg::Stencil::REPLACE);
         
         // switch off the writing to the color bit planes.
@@ -177,7 +176,7 @@ osg::Node* createMirroredScene(osg::Node* model)
     // occludes the mirror. 
     {        
         osg::Stencil* stencil = new osg::Stencil;
-        stencil->setFunction(osg::Stencil::ALWAYS,0,~0);
+        stencil->setFunction(osg::Stencil::ALWAYS,0,~0u);
         stencil->setOperation(osg::Stencil::KEEP, osg::Stencil::KEEP, osg::Stencil::REPLACE);
 
         osg::StateSet* statesetBin2 = new osg::StateSet();        
@@ -197,7 +196,7 @@ osg::Node* createMirroredScene(osg::Node* model)
     
         // set up the stencil ops so that only operator on this mirrors stencil value.
         osg::Stencil* stencil = new osg::Stencil;
-        stencil->setFunction(osg::Stencil::EQUAL,1,~0);
+        stencil->setFunction(osg::Stencil::EQUAL,1,~0u);
         stencil->setOperation(osg::Stencil::KEEP, osg::Stencil::KEEP, osg::Stencil::KEEP);
         
         // switch off the writing to the color bit planes.
@@ -255,7 +254,7 @@ osg::Node* createMirroredScene(osg::Node* model)
         dstate->setMode(GL_CULL_FACE,osg::StateAttribute::OVERRIDE|osg::StateAttribute::OFF);
 
         osg::Stencil* stencil = new osg::Stencil;
-        stencil->setFunction(osg::Stencil::EQUAL,1,~0);
+        stencil->setFunction(osg::Stencil::EQUAL,1,~0u);
         stencil->setOperation(osg::Stencil::KEEP, osg::Stencil::KEEP, osg::Stencil::KEEP);
         dstate->setAttributeAndModes(stencil,osg::StateAttribute::ON);
 
@@ -282,7 +281,7 @@ osg::Node* createMirroredScene(osg::Node* model)
         depth->setFunction(osg::Depth::ALWAYS);
 
         osg::Stencil* stencil = new osg::Stencil;
-        stencil->setFunction(osg::Stencil::EQUAL,1,~0);
+        stencil->setFunction(osg::Stencil::EQUAL,1,~0u);
         stencil->setOperation(osg::Stencil::KEEP, osg::Stencil::KEEP, osg::Stencil::ZERO);
 
         // set up additive blending.
@@ -382,7 +381,7 @@ int main( int argc, char **argv )
     osg::ref_ptr<osg::MatrixTransform> loadedModelTransform = new osg::MatrixTransform;
     loadedModelTransform->addChild(loadedModel.get());
 
-    osg::ref_ptr<osg::NodeCallback> nc = new osgUtil::TransformCallback(loadedModelTransform->getBound().center(),osg::Vec3(0.0f,0.0f,1.0f),osg::inDegrees(45.0f));
+    osg::ref_ptr<osg::NodeCallback> nc = new osg::AnimationPathCallback(loadedModelTransform->getBound().center(),osg::Vec3(0.0f,0.0f,1.0f),osg::inDegrees(45.0f));
     loadedModelTransform->setUpdateCallback(nc.get());
 
 
