@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -118,7 +118,7 @@ bool SlideEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIAction
             if (ea.getKey()=='a')
             {
                 _autoSteppingActive = !_autoSteppingActive;
-                _previousTime = ea.time();
+                _previousTime = ea.getTime();
                 return true;
             }
             else if (ea.getKey()=='n')
@@ -475,9 +475,9 @@ int main( int argc, char **argv )
     // now the windows have been realized we switch off the cursor to prevent it
     // distracting the people seeing the stereo images.
     float fovy = 1.0f;
-    for( unsigned int i = 0; i < viewer.getCameraConfig()->getNumberOfCameras(); i++ )
+    for( unsigned int i = 0; i < viewer.getNumberOfCameras(); i++ )
     {
-        Producer::Camera* cam = viewer.getCameraConfig()->getCamera(i);
+        Producer::Camera* cam = viewer.getCamera(i);
         Producer::RenderSurface* rs = cam->getRenderSurface();
         rs->useCursor(false);
         fovy = osg::DegreesToRadians(cam->getLensVerticalFov());
