@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -20,8 +20,7 @@
 #include <osgProducer/Viewer>
 
 #include <osg/Geode>
-#include <osg/Projection>
-#include <osg/MatrixTransform>
+#include <osg/CameraNode>
 #include <osg/ShapeDrawable>
 #include <osg/Sequence>
 
@@ -76,6 +75,7 @@ osg::Group* createHUDText()
 
         // right to left layouts would be used for hebrew or arabic fonts.
         text->setLayout(osgText::Text::RIGHT_TO_LEFT);
+        text->setAlignment(osgText::Text::RIGHT_BASE_LINE);
 
         text->setText("text->setLayout(osgText::Text::RIGHT_TO_LEFT);");
         geode->addDrawable(text);
@@ -170,7 +170,7 @@ osg::Group* createHUDText()
         // use text that is 20 units high.
         text->setCharacterSize(20); // small
         
-        text->setText("text->setCharacterSize(15.0f); // small");
+        text->setText("text->setCharacterSize(20.0f); // small");
         geode->addDrawable(text);
     }
     
@@ -218,18 +218,22 @@ osg::Group* createHUDText()
     typedef std::pair<osgText::Text::AlignmentType,std::string> AlignmentPair;
     typedef std::vector<AlignmentPair> AlignmentList;
     AlignmentList alignmentList;
-    alignmentList.push_back(AlignmentPair(osgText::Text::LEFT_TOP,"text->setAlignment(osgText::Text::LEFT_TOP);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::LEFT_CENTER,"text->setAlignment(osgText::Text::LEFT_CENTER);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::LEFT_BOTTOM,"text->setAlignment(osgText::Text::LEFT_BOTTOM);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::CENTER_TOP,"text->setAlignment(osgText::Text::CENTER_TOP);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::CENTER_CENTER,"text->setAlignment(osgText::Text::CENTER_CENTER);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::CENTER_BOTTOM,"text->setAlignment(osgText::Text::CENTER_BOTTOM);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::RIGHT_TOP,"text->setAlignment(osgText::Text::RIGHT_TOP);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::RIGHT_CENTER,"text->setAlignment(osgText::Text::RIGHT_CENTER);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::RIGHT_BOTTOM,"text->setAlignment(osgText::Text::RIGHT_BOTTOM);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::LEFT_BASE_LINE,"text->setAlignment(osgText::Text::BASE_LINE);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::CENTER_BASE_LINE,"text->setAlignment(osgText::Text::CENTER_BASE_LINE);"));
-    alignmentList.push_back(AlignmentPair(osgText::Text::RIGHT_BASE_LINE,"text->setAlignment(osgText::Text::RIGHT_BASE_LINE);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::LEFT_TOP,"text->setAlignment(\nosgText::Text::LEFT_TOP);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::LEFT_CENTER,"text->setAlignment(\nosgText::Text::LEFT_CENTER);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::LEFT_BOTTOM,"text->setAlignment(\nosgText::Text::LEFT_BOTTOM);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::CENTER_TOP,"text->setAlignment(\nosgText::Text::CENTER_TOP);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::CENTER_CENTER,"text->setAlignment(\nosgText::Text::CENTER_CENTER);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::CENTER_BOTTOM,"text->setAlignment(\nosgText::Text::CENTER_BOTTOM);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::RIGHT_TOP,"text->setAlignment(\nosgText::Text::RIGHT_TOP);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::RIGHT_CENTER,"text->setAlignment(\nosgText::Text::RIGHT_CENTER);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::RIGHT_BOTTOM,"text->setAlignment(\nosgText::Text::RIGHT_BOTTOM);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::LEFT_BASE_LINE,"text->setAlignment(\nosgText::Text::LEFT_BASE_LINE);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::CENTER_BASE_LINE,"text->setAlignment(\nosgText::Text::CENTER_BASE_LINE);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::RIGHT_BASE_LINE,"text->setAlignment(\nosgText::Text::RIGHT_BASE_LINE);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::LEFT_BOTTOM_BASE_LINE,"text->setAlignment(\nosgText::Text::LEFT_BOTTOM_BASE_LINE);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::CENTER_BOTTOM_BASE_LINE,"text->setAlignment(\nosgText::Text::CENTER_BOTTOM_BASE_LINE);"));
+    alignmentList.push_back(AlignmentPair(osgText::Text::RIGHT_BOTTOM_BASE_LINE,"text->setAlignment(\nosgText::Text::RIGHT_BOTTOM_BASE_LINE);"));
+
 
     osg::Sequence* sequence = new osg::Sequence;
     {
@@ -497,18 +501,15 @@ int main( int argc, char **argv )
 
         {
             // create the hud.
-            osg::Projection* projection = new osg::Projection;
-            projection->setMatrix(osg::Matrix::ortho2D(0,1280,0,1024));
+            osg::CameraNode* camera = new osg::CameraNode;
+            camera->setReferenceFrame(osg::Transform::ABSOLUTE_RF);
+            camera->setProjectionMatrixAsOrtho2D(0,1280,0,1024);
+            camera->setViewMatrix(osg::Matrix::identity());
+            camera->setClearMask(GL_DEPTH_BUFFER_BIT);
+            camera->addChild(createHUDText());
+            camera->getOrCreateStateSet()->setMode(GL_LIGHTING,osg::StateAttribute::OFF);
 
-            osg::MatrixTransform* modelview_abs = new osg::MatrixTransform;
-            modelview_abs->setReferenceFrame(osg::Transform::ABSOLUTE_RF);
-            modelview_abs->setMatrix(osg::Matrix::identity());
-
-            modelview_abs->addChild(createHUDText());
-
-            projection->addChild(modelview_abs);
-
-            group->addChild(projection);
+            group->addChild(camera);
         }
 
         group->addChild(create3DText(center,radius));
@@ -558,3 +559,4 @@ int main( int argc, char **argv )
 
     return 0;
 }
+
