@@ -12,13 +12,17 @@
 #include <osg/TexEnv>
 #include <osg/TexEnvCombine>
 #include <osg/Material>
+#include <osg/PrimitiveSet>
 #include <osg/Endian>
 
 #include <osgDB/Registry>
 #include <osgDB/ReadFile>
 #include <osgDB/WriteFile>
+#include <osgDB/FileUtils>
 #include <osgDB/FileNameUtils>
 
+#include <osgGA/EventVisitor>
+
 #include <osgUtil/CullVisitor>
 
 #include <osgProducer/Viewer>
@@ -145,6 +149,8 @@ struct PassThroughTransformFunction
 
 struct ProcessRow
 {
+    virtual ~ProcessRow() {}
+
     virtual void operator() (unsigned int num,
                     GLenum source_pixelFormat, unsigned char* source, 
                     GLenum dest_pixelFormat, unsigned char* dest) const 
@@ -486,8 +492,8 @@ osg::Image* createTexture3D(ImageList& imageList, ProcessRow& processRow,
             int num_t = osg::minimum(image->t(), image_3d->t());
             int num_s = osg::minimum(image->s(), image_3d->s());
         
-	    unsigned int s_offset_dest = (image->s()<s_nearestPowerOfTwo) ? s_nearestPowerOfTwo/2 - image->s()/2 : 0;
-	    unsigned int t_offset_dest = (image->t()<t_nearestPowerOfTwo) ? t_nearestPowerOfTwo/2 - image->t()/2 : 0;
+            unsigned int s_offset_dest = (image->s()<s_nearestPowerOfTwo) ? s_nearestPowerOfTwo/2 - image->s()/2 : 0;
+            unsigned int t_offset_dest = (image->t()<t_nearestPowerOfTwo) ? t_nearestPowerOfTwo/2 - image->t()/2 : 0;
 
             for(int r=0;r<num_r;++r, ++curr_dest_r)
             {
@@ -603,7 +609,7 @@ osg::Node* createCube(float size,float alpha, unsigned int numSlices, float slic
 
     float halfSize = size*0.5f;
     float y = halfSize;
-    float dy =-size/(float)(numSlices-1)*sliceEnd;
+    float dy =-size*1.4/(float)(numSlices-1)*sliceEnd;
 
     //y = -halfSize;
     //dy *= 0.5;
@@ -638,6 +644,285 @@ osg::Node* createCube(float size,float alpha, unsigned int numSlices, float slic
     return billboard;
 }
 
+class FollowMouseCallback : public osgGA::GUIEventHandler, public osg::StateSet::Callback
+{
+    public:
+    
+        FollowMouseCallback()
+        {
+            _updateTransparency = false;
+            _updateAlphaCutOff = false;
+            _updateSampleDensity = false;
+        }
+
+        FollowMouseCallback(const FollowMouseCallback&,const osg::CopyOp&) {}
+
+        META_Object(osg,FollowMouseCallback);
+
+        virtual void operator() (osg::StateSet* stateset, osg::NodeVisitor* nv)
+        {
+            if (nv->getVisitorType()==osg::NodeVisitor::EVENT_VISITOR)
+            {
+                osgGA::EventVisitor* ev = dynamic_cast<osgGA::EventVisitor*>(nv);
+                if (ev)
+                {
+                    osgGA::GUIActionAdapter* aa = ev->getActionAdapter();
+                    osgGA::EventQueue::Events& events = ev->getEvents();
+                    for(osgGA::EventQueue::Events::iterator itr=events.begin();
+                        itr!=events.end();
+                        ++itr)
+                    {
+                        handle(*(*itr), *aa, stateset, ev);
+                    }
+                }
+            }
+        }
+        
+        virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&, osg::Object* object, osg::NodeVisitor*)
+        {
+            osg::StateSet* stateset = dynamic_cast<osg::StateSet*>(object);
+            if (!stateset) return false;
+            
+            switch(ea.getEventType())
+            {
+                case(osgGA::GUIEventAdapter::MOVE):
+                case(osgGA::GUIEventAdapter::DRAG):
+                {
+                    float v = ea.getY()*0.5f+0.5f;
+                    osg::Uniform* uniform = 0;
+                    if (_updateTransparency && (uniform = stateset->getUniform("transparency"))) uniform->set(v);
+                    if (_updateAlphaCutOff && (uniform = stateset->getUniform("alphaCutOff"))) uniform->set(v);
+                    if (_updateSampleDensity && (uniform = stateset->getUniform("sampleDensity"))) uniform->set(powf(v,5));
+                    break;
+                }
+                case(osgGA::GUIEventAdapter::KEYDOWN):
+                {
+                    if (ea.getKey()=='t') _updateTransparency = true;
+                    if (ea.getKey()=='a') _updateAlphaCutOff = true;
+                    if (ea.getKey()=='d') _updateSampleDensity = true;
+                    break;
+                }
+                case(osgGA::GUIEventAdapter::KEYUP):
+                {
+                    if (ea.getKey()=='t') _updateTransparency = false;
+                    if (ea.getKey()=='a') _updateAlphaCutOff = false;
+                    if (ea.getKey()=='d') _updateSampleDensity = false;
+                    break;
+                }
+                default:
+                    break;
+            }
+            return false;
+        }
+                
+        virtual void accept(osgGA::GUIEventHandlerVisitor& v)
+        {
+            v.visit(*this);
+        }
+        
+        bool _updateTransparency;
+        bool _updateAlphaCutOff;
+        bool _updateSampleDensity;
+
+};
+
+osg::Node* createShaderModel(osg::ref_ptr<osg::Image>& image_3d, osg::ref_ptr<osg::Image>& /*normalmap_3d*/,
+                       osg::Texture::InternalFormatMode internalFormatMode,
+                       float /*xSize*/, float /*ySize*/, float /*zSize*/,
+                       float /*xMultiplier*/, float /*yMultiplier*/, float /*zMultiplier*/,
+                       unsigned int /*numSlices*/=500, float /*sliceEnd*/=1.0f, float alphaFuncValue=0.02f)
+{
+    osg::Geode* geode = new osg::Geode;
+    osg::StateSet* stateset = geode->getOrCreateStateSet();
+    
+    stateset->setEventCallback(new FollowMouseCallback);
+
+    // set up the 3d texture itself,
+    // note, well set the filtering up so that mip mapping is disabled,
+    // gluBuild3DMipsmaps doesn't do a very good job of handled the
+    // inbalanced dimensions of the 256x256x4 texture.
+    osg::Texture3D* texture3D = new osg::Texture3D;
+    texture3D->setFilter(osg::Texture3D::MIN_FILTER,osg::Texture3D::LINEAR);
+    texture3D->setFilter(osg::Texture3D::MAG_FILTER,osg::Texture3D::LINEAR);
+    texture3D->setWrap(osg::Texture3D::WRAP_R,osg::Texture3D::CLAMP);
+    texture3D->setWrap(osg::Texture3D::WRAP_S,osg::Texture3D::CLAMP);
+    texture3D->setWrap(osg::Texture3D::WRAP_T,osg::Texture3D::CLAMP);
+    if (image_3d->getPixelFormat()==GL_ALPHA || 
+        image_3d->getPixelFormat()==GL_LUMINANCE)
+    {
+        texture3D->setInternalFormatMode(osg::Texture3D::USE_USER_DEFINED_FORMAT);
+        texture3D->setInternalFormat(GL_INTENSITY);
+    }
+    else
+    {
+        texture3D->setInternalFormatMode(internalFormatMode);
+    }
+
+    texture3D->setImage(image_3d.get());
+
+    stateset->setTextureAttributeAndModes(0,texture3D,osg::StateAttribute::ON);
+
+    osg::Program* program = new osg::Program;
+    stateset->setAttribute(program);
+
+    // get shaders from source
+    std::string vertexShaderFile = osgDB::findDataFile("volume.vert");
+    if (!vertexShaderFile.empty())
+    {
+        program->addShader(osg::Shader::readShaderFile(osg::Shader::VERTEX, vertexShaderFile));
+    }
+    else
+    {
+        char vertexShaderSource[] = 
+            "varying vec3 texcoord;\n"
+            "varying vec3 deltaTexCoord;\n"
+            "\n"
+            "void main(void)\n"
+            "{\n"
+            "    texcoord = gl_MultiTexCoord0.xyz;\n"
+            "    gl_Position     = ftransform();  \n"
+            "    deltaTexCoord = normalize(gl_ModelViewMatrixInverse * vec4(0,0,0,1) - gl_Vertex);\n"
+            "}\n";
+
+        osg::Shader* vertex_shader = new osg::Shader(osg::Shader::VERTEX, vertexShaderSource);
+        program->addShader(vertex_shader);
+
+    }
+    
+    std::string fragmentShaderFile = osgDB::findDataFile("volume.frag");
+    if (!fragmentShaderFile.empty())
+    {
+        program->addShader(osg::Shader::readShaderFile(osg::Shader::FRAGMENT, fragmentShaderFile));
+    }
+    else
+    {
+        //////////////////////////////////////////////////////////////////
+        // fragment shader
+        //
+        char fragmentShaderSource[] = 
+            "uniform sampler3D baseTexture;\n"
+            "uniform float sampleDensity;\n"
+            "uniform float transparency;\n"
+            "uniform float alphaCutOff;\n"
+            "\n"
+            "varying vec3 deltaTexCoord;\n"
+            "varying vec3 texcoord;\n"
+            "void main(void) \n"
+            "{ \n"
+            "    vec3 deltaTexCoord2 = normalize(deltaTexCoord)*sampleDensity; \n"
+            "\n"
+            "    gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0); \n"
+            "    \n"
+            "    while (texcoord.x>=0.0 && texcoord.x<=1.0 &&\n"
+            "           texcoord.y>=0.0 && texcoord.y<=1.0 &&\n"
+            "           texcoord.z>=0.0 && texcoord.z<=1.0)\n"
+            "    {\n"
+            "       vec4 color = texture3D( baseTexture, texcoord);\n"
+            "       float r = color[3]*transparency;\n"
+            "       if (r>alphaCutOff)\n"
+            "       {\n"
+            "         gl_FragColor.xyz = gl_FragColor.xyz*(1.0-r)+color.xyz*r;\n"
+            "         gl_FragColor.w += r;\n"
+            "       }\n"
+            "       texcoord += deltaTexCoord2; \n"
+            "    }\n"
+            "    if (gl_FragColor.w>1.0) gl_FragColor.w = 1.0; \n"
+            "}\n";
+
+        osg::Shader* fragment_shader = new osg::Shader(osg::Shader::FRAGMENT, fragmentShaderSource);
+        program->addShader(fragment_shader);
+    }
+
+    osg::Uniform* baseTextureSampler = new osg::Uniform("baseTexture",0);
+    stateset->addUniform(baseTextureSampler);
+
+    osg::Uniform* sampleDensity = new osg::Uniform("sampleDensity", 0.01f);
+    stateset->addUniform(sampleDensity);
+
+    osg::Uniform* transpancy = new osg::Uniform("transparency",0.5f);
+    stateset->addUniform(transpancy);
+
+    osg::Uniform* alphaCutOff = new osg::Uniform("alphaCutOff",alphaFuncValue);
+    stateset->addUniform(alphaCutOff);
+
+    stateset->setMode(GL_CULL_FACE, osg::StateAttribute::ON);
+
+    {
+        osg::Geometry* geom = new osg::Geometry;
+
+        osg::Vec3Array* coords = new osg::Vec3Array(8);
+        (*coords)[0].set(0,0,0);
+        (*coords)[1].set(1,0,0);
+        (*coords)[2].set(1,1,0);
+        (*coords)[3].set(0,1,0);
+        (*coords)[4].set(0,0,1);
+        (*coords)[5].set(1,0,1);
+        (*coords)[6].set(1,1,1);
+        (*coords)[7].set(0,1,1);
+        geom->setVertexArray(coords);
+
+        osg::Vec3Array* tcoords = new osg::Vec3Array(8);
+        (*tcoords)[0].set(0,0,0);
+        (*tcoords)[1].set(1,0,0);
+        (*tcoords)[2].set(1,1,0);
+        (*tcoords)[3].set(0,1,0);
+        (*tcoords)[4].set(0,0,1);
+        (*tcoords)[5].set(1,0,1);
+        (*tcoords)[6].set(1,1,1);
+        (*tcoords)[7].set(0,1,1);
+        geom->setTexCoordArray(0,tcoords);
+
+        osg::Vec4Array* colours = new osg::Vec4Array(1);
+        (*colours)[0].set(1.0f,1.0f,1.0,1.0f);
+        geom->setColorArray(colours);
+        geom->setColorBinding(osg::Geometry::BIND_OVERALL);
+
+        osg::DrawElementsUShort* drawElements = new osg::DrawElementsUShort(GL_QUADS);
+        // bottom
+        drawElements->push_back(0);
+        drawElements->push_back(1);
+        drawElements->push_back(2);
+        drawElements->push_back(3);
+        
+        // bottom
+        drawElements->push_back(3);
+        drawElements->push_back(2);
+        drawElements->push_back(6);
+        drawElements->push_back(7);
+
+        // left
+        drawElements->push_back(0);
+        drawElements->push_back(3);
+        drawElements->push_back(7);
+        drawElements->push_back(4);
+
+        // right
+        drawElements->push_back(5);
+        drawElements->push_back(6);
+        drawElements->push_back(2);
+        drawElements->push_back(1);
+
+        // front
+        drawElements->push_back(1);
+        drawElements->push_back(0);
+        drawElements->push_back(4);
+        drawElements->push_back(5);
+
+        // top
+        drawElements->push_back(7);
+        drawElements->push_back(6);
+        drawElements->push_back(5);
+        drawElements->push_back(4);
+
+        geom->addPrimitiveSet(drawElements);
+
+        geode->addDrawable(geom);
+
+    } 
+
+    return geode;
+}
+
 osg::Node* createModel(osg::ref_ptr<osg::Image>& image_3d, osg::ref_ptr<osg::Image>& normalmap_3d,
                        osg::Texture::InternalFormatMode internalFormatMode,
                        float xSize, float ySize, float zSize,
@@ -726,8 +1011,8 @@ osg::Node* createModel(osg::ref_ptr<osg::Image>& image_3d, osg::ref_ptr<osg::Ima
 
     if (normalmap_3d.valid())
     {
-    	if (two_pass)
-	{
+        if (two_pass)
+        {
 
             // set up normal texture
             osg::Texture3D* bump_texture3D = new osg::Texture3D;
@@ -791,7 +1076,7 @@ osg::Node* createModel(osg::ref_ptr<osg::Image>& image_3d, osg::ref_ptr<osg::Ima
 
             stateset->setTextureAttributeAndModes(1,new osg::TexEnv(),osg::StateAttribute::ON);
 
-	}
+        }
         else
         {
             osg::ref_ptr<osg::Image> normalmap_3d = createNormalMapTexture(image_3d.get());
@@ -928,16 +1213,16 @@ struct WriteRowOperator
     std::vector<osg::Vec4>  _colours;
     mutable unsigned int    _pos;
     
-    inline void luminance(float& l) const { l = _colours[_pos++].red(); } 
-    inline void alpha(float& a) const { a = _colours[_pos++].alpha(); } 
-    inline void luminance_alpha(float& l,float& a) const { l = _colours[_pos].red(); a = _colours[_pos++].alpha(); } 
-    inline void rgb(float& r,float& g,float& b) const { r = _colours[_pos].red(); g = _colours[_pos].green(); b = _colours[_pos].blue(); }
-    inline void rgba(float& r,float& g,float& b,float& a) const {  r = _colours[_pos].red(); g = _colours[_pos].green(); b = _colours[_pos].blue(); a = _colours[_pos++].alpha(); }
+    inline void luminance(float& l) const { l = _colours[_pos++].r(); } 
+    inline void alpha(float& a) const { a = _colours[_pos++].a(); } 
+    inline void luminance_alpha(float& l,float& a) const { l = _colours[_pos].r(); a = _colours[_pos++].a(); } 
+    inline void rgb(float& r,float& g,float& b) const { r = _colours[_pos].r(); g = _colours[_pos].g(); b = _colours[_pos].b(); }
+    inline void rgba(float& r,float& g,float& b,float& a) const {  r = _colours[_pos].r(); g = _colours[_pos].g(); b = _colours[_pos].b(); a = _colours[_pos++].a(); }
 };
 
 osg::Image* readRaw(int sizeX, int sizeY, int sizeZ, int numberBytesPerComponent, int numberOfComponents, const std::string& endian, const std::string& raw_filename)
 {
-    std::ifstream fin(raw_filename.c_str());
+    std::ifstream fin(raw_filename.c_str(), std::ifstream::binary);
     if (!fin) return 0;
 
     GLenum pixelFormat;
@@ -1063,9 +1348,9 @@ enum ColourSpaceOperation
     REPLACE_ALPHA_WITH_LUMINACE
 };
 
-struct ModulatAlphaByLuminanceOperator
+struct ModulateAlphaByLuminanceOperator
 {
-    ModulatAlphaByLuminanceOperator() {}
+    ModulateAlphaByLuminanceOperator() {}
 
     inline void luminance(float&) const {} 
     inline void alpha(float&) const {} 
@@ -1074,9 +1359,9 @@ struct ModulatAlphaByLuminanceOperator
     inline void rgba(float& r,float& g,float& b,float& a) const { float l = (r+g+b)*0.3333333; a *= l;}
 };
 
-struct ModulatAlphaByColourOperator
+struct ModulateAlphaByColourOperator
 {
-    ModulatAlphaByColourOperator(const osg::Vec4& colour):_colour(colour) { _lum = _colour.length(); }
+    ModulateAlphaByColourOperator(const osg::Vec4& colour):_colour(colour) { _lum = _colour.length(); }
     
     osg::Vec4 _colour;
     float _lum;
@@ -1085,7 +1370,7 @@ struct ModulatAlphaByColourOperator
     inline void alpha(float&) const {} 
     inline void luminance_alpha(float& l,float& a) const { a*= l*_lum; } 
     inline void rgb(float&,float&,float&) const {}
-    inline void rgba(float& r,float& g,float& b,float& a) const { a = (r*_colour.red()+g*_colour.green()+b*_colour.blue()+a*_colour.alpha()); }
+    inline void rgba(float& r,float& g,float& b,float& a) const { a = (r*_colour.r()+g*_colour.g()+b*_colour.b()+a*_colour.a()); }
 };
 
 struct ReplaceAlphaWithLuminanceOperator
@@ -1105,11 +1390,11 @@ void doColourSpaceConversion(ColourSpaceOperation op, osg::Image* image, osg::Ve
     {
         case (MODULATE_ALPHA_BY_LUMINANCE):
             std::cout<<"doing conversion MODULATE_ALPHA_BY_LUMINANCE"<<std::endl;
-            modifyImage(image,ModulatAlphaByLuminanceOperator()); 
+            modifyImage(image,ModulateAlphaByLuminanceOperator()); 
             break;
         case (MODULATE_ALPHA_BY_COLOUR):
             std::cout<<"doing conversion MODULATE_ALPHA_BY_COLOUR"<<std::endl;
-            modifyImage(image,ModulatAlphaByColourOperator(colour)); 
+            modifyImage(image,ModulateAlphaByColourOperator(colour)); 
             break;
         case (REPLACE_ALPHA_WITH_LUMINACE):
             std::cout<<"doing conversion REPLACE_ALPHA_WITH_LUMINACE"<<std::endl;
@@ -1131,6 +1416,8 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");
     arguments.getApplicationUsage()->addCommandLineOption("-n","Create normal map for per voxel lighting.");
     arguments.getApplicationUsage()->addCommandLineOption("-s <numSlices>","Number of slices to create.");
+    arguments.getApplicationUsage()->addCommandLineOption("--images [filenames]","Specify a stack of 2d images to build the 3d volume from.");
+    arguments.getApplicationUsage()->addCommandLineOption("--shader","Use OpenGL Shading Language.");
     arguments.getApplicationUsage()->addCommandLineOption("--xSize <size>","Relative width of rendered brick.");
     arguments.getApplicationUsage()->addCommandLineOption("--ySize <size>","Relative length of rendered brick.");
     arguments.getApplicationUsage()->addCommandLineOption("--zSize <size>","Relative height of rendered brick.");
@@ -1142,13 +1429,14 @@ int main( int argc, char **argv )
     arguments.getApplicationUsage()->addCommandLineOption("--s_maxTextureSize <size>","Set the texture maximum resolution in the s (x) dimension.");
     arguments.getApplicationUsage()->addCommandLineOption("--t_maxTextureSize <size>","Set the texture maximum resolution in the t (y) dimension.");
     arguments.getApplicationUsage()->addCommandLineOption("--r_maxTextureSize <size>","Set the texture maximum resolution in the r (z) dimension.");
-    arguments.getApplicationUsage()->addCommandLineOption("--compressed","Enable the usage of compressed textures");
-    arguments.getApplicationUsage()->addCommandLineOption("--compressed-arb","Enable the usage of OpenGL ARB compressed textures");
-    arguments.getApplicationUsage()->addCommandLineOption("--compressed-dxt1","Enable the usage of S3TC DXT1 compressed textures");
-    arguments.getApplicationUsage()->addCommandLineOption("--compressed-dxt3","Enable the usage of S3TC DXT3 compressed textures");
-    arguments.getApplicationUsage()->addCommandLineOption("--compressed-dxt5","Enable the usage of S3TC DXT5 compressed textures");
+    arguments.getApplicationUsage()->addCommandLineOption("--compressed","Enable the usage of compressed textures.");
+    arguments.getApplicationUsage()->addCommandLineOption("--compressed-arb","Enable the usage of OpenGL ARB compressed textures.");
+    arguments.getApplicationUsage()->addCommandLineOption("--compressed-dxt1","Enable the usage of S3TC DXT1 compressed textures.");
+    arguments.getApplicationUsage()->addCommandLineOption("--compressed-dxt3","Enable the usage of S3TC DXT3 compressed textures.");
+    arguments.getApplicationUsage()->addCommandLineOption("--compressed-dxt5","Enable the usage of S3TC DXT5 compressed textures.");
     arguments.getApplicationUsage()->addCommandLineOption("--modulate-alpha-by-luminance","For each pixel multiple the alpha value by the luminance.");
-    arguments.getApplicationUsage()->addCommandLineOption("--replace-alpha-with-luminance","For each pixel mSet the alpha value to the luminance");
+    arguments.getApplicationUsage()->addCommandLineOption("--replace-alpha-with-luminance","For each pixel mSet the alpha value to the luminance.");
+    arguments.getApplicationUsage()->addCommandLineOption("--num-components <num>","Set the number of components to in he target image.");
 //    arguments.getApplicationUsage()->addCommandLineOption("--raw <sizeX> <sizeY> <sizeZ> <numberBytesPerComponent> <numberOfComponents> <endian> <filename>","read a raw image data");
 
     // construct the viewer.
@@ -1225,6 +1513,12 @@ int main( int argc, char **argv )
     while(arguments.read("--replace-alpha-with-luminance")) { colourSpaceOperation = REPLACE_ALPHA_WITH_LUMINACE; }
         
     
+    unsigned int numComponentsDesired = 0; 
+    while(arguments.read("--num-components", numComponentsDesired)) {}
+
+    bool useShader = false; 
+    while(arguments.read("--shader")) { useShader = true; }
+
     osg::ref_ptr<osg::Image> image_3d;
 
     int sizeX, sizeY, sizeZ, numberBytesPerComponent, numberOfComponents;
@@ -1250,7 +1544,7 @@ int main( int argc, char **argv )
         
         // pack the textures into a single texture.
         ProcessRow processRow;
-        image_3d = createTexture3D(imageList, processRow, 0, s_maximumTextureSize, t_maximumTextureSize, r_maximumTextureSize);
+        image_3d = createTexture3D(imageList, processRow, numComponentsDesired, s_maximumTextureSize, t_maximumTextureSize, r_maximumTextureSize);
     }
 
 
@@ -1286,12 +1580,25 @@ int main( int argc, char **argv )
 
 
     // create a model from the images.
-    osg::Node* rootNode = createModel(image_3d, normalmap_3d, 
-                                      internalFormatMode,
-                                      xSize, ySize, zSize,
-                                      xMultiplier, yMultiplier, zMultiplier,
-                                      numSlices, sliceEnd, alphaFunc);
-
+    osg::Node* rootNode = 0;
+    
+    if (useShader)
+    {
+        rootNode = createShaderModel(image_3d, normalmap_3d, 
+                               internalFormatMode,
+                               xSize, ySize, zSize,
+                               xMultiplier, yMultiplier, zMultiplier,
+                               numSlices, sliceEnd, alphaFunc);
+    }
+    else
+    {
+        rootNode = createModel(image_3d, normalmap_3d, 
+                               internalFormatMode,
+                               xSize, ySize, zSize,
+                               xMultiplier, yMultiplier, zMultiplier,
+                               numSlices, sliceEnd, alphaFunc);
+    }
+    
     if (!outputFile.empty())
     {   
         std::string ext = osgDB::getFileExtension(outputFile);
