@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -189,3 +189,15 @@ void AutoTransform::accept(NodeVisitor& nv)
     // now do the proper accept
     Transform::accept(nv);
 }
+
+BoundingSphere AutoTransform::computeBound() const
+{
+    BoundingSphere bsphere;
+
+    if ( getAutoScaleToScreen() && _firstTimeToInitEyePoint )
+        return bsphere;
+
+    bsphere = Transform::computeBound();
+
+    return bsphere;
+}
