@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -20,7 +20,12 @@ using namespace osg;
 
 
 BlendColor::BlendColor() :
-            _constantColor(0.0f,0.0f,0.0f,0.0f)
+    _constantColor(0.0f,0.0f,0.0f,0.0f)
+{
+}
+
+BlendColor::BlendColor(const osg::Vec4& constantColor):
+    _constantColor(constantColor)
 {
 }
 
