@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -68,6 +68,7 @@ void DisplaySettings::setDisplaySettings(const DisplaySettings& vs)
     _minimumNumberStencilBits = vs._minimumNumberStencilBits;
 
     _maxNumOfGraphicsContexts = vs._maxNumOfGraphicsContexts;
+    _numMultiSamples = vs._numMultiSamples;
 }
 
 void DisplaySettings::merge(const DisplaySettings& vs)
@@ -82,6 +83,7 @@ void DisplaySettings::merge(const DisplaySettings& vs)
     
     if (vs._minimumNumberAlphaBits>_minimumNumberAlphaBits) _minimumNumberAlphaBits = vs._minimumNumberAlphaBits;
     if (vs._minimumNumberStencilBits>_minimumNumberStencilBits) _minimumNumberStencilBits = vs._minimumNumberStencilBits;
+    if (vs._numMultiSamples>_numMultiSamples) _numMultiSamples = vs._numMultiSamples;
 }
 
 void DisplaySettings::setDefaults()
@@ -108,12 +110,30 @@ void DisplaySettings::setDefaults()
     _depthBuffer = true;
     _minimumNumberAlphaBits = 0;
     _minimumNumberStencilBits = 0;
+    _minimumNumberAccumRedBits = 0;
+    _minimumNumberAccumGreenBits = 0;
+    _minimumNumberAccumBlueBits = 0;
+    _minimumNumberAccumAlphaBits = 0;
     
-    _maxNumOfGraphicsContexts = 3;
+    _maxNumOfGraphicsContexts = 1;
+    _numMultiSamples = 0;
+
+    #ifdef __sgi
+    // switch on anti-aliasing by default, just in case we have an Onyx :-)
+    _numMultiSamples = 4;
+    #endif
+}
+
+void DisplaySettings::setMinimumNumAccumBits(unsigned int red, unsigned int green, unsigned int blue, unsigned int alpha)
+{
+    _minimumNumberAccumRedBits = red;
+    _minimumNumberAccumGreenBits = green;
+    _minimumNumberAccumBlueBits = blue;
+    _minimumNumberAccumAlphaBits = alpha;
 }
 
 static ApplicationUsageProxy DisplaySetting_e0(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_DISPLAY_TYPE <type>","MONITOR | POWERWALL | REALITY_CENTER | HEAD_MOUNTED_DISPLAY");
-static ApplicationUsageProxy DisplaySetting_e1(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_STEREO_MODE <mode>","QUAD_BUFFER | ANAGLYPHIC | HORIZONTAL_SPLIT | VERTICAL_SPLIT | LEFT_EYE | RIGHT_EYE");
+static ApplicationUsageProxy DisplaySetting_e1(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_STEREO_MODE <mode>","QUAD_BUFFER | ANAGLYPHIC | HORIZONTAL_SPLIT | VERTICAL_SPLIT | LEFT_EYE | RIGHT_EYE | VERTICAL_INTERLACE | HORIZONTAL_INTERLACE");
 static ApplicationUsageProxy DisplaySetting_e2(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_STEREO <mode>","OFF | ON");
 static ApplicationUsageProxy DisplaySetting_e3(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_EYE_SEPARATION <float>","physical distance between eyes");
 static ApplicationUsageProxy DisplaySetting_e4(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_SCREEN_DISTANCE <float>","physical distance between eyes and screen");
@@ -128,7 +148,7 @@ static ApplicationUsageProxy DisplaySetting_e12(ApplicationUsage::ENVIRONMENTAL_
 
 void DisplaySettings::readEnvironmentalVariables()
 {
-    char *ptr;
+    const char* ptr = 0;
     
     if ((ptr = getenv("OSG_DISPLAY_TYPE")) != 0)
     {
@@ -159,31 +179,34 @@ void DisplaySettings::readEnvironmentalVariables()
         {
             _stereoMode = QUAD_BUFFER;
         }
-        else
-        if (strcmp(ptr,"ANAGLYPHIC")==0)
+        else if (strcmp(ptr,"ANAGLYPHIC")==0)
         {
             _stereoMode = ANAGLYPHIC;
         }
-        else
-        if (strcmp(ptr,"HORIZONTAL_SPLIT")==0)
+        else if (strcmp(ptr,"HORIZONTAL_SPLIT")==0)
         {
             _stereoMode = HORIZONTAL_SPLIT;
         }
-        else
-        if (strcmp(ptr,"VERTICAL_SPLIT")==0)
+        else if (strcmp(ptr,"VERTICAL_SPLIT")==0)
         {
             _stereoMode = VERTICAL_SPLIT;
         }
-        else
-        if (strcmp(ptr,"LEFT_EYE")==0)
+        else if (strcmp(ptr,"LEFT_EYE")==0)
         {
             _stereoMode = LEFT_EYE;
         }
-        else
-        if (strcmp(ptr,"RIGHT_EYE")==0)
+        else if (strcmp(ptr,"RIGHT_EYE")==0)
         {
             _stereoMode = RIGHT_EYE;
         }
+        else if (strcmp(ptr,"HORIZONTAL_INTERLACE")==0)
+        {
+            _stereoMode = HORIZONTAL_INTERLACE;
+        }
+        else if (strcmp(ptr,"VERTICAL_INTERLACE")==0)
+        {
+            _stereoMode = VERTICAL_INTERLACE;
+        }
     }
 
     if( (ptr = getenv("OSG_STEREO")) != 0)
@@ -286,6 +309,9 @@ void DisplaySettings::readCommandLine(ArgumentParser& arguments)
         arguments.getApplicationUsage()->addCommandLineOption("--stereo <mode>","ANAGLYPHIC | QUAD_BUFFER | HORIZONTAL_SPLIT | VERTICAL_SPLIT | LEFT_EYE | RIGHT_EYE | ON | OFF ");
         arguments.getApplicationUsage()->addCommandLineOption("--rgba","Request a RGBA color buffer visual");
         arguments.getApplicationUsage()->addCommandLineOption("--stencil","Request a stencil buffer visual");
+        arguments.getApplicationUsage()->addCommandLineOption("--accum-rgb","Request a rgb accumulator buffer visual");
+        arguments.getApplicationUsage()->addCommandLineOption("--accum-rgba","Request a rgb accumulator buffer visual");
+        arguments.getApplicationUsage()->addCommandLineOption("--samples <num>","Request a multisample visual");
     }
 
     std::string str;
@@ -304,6 +330,8 @@ void DisplaySettings::readCommandLine(ArgumentParser& arguments)
         else if (arguments.match(pos+1,"QUAD_BUFFER"))      { arguments.remove(pos,2); _stereo = true;_stereoMode = QUAD_BUFFER; }
         else if (arguments.match(pos+1,"HORIZONTAL_SPLIT")) { arguments.remove(pos,2); _stereo = true;_stereoMode = HORIZONTAL_SPLIT; }
         else if (arguments.match(pos+1,"VERTICAL_SPLIT"))   { arguments.remove(pos,2); _stereo = true;_stereoMode = VERTICAL_SPLIT; }
+        else if (arguments.match(pos+1,"HORIZONTAL_INTERLACE")) { arguments.remove(pos,2); _stereo = true;_stereoMode = HORIZONTAL_INTERLACE; }
+        else if (arguments.match(pos+1,"VERTICAL_INTERLACE"))   { arguments.remove(pos,2); _stereo = true;_stereoMode = VERTICAL_INTERLACE; }
         else if (arguments.match(pos+1,"LEFT_EYE"))         { arguments.remove(pos,2); _stereo = true;_stereoMode = LEFT_EYE; }
         else if (arguments.match(pos+1,"RIGHT_EYE"))        { arguments.remove(pos,2); _stereo = true;_stereoMode = RIGHT_EYE; }
         else if (arguments.match(pos+1,"ON"))               { arguments.remove(pos,2); _stereo = true; }
@@ -321,4 +349,19 @@ void DisplaySettings::readCommandLine(ArgumentParser& arguments)
     {
         _minimumNumberStencilBits = 1;
     }
+
+    while (arguments.read("--accum-rgb"))
+    {
+        setMinimumNumAccumBits(8,8,8,0);
+    }
+
+    while (arguments.read("--accum-rgba"))
+    {
+        setMinimumNumAccumBits(8,8,8,8);
+    }
+
+    while(arguments.read("--samples",str))
+    {
+        _numMultiSamples = atoi(str.c_str());
+    }
 }
