@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -299,6 +299,9 @@ Drawable::Drawable(const Drawable& drawable,const CopyOp& copyop):
 
 Drawable::~Drawable()
 {
+    // cleanly detatch any associated stateset (include remove parent links)
+    setStateSet(0);
+
     dirtyDisplayList();
 }
 
@@ -653,19 +656,15 @@ void Drawable::setEventCallback(EventCallback* ac)
 
 struct ComputeBound : public PrimitiveFunctor
 {
-        ComputeBound():_vertices(0) {}
-        
-        virtual void setVertexArray(unsigned int,const Vec2*) 
+   ComputeBound()  {   _vertices = 0;  _vertices4 = 0;}
+
+        virtual void setVertexArray(unsigned int,const Vec2*)
         {
             notify(WARN)<<"ComputeBound does not support Vec2* vertex arrays"<<std::endl;
         }
 
-        virtual void setVertexArray(unsigned int,const Vec3* vertices) { _vertices = vertices; }
-
-        virtual void setVertexArray(unsigned int,const Vec4*) 
-        {
-            notify(WARN)<<"ComputeBound does not support Vec4* vertex arrays"<<std::endl;
-        }
+        virtual void setVertexArray(unsigned int,const Vec3* vertices) { _vertices  = vertices; }
+        virtual void setVertexArray(unsigned int,const Vec4* vertices) { _vertices4 = vertices; }
 
         virtual void drawArrays(GLenum,GLint first,GLsizei count)
         {
@@ -677,6 +676,15 @@ struct ComputeBound : public PrimitiveFunctor
                     _bb.expandBy(*vert);
                 }
             }
+
+            if (_vertices4)
+            {
+                const osg::Vec4* vert = _vertices4+first;
+                for(;count>0;--count,++vert)
+                {
+                    _bb.expandBy(*((Vec3*) vert));
+                }
+            }
         }
 
         virtual void drawElements(GLenum,GLsizei count,const GLubyte* indices)
@@ -688,6 +696,14 @@ struct ComputeBound : public PrimitiveFunctor
                     _bb.expandBy(_vertices[*indices]);
                 }
             }
+
+            if (_vertices4)
+            {
+                for(;count>0;--count,++indices)
+                {
+                    _bb.expandBy(*((Vec3*)&_vertices4[*indices]));
+                }
+            }
         }
 
         virtual void drawElements(GLenum,GLsizei count,const GLushort* indices)
@@ -699,6 +715,14 @@ struct ComputeBound : public PrimitiveFunctor
                     _bb.expandBy(_vertices[*indices]);
                 }
             }
+
+            if (_vertices4)
+            {
+                for(;count>0;--count,++indices)
+                {
+                    _bb.expandBy(*((Vec3*)&_vertices4[*indices]));
+                }
+            }
         }
 
         virtual void drawElements(GLenum,GLsizei count,const GLuint* indices)
@@ -710,19 +734,28 @@ struct ComputeBound : public PrimitiveFunctor
                     _bb.expandBy(_vertices[*indices]);
                 }
             }
+
+            if (_vertices4)
+            {
+                for(;count>0;--count,++indices)
+                {
+                    _bb.expandBy(*((Vec3*)&_vertices4[*indices]));
+                }
+            }
         }
 
         virtual void begin(GLenum) {}
         virtual void vertex(const Vec2& vert) { _bb.expandBy(osg::Vec3(vert[0],vert[1],0.0f)); }
         virtual void vertex(const Vec3& vert) { _bb.expandBy(vert); }
         virtual void vertex(const Vec4& vert) { if (vert[3]!=0.0f) _bb.expandBy(osg::Vec3(vert[0],vert[1],vert[2])/vert[3]); }
-        virtual void vertex(float x,float y) { _bb.expandBy(x,y,1.0f); }
+        virtual void vertex(float x,float y)  { _bb.expandBy(x,y,1.0f); }
         virtual void vertex(float x,float y,float z) { _bb.expandBy(x,y,z); }
         virtual void vertex(float x,float y,float z,float w) { if (w!=0.0f) _bb.expandBy(x/w,y/w,z/w); }
         virtual void end() {}
-        
+       
         const Vec3*     _vertices;
-        BoundingBox     _bb;
+        const Vec4*     _vertices4;
+        BoundingBox     _bb; 
 };
 
 BoundingBox Drawable::computeBound() const
@@ -1294,6 +1327,14 @@ GLboolean Drawable::Extensions::glIsQuery(GLuint id) const
   return false;
 }
 
+void Drawable::Extensions::glDeleteQueries(GLsizei n, const GLuint *ids) const
+{
+    if (_gl_delete_queries_arb) 
+        _gl_delete_queries_arb(n, ids);
+    else
+        osg::notify(osg::WARN) << "Error: glIsQuery not supported by OpenGL driver" << std::endl;
+}
+
 void Drawable::Extensions::glGetQueryObjectiv(GLuint id, GLenum pname, GLint *params) const
 {
   if (_gl_get_query_objectiv_arb)
