@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -24,6 +24,18 @@
 #include <vector>
 #include <set>
 
+#if defined(WIN32)
+#include <windows.h>
+#endif
+
+float osg::getGLVersionNumber()
+{
+    // needs to be extended to do proper things with subversions like 1.5.1, etc.
+    char *versionstring   = (char*) glGetString( GL_VERSION );
+    std::string vs( versionstring );
+    return( atof( vs.substr( 0, vs.find( " " ) ).c_str() ) );
+}
+
 bool osg::isGLExtensionSupported(unsigned int contextID, const char *extension)
 {
     typedef std::set<std::string>  ExtensionSet;
@@ -56,6 +68,44 @@ bool osg::isGLExtensionSupported(unsigned int contextID, const char *extension)
             startOfWord = endOfWord+1;
         }
         if (*startOfWord!=0) extensionSet.insert(std::string(startOfWord));
+
+#if defined(WIN32)
+
+        // add WGL extensions to the list
+
+        typedef const char* WINAPI WGLGETEXTENSIONSSTRINGARB(HDC);
+        WGLGETEXTENSIONSSTRINGARB* wglGetExtensionsStringARB = 
+            (WGLGETEXTENSIONSSTRINGARB*)getGLExtensionFuncPtr("wglGetExtensionsStringARB");
+        
+        typedef const char* WINAPI WGLGETEXTENSIONSSTRINGEXT();
+        WGLGETEXTENSIONSSTRINGEXT* wglGetExtensionsStringEXT = 
+            (WGLGETEXTENSIONSSTRINGEXT*)getGLExtensionFuncPtr("wglGetExtensionsStringEXT");
+        
+        const char* wglextensions = 0;
+        
+        if (wglGetExtensionsStringARB)
+        {
+            HDC dc = wglGetCurrentDC();
+            wglextensions = wglGetExtensionsStringARB(dc);
+        }
+        else if (wglGetExtensionsStringEXT)
+        {
+            wglextensions = wglGetExtensionsStringEXT();
+        }
+
+        if (wglextensions)
+        {    
+            const char* startOfWord = wglextensions;
+            const char* endOfWord;
+            while ((endOfWord = strchr(startOfWord, ' ')))
+            {
+                extensionSet.insert(std::string(startOfWord, endOfWord));
+                startOfWord = endOfWord+1;
+            }
+            if (*startOfWord != 0) extensionSet.insert(std::string(startOfWord));
+        }
+
+#endif
         
         osg::notify(INFO)<<"OpenGL extensions supported by installed OpenGL drivers are:"<<std::endl;
         for(ExtensionSet::iterator itr=extensionSet.begin();
@@ -201,3 +251,67 @@ bool osg::isGLUExtensionSupported(unsigned int contextID, const char *extension)
 
     return result;
 }
+
+#if defined(WIN32)
+    #define WIN32_LEAN_AND_MEAN
+    #ifndef NOMINMAX
+        #define NOMINMAX
+    #endif // NOMINMAX
+    #include <windows.h>
+#elif defined(__APPLE__)
+    #include <mach-o/dyld.h>
+#else
+    #include <dlfcn.h>
+#endif
+
+void* osg::getGLExtensionFuncPtr(const char *funcName)
+{
+#if defined(WIN32)
+
+    return (void*)wglGetProcAddress(funcName);
+
+#elif defined(__APPLE__)
+
+    std::string temp( "_" );
+    temp += funcName;    // Mac OS X prepends an underscore on function names
+    if ( NSIsSymbolNameDefined( temp.c_str() ) )
+    {
+        NSSymbol symbol = NSLookupAndBindSymbol( temp.c_str() );
+        return NSAddressOfSymbol( symbol );
+    } else
+        return NULL;
+
+#elif defined (__sun) 
+
+     static void *handle = dlopen((const char *)0L, RTLD_LAZY);
+     return dlsym(handle, funcName);
+    
+#elif defined (__sgi)
+
+     static void *handle = dlopen((const char *)0L, RTLD_LAZY);
+     return dlsym(handle, funcName);
+
+#elif defined (__FreeBSD__)
+
+    return dlsym( RTLD_DEFAULT, funcName );
+
+#elif defined (__linux__)
+
+    typedef void (*__GLXextFuncPtr)(void);
+    typedef __GLXextFuncPtr (*GetProcAddressARBProc)(const char*);
+    static GetProcAddressARBProc s_glXGetProcAddressARB = (GetProcAddressARBProc)dlsym(0, "glXGetProcAddressARB");
+    if (s_glXGetProcAddressARB)
+    {
+        return (void*) (s_glXGetProcAddressARB)(funcName);
+    }
+    else
+    {
+        return dlsym(0, funcName);
+    }
+
+#else // all other unixes
+
+    return dlsym(0, funcName);
+
+#endif
+}
