@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -144,12 +144,7 @@ bool Group::insertChild( unsigned int index, Node *child )
     else return false;
 }
 
-bool Group::removeChild( Node *child )
-{
-    return removeChild(getChildIndex(child));
-}
-
-bool Group::removeChild(unsigned int pos,unsigned int numChildrenToRemove)
+bool Group::removeChildren(unsigned int pos,unsigned int numChildrenToRemove)
 {
     if (pos<_children.size() && numChildrenToRemove>0)
     {
