@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -102,7 +102,7 @@ int Image::compare(const Image& rhs) const
     COMPARE_StateAttribute_Parameter(_modifiedCount)
 
     // same buffer + same parameters = same image
-    if (_data == rhs._data) return 0;
+    if ((_data || rhs._data) && (_data == rhs._data)) return 0;
 
     // slowest comparison at the bottom!
     COMPARE_StateAttribute_Parameter(getFileName())    
@@ -308,9 +308,11 @@ unsigned int Image::getTotalSizeInBytesIncludingMipmaps() const
         case(GL_COMPRESSED_RGB_S3TC_DXT1_EXT):
         case(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT):
            sizeOfLastMipMap = maximum(sizeOfLastMipMap, 8u); // block size of 8
+           break;
         case(GL_COMPRESSED_RGBA_S3TC_DXT3_EXT):
         case(GL_COMPRESSED_RGBA_S3TC_DXT5_EXT):
            sizeOfLastMipMap = maximum(sizeOfLastMipMap, 16u); // block size of 16
+           break;
         default: break;
    }
 
@@ -426,7 +428,7 @@ void Image::readPixels(int x,int y,int width,int height,
 }
 
 
-void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMapsIfAvailable)
+void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMapsIfAvailable, GLenum type)
 {
     const osg::Texture::Extensions* extensions = osg::Texture::getExtensions(contextID,true);
     const osg::Texture3D::Extensions* extensions3D = osg::Texture3D::getExtensions(contextID,true);
@@ -445,6 +447,7 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
     GLint width;
     GLint height;
     GLint depth;
+    GLint packing;
 
     GLint numMipMaps = 0;
     if (copyMipMapsIfAvailable)
@@ -514,6 +517,8 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
         glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_WIDTH, &width);
         glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_HEIGHT, &height);
         glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_DEPTH, &depth);
+        glGetIntegerv(GL_UNPACK_ALIGNMENT, &packing);
+        glPixelStorei(GL_PACK_ALIGNMENT, packing);
 
         _data = data;
         _s = width;
@@ -521,10 +526,11 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
         _r = depth;
         
         _pixelFormat = internalformat;
-        _dataType = internalformat;
+        _dataType = type;
         _internalTextureFormat = internalformat;
         _mipmapData = mipMapData;
         _allocationMode=USE_NEW_DELETE;
+        _packing = packing;
         
         for(i=0;i<numMipMaps;++i)
         {
@@ -538,6 +544,12 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
     {
         MipmapDataType mipMapData;
 
+        // Get the internal texture format and packing value from OpenGL,
+        // instead of using possibly outdated values from the class.
+        glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_INTERNAL_FORMAT, &internalformat);
+        glGetIntegerv(GL_UNPACK_ALIGNMENT, &packing);
+        glPixelStorei(GL_PACK_ALIGNMENT, packing);
+
         unsigned int total_size = 0;
         GLint i;
         for(i=0;i<numMipMaps;++i)
@@ -548,7 +560,7 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
             glGetTexLevelParameteriv(textureMode, i, GL_TEXTURE_HEIGHT, &height);
             glGetTexLevelParameteriv(textureMode, i, GL_TEXTURE_DEPTH, &depth);
             
-            unsigned int level_size = computeRowWidthInBytes(width,_pixelFormat,GL_UNSIGNED_BYTE,_packing)*height*depth;
+            unsigned int level_size = computeRowWidthInBytes(width,internalformat,type,packing)*height*depth;
 
             total_size += level_size;
         }
@@ -563,7 +575,6 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
 
         deallocateData(); // and sets it to NULL.
 
-        glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_INTERNAL_FORMAT, &internalformat);
         glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_WIDTH, &width);
         glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_HEIGHT, &height);
         glGetTexLevelParameteriv(textureMode, 0, GL_TEXTURE_DEPTH, &depth);
@@ -574,10 +585,11 @@ void Image::readImageFromCurrentTexture(unsigned int contextID, bool copyMipMaps
         _r = depth;
         
         _pixelFormat = internalformat;
-        _dataType = GL_UNSIGNED_BYTE;
+        _dataType = type;
         _internalTextureFormat = internalformat;
         _mipmapData = mipMapData;
         _allocationMode=USE_NEW_DELETE;
+        _packing = packing;
         
         for(i=0;i<numMipMaps;++i)
         {
@@ -888,13 +900,34 @@ bool Image::isImageTranslucent() const
             const unsigned char* d = data(0,it,ir);
             switch(_dataType)
             {
-                case(GL_BYTE):              return _findLowerAlphaValueInRow(s(), (char*)d +offset,            (char)127, delta);
-                case(GL_UNSIGNED_BYTE):     return _findLowerAlphaValueInRow(s(), (unsigned char*)d + offset,  (unsigned char)255, delta);
-                case(GL_SHORT):             return _findLowerAlphaValueInRow(s(), (short*)d + offset,          (short)32767, delta);
-                case(GL_UNSIGNED_SHORT):    return _findLowerAlphaValueInRow(s(), (unsigned short*)d + offset, (unsigned short)65535, delta);
-                case(GL_INT):               return _findLowerAlphaValueInRow(s(), (int*)d + offset,            (int)2147483647, delta);
-                case(GL_UNSIGNED_INT):      return _findLowerAlphaValueInRow(s(), (unsigned int*)d + offset,   4294967295u, delta);
-                case(GL_FLOAT):             return _findLowerAlphaValueInRow(s(), (float*)d + offset,          1.0f, delta);
+                case(GL_BYTE):
+                    if (_findLowerAlphaValueInRow(s(), (char*)d +offset, (char)127, delta))
+                        return true;
+                    break;
+                case(GL_UNSIGNED_BYTE):
+                    if (_findLowerAlphaValueInRow(s(), (unsigned char*)d + offset, (unsigned char)255, delta))
+                        return true;
+                    break;
+                case(GL_SHORT):
+                    if (_findLowerAlphaValueInRow(s(), (short*)d + offset, (short)32767, delta))
+                        return true;
+                    break;
+                case(GL_UNSIGNED_SHORT):
+                    if (_findLowerAlphaValueInRow(s(), (unsigned short*)d + offset, (unsigned short)65535, delta))
+                        return true;
+                    break;
+                case(GL_INT):
+                    if (_findLowerAlphaValueInRow(s(), (int*)d + offset, (int)2147483647, delta))
+                        return true;
+                    break;
+                case(GL_UNSIGNED_INT):
+                    if (_findLowerAlphaValueInRow(s(), (unsigned int*)d + offset, 4294967295u, delta))
+                        return true;
+                    break;
+                case(GL_FLOAT):
+                    if (_findLowerAlphaValueInRow(s(), (float*)d + offset, 1.0f, delta))
+                        return true;
+                    break;
             }
         }
     }
