@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -124,9 +124,9 @@ void PagedLOD::traverse(NodeVisitor& nv)
             else
             {
                 osg::CullStack* cullStack = dynamic_cast<osg::CullStack*>(&nv);
-                if (cullStack)
+                if (cullStack && cullStack->getLODScale()>0.0f)
                 {
-                    required_range = cullStack->pixelSize(getBound());
+                    required_range = cullStack->clampedPixelSize(getBound()) / cullStack->getLODScale();
                 }
                 else
                 {
@@ -134,7 +134,7 @@ void PagedLOD::traverse(NodeVisitor& nv)
                     // finding out the max range
                     for(unsigned int i=0;i<_rangeList.size();++i)
                     {
-                        required_range = osg::minimum(required_range,_rangeList[i].first);
+                        required_range = osg::maximum(required_range,_rangeList[i].first);
                     }
                 }
             }
@@ -170,12 +170,18 @@ void PagedLOD::traverse(NodeVisitor& nv)
                     _children[numChildren-1]->accept(nv);
                 }
                 
-                // now request the loading of the next unload child.
+                // now request the loading of the next unloaded child.
                 if (nv.getDatabaseRequestHandler() && numChildren<_perRangeDataList.size())
                 {
                     // compute priority from where abouts in the required range the distance falls.
                     float priority = (_rangeList[numChildren].second-required_range)/(_rangeList[numChildren].second-_rangeList[numChildren].first);
                     
+                    // invert priority for PIXEL_SIZE_ON_SCREEN mode
+                    if(_rangeMode==PIXEL_SIZE_ON_SCREEN)
+                    {
+                        priority = -priority;
+                    }
+                    
                     // modify the priority according to the child's priority offset and scale.
                     priority = _perRangeDataList[numChildren]._priorityOffset + priority * _perRangeDataList[numChildren]._priorityScale;
                     
@@ -185,7 +191,7 @@ void PagedLOD::traverse(NodeVisitor& nv)
                     }
                     else
                     {
-                        // prepend the databasePath to the childs filename.
+                        // prepend the databasePath to the child's filename.
                         nv.getDatabaseRequestHandler()->requestNodeFile(_databasePath+_perRangeDataList[numChildren]._filename,this,priority,nv.getFrameStamp());
                     }
                 }
@@ -202,27 +208,6 @@ void PagedLOD::traverse(NodeVisitor& nv)
 }
 
 
-void PagedLOD::childRemoved(unsigned int pos, unsigned int numChildrenToRemove)
-{
-    LOD::childRemoved(pos, numChildrenToRemove);
-}
-
-void PagedLOD::childInserted(unsigned int pos)
-{
-    LOD::childInserted(pos);
-}
-
-void PagedLOD::rangeRemoved(unsigned int pos, unsigned int numChildrenToRemove)
-{
-    LOD::rangeRemoved(pos, numChildrenToRemove);
-}
-
-void PagedLOD::rangeInserted(unsigned int pos)
-{
-    LOD::rangeInserted(pos);
-    expandPerRangeDataTo(pos);
-}
-
 void PagedLOD::expandPerRangeDataTo(unsigned int pos)
 {
     if (pos>=_perRangeDataList.size()) _perRangeDataList.resize(pos+1);
@@ -232,7 +217,7 @@ bool PagedLOD::addChild( Node *child )
 {
     if (LOD::addChild(child))
     {
-        expandPerRangeDataTo(_children.size());
+        expandPerRangeDataTo(_children.size()-1);
         return true;
     }
     return false;
@@ -242,7 +227,7 @@ bool PagedLOD::addChild(Node *child, float min, float max)
 {
     if (LOD::addChild(child,min,max))
     {
-        expandPerRangeDataTo(_children.size());
+        expandPerRangeDataTo(_children.size()-1);
         return true;
     }
     return false;
@@ -261,16 +246,12 @@ bool PagedLOD::addChild(Node *child, float min, float max,const std::string& fil
     return false;
 }
 
-bool PagedLOD::removeChild( Node *child )
+bool PagedLOD::removeChildren( unsigned int pos,unsigned int numChildrenToRemove)
 {
-    // find the child's position.
-    unsigned int pos=getChildIndex(child);
-    if (pos==_children.size()) return false;
-    
-    if (pos<_rangeList.size()) _rangeList.erase(_rangeList.begin()+pos);
-    if (pos<_perRangeDataList.size()) _perRangeDataList.erase(_perRangeDataList.begin()+pos);
-    
-    return Group::removeChild(child);    
+    if (pos<_rangeList.size()) _rangeList.erase(_rangeList.begin()+pos, osg::minimum(_rangeList.begin()+(pos+numChildrenToRemove), _rangeList.end()) );
+    if (pos<_perRangeDataList.size()) _perRangeDataList.erase(_perRangeDataList.begin()+pos, osg::minimum(_perRangeDataList.begin()+ (pos+numChildrenToRemove), _perRangeDataList.end()) );
+            
+    return Group::removeChildren(pos,numChildrenToRemove);
 }
 
 bool PagedLOD::removeExpiredChildren(double expiryTime,NodeList& removedChildren)
@@ -281,7 +262,7 @@ bool PagedLOD::removeExpiredChildren(double expiryTime,NodeList& removedChildren
         {
             osg::Node* nodeToRemove = _children[_children.size()-1].get();
             removedChildren.push_back(nodeToRemove);
-            return Group::removeChild(nodeToRemove);
+            return Group::removeChildren(_children.size()-1,1);
         }
     }
     return false;
