@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -13,9 +13,28 @@
 #include <osg/PrimitiveSet>
 #include <osg/BufferObject>
 #include <osg/State>
+#include <osg/Notify>
 
 using namespace osg;
 
+unsigned int PrimitiveSet::getNumPrimitives() const
+{
+    switch(_mode)
+    {
+        case(POINTS): return getNumIndices();
+        case(LINES): return getNumIndices()/2;
+        case(TRIANGLES): return getNumIndices()/3;
+        case(QUADS): return getNumIndices()/4;
+        case(LINE_STRIP):
+        case(LINE_LOOP):
+        case(TRIANGLE_STRIP):
+        case(TRIANGLE_FAN):
+        case(QUAD_STRIP):
+        case(POLYGON): return 1;
+    }
+    return 0;
+}
+
 void DrawArrays::draw(State&, bool) const 
 {
     glDrawArrays(_mode,_first,_count);
@@ -31,10 +50,28 @@ void DrawArrays::accept(PrimitiveIndexFunctor& functor) const
     functor.drawArrays(_mode,_first,_count);
 }
 
+unsigned int DrawArrayLengths::getNumPrimitives() const
+{
+    switch(_mode)
+    {
+        case(POINTS): return getNumIndices();
+        case(LINES): return getNumIndices()/2;
+        case(TRIANGLES): return getNumIndices()/3;
+        case(QUADS): return getNumIndices()/4;
+        case(LINE_STRIP):
+        case(LINE_LOOP):
+        case(TRIANGLE_STRIP):
+        case(TRIANGLE_FAN):
+        case(QUAD_STRIP):
+        case(POLYGON): return size();
+    }
+    return 0;
+}
+
 void DrawArrayLengths::draw(State&, bool) const
 {
     GLint first = _first;
-    for(VectorSizei::const_iterator itr=begin();
+    for(vector_type::const_iterator itr=begin();
         itr!=end();
         ++itr)
     {
@@ -46,7 +83,7 @@ void DrawArrayLengths::draw(State&, bool) const
 void DrawArrayLengths::accept(PrimitiveFunctor& functor) const
 {
     GLint first = _first;
-    for(VectorSizei::const_iterator itr=begin();
+    for(vector_type::const_iterator itr=begin();
         itr!=end();
         ++itr)
     {
@@ -58,7 +95,7 @@ void DrawArrayLengths::accept(PrimitiveFunctor& functor) const
 void DrawArrayLengths::accept(PrimitiveIndexFunctor& functor) const
 {
     GLint first = _first;
-    for(VectorSizei::const_iterator itr=begin();
+    for(vector_type::const_iterator itr=begin();
         itr!=end();
         ++itr)
     {
@@ -70,7 +107,7 @@ void DrawArrayLengths::accept(PrimitiveIndexFunctor& functor) const
 unsigned int DrawArrayLengths::getNumIndices() const
 {
     unsigned int count = 0;
-    for(VectorSizei::const_iterator itr=begin();
+    for(vector_type::const_iterator itr=begin();
         itr!=end();
         ++itr)
     {
@@ -81,12 +118,29 @@ unsigned int DrawArrayLengths::getNumIndices() const
 
 DrawElementsUByte::~DrawElementsUByte()
 {
-    for(unsigned int i=0;i<_vboList.size();++i)
+    releaseGLObjects();
+}
+
+void DrawElementsUByte::releaseGLObjects(State* state) const
+{
+    if (state)
     {
-        if (_vboList[i] != 0)
+        unsigned int contextID = state->getContextID();
+        if (_vboList[contextID]._objectID != 0)
         {
-            BufferObject::deleteBufferObject(i,_vboList[i]);
-            _vboList[i] = 0;
+            BufferObject::deleteBufferObject(contextID,_vboList[contextID]._objectID);
+            _vboList[contextID]._objectID = 0;
+        }
+    }
+    else
+    {
+        for(unsigned int i=0;i<_vboList.size();++i)
+        {
+            if (_vboList[i]._objectID != 0)
+            {
+                BufferObject::deleteBufferObject(i,_vboList[i]._objectID);
+                _vboList[i]._objectID = 0;
+            }
         }
     }
 }
@@ -95,21 +149,32 @@ void DrawElementsUByte::draw(State& state, bool useVertexBufferObjects) const
 {
     if (useVertexBufferObjects)
     {
-        const BufferObject::Extensions* extensions = BufferObject::getExtensions(state.getContextID(), true);
+        unsigned int contextID = state.getContextID();
+        const BufferObject::Extensions* extensions = BufferObject::getExtensions(contextID, true);
 
-        GLuint& buffer = _vboList[state.getContextID()];
+        GLuint& buffer = _vboList[contextID]._objectID;
         if (!buffer)
         {
             extensions->glGenBuffers(1, &buffer);
             extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, buffer);
             extensions->glBufferData(GL_ELEMENT_ARRAY_BUFFER_ARB, size() * 1, &front(), GL_STATIC_DRAW_ARB);
+            // osg::notify(osg::NOTICE)<<"Generating ubyte buffer"<<buffer<<std::endl;
+            
+            _vboList[contextID]._modifiedCount = _modifiedCount;
         }
         else
         {
             extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, buffer);
+        
+            if (_vboList[contextID]._modifiedCount != _modifiedCount) 
+            {
+                extensions->glBufferData(GL_ELEMENT_ARRAY_BUFFER_ARB, size() * 1, &front(), GL_STATIC_DRAW_ARB);
+                _vboList[contextID]._modifiedCount = _modifiedCount;
+            }
         }
     
         glDrawElements(_mode, size(), GL_UNSIGNED_BYTE, 0);
+        
         extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, 0);
     }
     else 
@@ -141,12 +206,29 @@ void DrawElementsUByte::offsetIndices(int offset)
 
 DrawElementsUShort::~DrawElementsUShort()
 {
-    for(unsigned int i=0;i<_vboList.size();++i)
+    releaseGLObjects();
+}
+
+void DrawElementsUShort::releaseGLObjects(State* state) const
+{
+    if (state)
     {
-        if (_vboList[i] != 0)
+        unsigned int contextID = state->getContextID();
+        if (_vboList[contextID]._objectID != 0)
         {
-            BufferObject::deleteBufferObject(i,_vboList[i]);
-            _vboList[i] = 0;
+            BufferObject::deleteBufferObject(contextID,_vboList[contextID]._objectID);
+            _vboList[contextID]._objectID = 0;
+        }
+    }
+    else
+    {
+        for(unsigned int i=0;i<_vboList.size();++i)
+        {
+            if (_vboList[i]._objectID != 0)
+            {
+                BufferObject::deleteBufferObject(i,_vboList[i]._objectID);
+                _vboList[i]._objectID = 0;
+            }
         }
     }
 }
@@ -155,22 +237,34 @@ void DrawElementsUShort::draw(State& state, bool useVertexBufferObjects) const
 {
     if (useVertexBufferObjects)
     {
-        const BufferObject::Extensions* extensions = BufferObject::getExtensions(state.getContextID(), true);
+        unsigned int contextID = state.getContextID();
+        const BufferObject::Extensions* extensions = BufferObject::getExtensions(contextID, true);
 
-        GLuint& buffer = _vboList[state.getContextID()];
+        GLuint& buffer = _vboList[contextID]._objectID;
         if (!buffer)
         {
+        
             extensions->glGenBuffers(1, &buffer);
             extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, buffer);
             extensions->glBufferData(GL_ELEMENT_ARRAY_BUFFER_ARB, size() * 2, &front(), GL_STATIC_DRAW_ARB);
+            // osg::notify(osg::NOTICE)<<"Generating ushort buffer"<<buffer<<std::endl;
+            
+            _vboList[contextID]._modifiedCount = _modifiedCount;
         }
         else
         {
             extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, buffer);
+        
+            if (_vboList[contextID]._modifiedCount != _modifiedCount) 
+            {
+                extensions->glBufferData(GL_ELEMENT_ARRAY_BUFFER_ARB, size() * 2, &front(), GL_STATIC_DRAW_ARB);
+                _vboList[contextID]._modifiedCount = _modifiedCount;
+            }
         }
     
-        extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, 0);
         glDrawElements(_mode, size(), GL_UNSIGNED_SHORT, 0);
+
+        extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, 0);
     }
     else 
     {
@@ -201,12 +295,29 @@ void DrawElementsUShort::offsetIndices(int offset)
 
 DrawElementsUInt::~DrawElementsUInt()
 {
-    for(unsigned int i=0;i<_vboList.size();++i)
+    releaseGLObjects();
+}
+
+void DrawElementsUInt::releaseGLObjects(State* state) const
+{
+    if (state)
     {
-        if (_vboList[i] != 0)
+        unsigned int contextID = state->getContextID();
+        if (_vboList[contextID]._objectID != 0)
         {
-            BufferObject::deleteBufferObject(i,_vboList[i]);
-            _vboList[i] = 0;
+            BufferObject::deleteBufferObject(contextID,_vboList[contextID]._objectID);
+            _vboList[contextID]._objectID = 0;
+        }
+    }
+    else
+    {
+        for(unsigned int i=0;i<_vboList.size();++i)
+        {
+            if (_vboList[i]._objectID != 0)
+            {
+                BufferObject::deleteBufferObject(i,_vboList[i]._objectID);
+                _vboList[i]._objectID = 0;
+            }
         }
     }
 }
@@ -215,21 +326,33 @@ void DrawElementsUInt::draw(State& state, bool useVertexBufferObjects) const
 {
     if (useVertexBufferObjects)
     {
-        const BufferObject::Extensions* extensions = BufferObject::getExtensions(state.getContextID(), true);
-
-        GLuint& buffer = _vboList[state.getContextID()];
+        unsigned int contextID = state.getContextID();
+        const BufferObject::Extensions* extensions = BufferObject::getExtensions(contextID, true);
+        
+        GLuint& buffer = _vboList[contextID]._objectID;
         if (!buffer)
         {
             extensions->glGenBuffers(1, &buffer);
             extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, buffer);
             extensions->glBufferData(GL_ELEMENT_ARRAY_BUFFER_ARB, size() * 4, &front(), GL_STATIC_DRAW_ARB);
+            // osg::notify(osg::NOTICE)<<"Generating buffer int"<<buffer<<std::endl;
+            
+            _vboList[contextID]._modifiedCount = _modifiedCount;
         }
         else
         {
+            // osg::notify(osg::NOTICE)<<"binding buffer int"<<buffer<<std::endl;
             extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, buffer);
+        
+            if (_vboList[contextID]._modifiedCount != _modifiedCount) 
+            {
+                extensions->glBufferData(GL_ELEMENT_ARRAY_BUFFER_ARB, size() * 4, &front(), GL_STATIC_DRAW_ARB);
+                _vboList[contextID]._modifiedCount = _modifiedCount;
+            }
         }
     
         glDrawElements(_mode, size(), GL_UNSIGNED_INT, 0);
+        
         extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, 0);
     }
     else 
