@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  * Copyright (C) 2003-2005 3Dlabs Inc. Ltd.
  * Copyright (C) 2004-2005 Nathan Cournia
  *
@@ -13,7 +13,7 @@
 */
 
 /* file:        src/osg/Program.cpp
- * author:      Mike Weiblen 2005-05-05
+ * author:      Mike Weiblen 2006-03-25
 */
 
 #include <fstream>
@@ -270,7 +270,14 @@ void GL2Extensions::lowestCommonDenominator(const GL2Extensions& rhs)
 
 void GL2Extensions::setupGL2Extensions(unsigned int contextID)
 {
-    _glVersion = atof( (const char*)glGetString( GL_VERSION ) );
+    const char* version = (const char*) glGetString( GL_VERSION );
+    if (!version)
+    {
+        osg::notify(osg::FATAL)<<"Error: OpenGL version test failed, requires valid graphics context."<<std::endl;
+        return;
+    }
+    
+    _glVersion = atof( version );
     _glslLanguageVersion = 0.0f;
 
     _isShaderObjectsSupported = osg::isGLExtensionSupported(contextID,"GL_ARB_shader_objects");
@@ -1992,9 +1999,19 @@ void Program::dirtyProgram()
 }
 
 
-void Program::releaseGLObjects(osg::State* /*state*/) const
+void Program::releaseGLObjects(osg::State* state) const
 {
-    // TODO
+    for( unsigned int i=0; i < _shaderList.size(); ++i )
+    {
+        if (_shaderList[i].valid()) _shaderList[i]->releaseGLObjects(state);
+    }
+
+    if (!state) _pcpList.setAllElementsTo(0);
+    else
+    {
+        unsigned int contextID = state->getContextID();
+        _pcpList[contextID] = 0;
+    }   
 }
 
 
@@ -2020,12 +2037,14 @@ bool Program::removeShader( Shader* shader )
     if( !shader ) return false;
 
     // Shader must exist to be removed.
-    for( unsigned int i=0; i < _shaderList.size(); ++i )
+    for( ShaderList::iterator itr = _shaderList.begin();
+         itr != _shaderList.end();
+         ++itr)
     {
-        if( shader == _shaderList[i].get() )
+        if( shader == itr->get() )
         {
             shader->removeProgramRef( this );
-            _shaderList[i] = 0;
+            _shaderList.erase(itr);
             dirtyProgram();
             return true;
         }
@@ -2064,6 +2083,12 @@ void Program::apply( osg::State& state ) const
     if( pcp->needsLink() ) compileGLObjects( state );
     if( pcp->isLinked() )
     {
+        // for shader debugging: to minimize performance impact,
+        // optionally validate based on notify level.
+        // TODO: enable this using notify level, or perhaps its own getenv()?
+        if( osg::isNotifyEnabled(osg::INFO) )
+            pcp->validateProgram();
+
         pcp->useProgram();
         state.setLastAppliedProgramObject(pcp);
     }
@@ -2106,12 +2131,12 @@ bool Program::getGlProgramInfoLog(unsigned int contextID, std::string& log) cons
     return getPCP( contextID )->getInfoLog( log );
 }
 
-const Program::NameInfoMap& Program::getActiveUniforms(unsigned int contextID) const
+const Program::ActiveVarInfoMap& Program::getActiveUniforms(unsigned int contextID) const
 {
     return getPCP( contextID )->getActiveUniforms();
 }
 
-const Program::NameInfoMap& Program::getActiveAttribs(unsigned int contextID) const
+const Program::ActiveVarInfoMap& Program::getActiveAttribs(unsigned int contextID) const
 {
     return getPCP( contextID )->getActiveAttribs();
 }
@@ -2152,6 +2177,7 @@ void Program::PerContextProgram::linkProgram()
     osg::notify(osg::INFO)
         << "Linking osg::Program \"" << _program->getName() << "\""
         << " id=" << _glProgramHandle
+        << " contextID=" << _contextID
         <<  std::endl;
 
     _uniformInfoMap.clear();
@@ -2214,11 +2240,12 @@ void Program::PerContextProgram::linkProgram()
             
             if( loc != -1 )
             {
-                _uniformInfoMap[name] = std::pair<GLint,GLenum>(loc,type);
+                _uniformInfoMap[name] = ActiveVarInfo(loc,type,size);
 
                 osg::notify(osg::INFO)
                     << "\tUniform \"" << name << "\""
                     << " loc="<< loc
+                    << " size="<< size
                     << " type=" << Uniform::getTypename((Uniform::Type)type)
                     << std::endl;
             }
@@ -2245,11 +2272,12 @@ void Program::PerContextProgram::linkProgram()
             
             if( loc != -1 )
             {
-                _attribInfoMap[name] = std::pair<GLint,GLenum>(loc,type);
+                _attribInfoMap[name] = ActiveVarInfo(loc,type,size);
 
                 osg::notify(osg::INFO)
                     << "\tAttrib \"" << name << "\""
                     << " loc=" << loc
+                    << " size=" << size
                     << std::endl;
             }
         }
@@ -2258,6 +2286,29 @@ void Program::PerContextProgram::linkProgram()
     osg::notify(osg::INFO) << std::endl;
 }
 
+bool Program::PerContextProgram::validateProgram()
+{
+    GLint validated = GL_FALSE;
+    _extensions->glValidateProgram( _glProgramHandle );
+    _extensions->glGetProgramiv( _glProgramHandle, GL_VALIDATE_STATUS, &validated );
+    if( validated == GL_TRUE)
+        return true;
+
+    osg::notify(osg::INFO)
+        << "glValidateProgram FAILED \"" << _program->getName() << "\""
+        << " id=" << _glProgramHandle
+        << " contextID=" << _contextID
+        <<  std::endl;
+
+    std::string infoLog;
+    if( getInfoLog(infoLog) )
+        osg::notify(osg::INFO) << "infolog:\n" << infoLog << std::endl;
+
+    osg::notify(osg::INFO) << std::endl;
+    
+    return false;
+}
+
 bool Program::PerContextProgram::getInfoLog( std::string& infoLog ) const
 {
     return _extensions->getProgramInfoLog( _glProgramHandle, infoLog );
