@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -21,6 +21,7 @@ using namespace osg;
 
 State::State()
 {
+    _graphicsContext = 0;
     _contextID = 0;
     _identity = new osg::RefMatrix(); // default RefMatrix constructs to identity.
     _initialViewMatrix = _identity;
@@ -52,7 +53,7 @@ State::~State()
 void State::reset()
 {
 
-/*
+#if 1
     for(ModeMap::iterator mitr=_modeMap.begin();
         mitr!=_modeMap.end();
         ++mitr)
@@ -62,9 +63,10 @@ void State::reset()
         ms.last_applied_value = !ms.global_default_value;
         ms.changed = true;
     }        
-*/
-
+#else
     _modeMap.clear();
+#endif
+
     _modeMap[GL_DEPTH_TEST].global_default_value = true;
     _modeMap[GL_DEPTH_TEST].changed = true;
     
@@ -825,9 +827,13 @@ bool State::checkGLErrors(const char* str) const
     GLenum errorNo = glGetError();
     if (errorNo!=GL_NO_ERROR)
     {
-        osg::notify(WARN)<<"Warning: detected OpenGL error '"<<gluErrorString(errorNo);
-        if (str) osg::notify(WARN)<<"' at "<<str<< std::endl;
-        else osg::notify(WARN)<<"' in osg::State."<< std::endl;
+        const char* error = (char*)gluErrorString(errorNo);
+        if (error) osg::notify(WARN)<<"Warning: detected OpenGL error '" << error<<"'";
+        else       osg::notify(WARN)<<"Warning: detected OpenGL error number 0x" << std::hex << errorNo;
+
+        if (str) osg::notify(WARN)<<" at "<<str<< std::endl;
+        else     osg::notify(WARN)<<" in osg::State."<< std::endl;
+
         return true;
     }
     return false;
@@ -838,7 +844,10 @@ bool State::checkGLErrors(StateAttribute::GLMode mode) const
     GLenum errorNo = glGetError();
     if (errorNo!=GL_NO_ERROR)
     {
-        osg::notify(WARN)<<"Warning: detected OpenGL error '"<<gluErrorString(errorNo)<<"' after applying GLMode 0x"<<hex<<mode<<dec<< std::endl;
+        const char* error = (char*)gluErrorString(errorNo);
+        if (error) osg::notify(WARN)<<"Warning: detected OpenGL error '"<< error <<"' after applying GLMode 0x"<<hex<<mode<<dec<< std::endl;
+        else       osg::notify(WARN)<<"Warning: detected OpenGL error number 0x"<< std::hex << errorNo <<" after applying GLMode 0x"<<hex<<mode<<dec<< std::endl;
+
         return true;
     }
     return false;
@@ -849,10 +858,13 @@ bool State::checkGLErrors(const StateAttribute* attribute) const
     GLenum errorNo = glGetError();
     if (errorNo!=GL_NO_ERROR)
     {
-        osg::notify(WARN)<<"Warning: detected OpenGL error '"<<gluErrorString(errorNo)<<"' after applying attribute "<<attribute->className()<<" "<<attribute<< std::endl;
-        //attribute->apply(*this);
+        const char* error = (char*)gluErrorString(errorNo);
+        if (error) osg::notify(WARN)<<"Warning: detected OpenGL error '"<< error <<"' after applying attribute "<<attribute->className()<<" "<<attribute<< std::endl;
+        else       osg::notify(WARN)<<"Warning: detected OpenGL error number 0x"<< std::hex << errorNo <<" after applying attribute "<<attribute->className()<<" "<<attribute<< std::endl;
+
         return true;
     }
     return false;
 }
 
+
