@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -423,7 +423,7 @@ void StateSet::setGlobalDefaults()
 
 
     setMode(GL_DEPTH_TEST,StateAttribute::ON);
-    setAttributeAndModes(new AlphaFunc,StateAttribute::OFF);
+    // setAttributeAndModes(new AlphaFunc,StateAttribute::OFF);
     setAttributeAndModes(new BlendFunc,StateAttribute::OFF);
 
     Material *material       = new Material;
@@ -438,6 +438,7 @@ void StateSet::clear()
 
     setRenderBinToInherit();
 
+
     // remove self from as attributes parent
     for(AttributeList::iterator itr=_attributeList.begin();
         itr!=_attributeList.end();
@@ -449,9 +450,6 @@ void StateSet::clear()
     _modeList.clear();
     _attributeList.clear();
     
-    
-    _textureModeList.clear();
-    _textureAttributeList.clear();
 
     // remove self from as texture attributes parent
     for(unsigned int i=0;i<_textureAttributeList.size();++i)
@@ -465,6 +463,9 @@ void StateSet::clear()
         }
     }
 
+    _textureModeList.clear();
+    _textureAttributeList.clear();
+
 
     // remove self from uniforms parent
     for(UniformList::iterator uitr = _uniformList.begin();
@@ -923,7 +924,7 @@ Uniform* StateSet::getUniform(const std::string& name)
     else return 0;
 }
 
-Uniform* StateSet::getOrCreateUniform(const std::string& name, Uniform::Type type)
+Uniform* StateSet::getOrCreateUniform(const std::string& name, Uniform::Type type, unsigned int numElements)
 {
     // for look for an appropriate uniform.
     UniformList::iterator itr = _uniformList.find(name);
@@ -935,7 +936,7 @@ Uniform* StateSet::getOrCreateUniform(const std::string& name, Uniform::Type typ
 
     // no uniform found matching name so create it..
     
-    Uniform* uniform = new Uniform(type,name);
+    Uniform* uniform = new Uniform(type,name,numElements);
     addUniform(uniform);
     
     return uniform;
@@ -1130,6 +1131,32 @@ const StateSet::RefAttributePair* StateSet::getTextureAttributePair(unsigned int
     return getAttributePair(_textureAttributeList[unit],type,0);
 }
 
+bool StateSet::checkValidityOfAssociatedModes(osg::State& state) const
+{
+
+
+    bool modesValid = true;
+    for(AttributeList::const_iterator itr = _attributeList.begin();
+        itr!=_attributeList.end();
+        ++itr)
+    {
+        if (!itr->second.first->checkValidityOfAssociatedModes(state)) modesValid = false;
+    }
+
+    for(TextureAttributeList::const_iterator taitr=_textureAttributeList.begin();
+        taitr!=_textureAttributeList.end();
+        ++taitr)
+    {
+        for(AttributeList::const_iterator itr = taitr->begin();
+            itr!=taitr->end();
+            ++itr)
+        {
+            if (!itr->second.first->checkValidityOfAssociatedModes(state)) modesValid = false;
+        }
+    }
+
+    return modesValid;
+}
 
 void StateSet::compileGLObjects(State& state) const
 {
