@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -17,6 +17,7 @@
 #include <osg/Notify>
 #include <osg/GLU>
 #include <osg/Timer>
+#include <osg/ApplicationUsage>
 
 #include <OpenThreads/ScopedLock>
 #include <OpenThreads/Mutex>
@@ -42,6 +43,8 @@ using namespace osg;
 
 // #define DO_TIMING
 
+ApplicationUsageProxy Texture_e0(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_MAX_TEXTURE_SIZE","Set the maximum size of textures.");
+
 unsigned int Texture::s_numberTextureReusedLastInLastFrame = 0;
 unsigned int Texture::s_numberNewTextureInLastFrame = 0;
 unsigned int Texture::s_numberDeletedTextureInLastFrame = 0;
@@ -337,11 +340,13 @@ Texture::Texture():
             _useHardwareMipMapGeneration(true),
             _unrefImageDataAfterApply(false),
             _clientStorageHint(false),
-            _resizeNonPowerOfTwoHint(false),
+            _resizeNonPowerOfTwoHint(true),
             _borderColor(0.0, 0.0, 0.0, 0.0),
             _borderWidth(0),
             _internalFormatMode(USE_IMAGE_DATA_FORMAT),
             _internalFormat(0),
+            _sourceFormat(0),
+            _sourceType(0),
             _use_shadow_comparison(false),
             _shadow_compare_func(LEQUAL),
             _shadow_texture_mode(LUMINANCE),
@@ -365,6 +370,8 @@ Texture::Texture(const Texture& text,const CopyOp& copyop):
             _borderWidth(text._borderWidth),
             _internalFormatMode(text._internalFormatMode),
             _internalFormat(text._internalFormat),
+            _sourceFormat(text._sourceFormat),
+            _sourceType(text._sourceType),
             _use_shadow_comparison(text._use_shadow_comparison),
             _shadow_compare_func(text._shadow_compare_func),
             _shadow_texture_mode(text._shadow_texture_mode),
@@ -395,6 +402,9 @@ int Texture::compareTexture(const Texture& rhs) const
         COMPARE_StateAttribute_Parameter(_internalFormat)
     }
 
+    COMPARE_StateAttribute_Parameter(_sourceFormat)
+    COMPARE_StateAttribute_Parameter(_sourceType)
+
     COMPARE_StateAttribute_Parameter(_use_shadow_comparison)
     COMPARE_StateAttribute_Parameter(_shadow_compare_func)
     COMPARE_StateAttribute_Parameter(_shadow_texture_mode)
@@ -589,7 +599,7 @@ bool Texture::isCompressedInternalFormat() const
     return isCompressedInternalFormat(getInternalFormat());
 }
 
-bool Texture::isCompressedInternalFormat(GLint internalFormat) const
+bool Texture::isCompressedInternalFormat(GLint internalFormat)
 {
     switch(internalFormat)
     {
@@ -609,12 +619,17 @@ bool Texture::isCompressedInternalFormat(GLint internalFormat) const
     }
 }
 
-void Texture::getCompressedSize(GLenum internalFormat, GLint width, GLint height, GLint depth, GLint& blockSize, GLint& size) const
+void Texture::getCompressedSize(GLenum internalFormat, GLint width, GLint height, GLint depth, GLint& blockSize, GLint& size)
 {
     if (internalFormat == GL_COMPRESSED_RGB_S3TC_DXT1_EXT || internalFormat == GL_COMPRESSED_RGBA_S3TC_DXT1_EXT)
         blockSize = 8;
-    else
+    else if (internalFormat == GL_COMPRESSED_RGBA_S3TC_DXT3_EXT || internalFormat == GL_COMPRESSED_RGBA_S3TC_DXT5_EXT)
         blockSize = 16;
+    else
+    {
+        notify(WARN)<<"Texture::getCompressedSize(...) : cannot compute correct size of compressed format ("<<internalFormat<<") returning 0."<<std::endl;
+        blockSize = 0;
+    }
          
     size = ((width+3)/4)*((height+3)/4)*depth*blockSize;        
 }
@@ -682,9 +697,9 @@ void Texture::applyTexParameters(GLenum target, State& state) const
     }
 
     if (extensions->isShadowSupported() && target == GL_TEXTURE_2D)
-      {
+    {
         if (_use_shadow_comparison)
-          {
+        {
             glTexParameteri(target, GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE_ARB);
             glTexParameteri(target, GL_TEXTURE_COMPARE_FUNC_ARB, _shadow_compare_func);
             glTexParameteri(target, GL_DEPTH_TEXTURE_MODE_ARB, _shadow_texture_mode);
@@ -692,15 +707,15 @@ void Texture::applyTexParameters(GLenum target, State& state) const
             // if ambient value is 0 - it is default behaviour of GL_ARB_shadow
             // no need for GL_ARB_shadow_ambient in this case
             if (extensions->isShadowAmbientSupported() && _shadow_ambient > 0)
-              {
+            {
                 glTexParameterf(target, TEXTURE_COMPARE_FAIL_VALUE_ARB, _shadow_ambient);
-              }
-          }
+            }
+        }
         else 
-          {
+        {
             glTexParameteri(target, GL_TEXTURE_COMPARE_MODE_ARB, GL_NONE);
-          }
-      }
+        }
+    }
 
     getTextureParameterDirty(state.getContextID()) = false;
 
@@ -788,6 +803,31 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
 
     // select the internalFormat required for the texture.
     bool compressed_image = isCompressedInternalFormat((GLenum)image->getPixelFormat());
+
+    // If the texture's internal format is a compressed type, then the
+    // user is requesting that the graphics card compress the image if it's
+    // not already compressed. However, if the image is not a multiple of 
+    // four in each dimension the subsequent calls to glTexSubImage* will
+    // fail. Revert to uncompressed format in this case.
+    if (isCompressedInternalFormat(_internalFormat) &&
+        (((inwidth >> 2) << 2) != inwidth ||
+         ((inheight >> 2) << 2) != inheight))
+    {
+        osg::notify(osg::NOTICE)<<"Received a request to compress an image, but image size is not a multiple of four ("<<inwidth<<"x"<<inheight<<"). Reverting to uncompressed.\n";
+        switch(_internalFormat)
+        {
+            case GL_COMPRESSED_RGB_S3TC_DXT1_EXT:
+            case GL_COMPRESSED_RGB: _internalFormat = GL_RGB; break;
+            case GL_COMPRESSED_RGBA_S3TC_DXT1_EXT:
+            case GL_COMPRESSED_RGBA_S3TC_DXT3_EXT:
+            case GL_COMPRESSED_RGBA_S3TC_DXT5_EXT:
+            case GL_COMPRESSED_RGBA: _internalFormat = GL_RGBA; break;
+            case GL_COMPRESSED_ALPHA: _internalFormat = GL_ALPHA; break;
+            case GL_COMPRESSED_LUMINANCE: _internalFormat = GL_LUMINANCE; break;
+            case GL_COMPRESSED_LUMINANCE_ALPHA: _internalFormat = GL_LUMINANCE_ALPHA; break;
+            case GL_COMPRESSED_INTENSITY: _internalFormat = GL_INTENSITY; break;
+        }
+    }
     
     glPixelStorei(GL_UNPACK_ALIGNMENT,image->getPacking());
     
@@ -1021,6 +1061,19 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
     }
     // else image size the same as when loaded so we can go ahead and subload
     
+    // If the texture's internal format is a compressed type, then the
+    // user is requesting that the graphics card compress the image if it's
+    // not already compressed. However, if the image is not a multiple of 
+    // four in each dimension the subsequent calls to glTexSubImage* will
+    // fail. Revert to uncompressed format in this case.
+    if (isCompressedInternalFormat(_internalFormat) &&
+        (((inwidth >> 2) << 2) != inwidth ||
+         ((inheight >> 2) << 2) != inheight))
+    {
+        applyTexImage2D_load(state, target, image, inwidth, inheight, numMipmapLevels);
+        return;
+    }
+
 #ifdef DO_TIMING
     osg::Timer_t start_tick = osg::Timer::instance()->tick();
     osg::notify(osg::NOTICE)<<"glTexSubImage2D pixelFormat = "<<std::hex<<image->getPixelFormat()<<std::dec<<std::endl;
@@ -1124,7 +1177,7 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
         else if (extensions->isCompressedTexImage2DSupported())
         {        
             GLint blockSize,size;
-            getCompressedSize(_internalFormat, inwidth, inheight, 1, blockSize,size);
+            getCompressedSize(image->getInternalTextureFormat(), inwidth, inheight, 1, blockSize,size);
 
             extensions->glCompressedTexSubImage2D(target, 0, 
                 0,0,
@@ -1176,7 +1229,7 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
                     if (height == 0)
                         height = 1;
 
-                    getCompressedSize(_internalFormat, width, height, 1, blockSize,size);
+                    getCompressedSize(image->getInternalTextureFormat(), width, height, 1, blockSize,size);
 
                     //state.checkGLErrors("before extensions->glCompressedTexSubImage2D(");
 
@@ -1197,7 +1250,9 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
         }
         else
         {
-            notify(WARN)<<"Warning:: cannot subload mip mapped texture from non mipmapped image."<<std::endl;
+            //notify(WARN)<<"Warning:: cannot subload mip mapped texture from non mipmapped image."<<std::endl;
+            applyTexImage2D_load(state, target, image, inwidth, inheight,numMipmapLevels); 
+            return;
         }
     }
     
@@ -1321,7 +1376,15 @@ void Texture::Extensions::lowestCommonDenominator(const Extensions& rhs)
 
 void Texture::Extensions::setupGLExtensions(unsigned int contextID)
 {
-    float glVersion = atof( (const char *)glGetString( GL_VERSION ) );
+    const char* version = (const char*) glGetString( GL_VERSION );
+    if (!version)
+    {
+        osg::notify(osg::FATAL)<<"Error: In Texture::Extensions::setupGLExtensions(..) OpenGL version test failed, requires valid graphics context."<<std::endl;
+        return;
+    }
+    
+
+    float glVersion = atof( (const char *)version );
     const char* renderer = (const char*) glGetString(GL_RENDERER);
     std::string rendererString(renderer ? renderer : "");
     
@@ -1358,6 +1421,12 @@ void Texture::Extensions::setupGLExtensions(unsigned int contextID)
         osg::notify(osg::INFO)<<"Disabling _isNonPowerOfTwoTextureMipMappedSupported for ATI hardware."<<std::endl;
     }
 
+    if (rendererString.find("GeForce FX")!=std::string::npos)
+    {
+        _isNonPowerOfTwoTextureMipMappedSupported = false;
+        osg::notify(osg::INFO)<<"Disabling _isNonPowerOfTwoTextureMipMappedSupported for GeForce FX hardware."<<std::endl;
+    }
+
     glGetIntegerv(GL_MAX_TEXTURE_SIZE,&_maxTextureSize);
 
     char *ptr;
@@ -1425,3 +1494,4 @@ void Texture::Extensions::glGetCompressedTexImage(GLenum target, GLint level, GL
         notify(WARN)<<"Error: glGetCompressedTexImage not supported by OpenGL driver"<<std::endl;
     }
 }
+
