@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -158,6 +158,28 @@ void Texture1D::apply(State& state) const
             non_const_this->_image = 0;
         }
         
+    }
+    else if ( (_textureWidth!=0) && (_internalFormat!=0) )
+    {
+        _textureObjectBuffer[contextID] = textureObject = generateTextureObject(
+                contextID,GL_TEXTURE_1D,_numMipmapLevels,_internalFormat,_textureWidth,1,1,0);
+        
+        textureObject->bind();
+
+        applyTexParameters(GL_TEXTURE_1D,state);
+
+        // no image present, but dimensions at set so lets create the texture
+        glTexImage1D( GL_TEXTURE_1D, 0, _internalFormat,
+                     _textureWidth, _borderWidth,
+                     _sourceFormat ? _sourceFormat : _internalFormat,
+                     _sourceType ? _sourceType : GL_UNSIGNED_BYTE,
+                     0);                
+                     
+        if (_readPBuffer.valid())
+        {
+            _readPBuffer->bindPBufferToTexture(GL_FRONT);
+        }
+        
     }
     else
     {
