@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -130,10 +130,14 @@ void Texture2D::apply(State& state) const
             applyTexImage2D_subload(state,GL_TEXTURE_2D,_image.get(),
                                     _textureWidth, _textureHeight, _internalFormat, _numMipmapLevels);
  
-            // update the modified tag to show that it is upto date.
+            // update the modified tag to show that it is up to date.
             getModifiedCount(contextID) = _image->getModifiedCount();
      
         }
+        else if (_readPBuffer.valid())
+        {
+            _readPBuffer->bindPBufferToTexture(GL_FRONT);
+        }
 
     }
     else if (_subloadCallback.valid())
@@ -159,11 +163,14 @@ void Texture2D::apply(State& state) const
     else if (_image.valid() && _image->data())
     {
 
+        // keep the image around at least till we go out of scope.
+        osg::ref_ptr<osg::Image> image = _image;
+
         // compute the internal texture format, this set the _internalFormat to an appropriate value.
         computeInternalFormat();
         
         // compute the dimensions of the texture.
-        computeRequiredTextureDimensions(state,*_image,_textureWidth, _textureHeight, _numMipmapLevels);
+        computeRequiredTextureDimensions(state,*image,_textureWidth, _textureHeight, _numMipmapLevels);
         
         
         _textureObjectBuffer[contextID] = textureObject = generateTextureObject(
@@ -176,13 +183,13 @@ void Texture2D::apply(State& state) const
         if (textureObject->isAllocated())
         {
             //std::cout<<"Reusing texture object"<<std::endl;
-            applyTexImage2D_subload(state,GL_TEXTURE_2D,_image.get(),
+            applyTexImage2D_subload(state,GL_TEXTURE_2D,image.get(),
                                  _textureWidth, _textureHeight, _internalFormat, _numMipmapLevels);
         }
         else
         {
             //std::cout<<"Creating new texture object"<<std::endl;
-            applyTexImage2D_load(state,GL_TEXTURE_2D,_image.get(),
+            applyTexImage2D_load(state,GL_TEXTURE_2D,image.get(),
                                  _textureWidth, _textureHeight, _numMipmapLevels);
                                  
             textureObject->setAllocated(true);
@@ -190,10 +197,10 @@ void Texture2D::apply(State& state) const
         
         
         // update the modified tag to show that it is upto date.
-        getModifiedCount(contextID) = _image->getModifiedCount();
+        getModifiedCount(contextID) = image->getModifiedCount();
 
 
-        if (_unrefImageDataAfterApply && areAllTextureObjectsLoaded() && _image->getDataVariance()==STATIC)
+        if (_unrefImageDataAfterApply && areAllTextureObjectsLoaded() && image->getDataVariance()==STATIC)
         {
             Texture2D* non_const_this = const_cast<Texture2D*>(this);
             non_const_this->_image = 0;
@@ -205,6 +212,28 @@ void Texture2D::apply(State& state) const
         // perhaps it is the first glBind which is not required...
         //glBindTexture( GL_TEXTURE_2D, handle );
         
+    }
+    else if ( (_textureWidth!=0) && (_textureHeight!=0) && (_internalFormat!=0) )
+    {
+        _textureObjectBuffer[contextID] = textureObject = generateTextureObject(
+                contextID,GL_TEXTURE_2D,_numMipmapLevels,_internalFormat,_textureWidth,_textureHeight,1,0);
+        
+        textureObject->bind();
+
+        applyTexParameters(GL_TEXTURE_2D,state);
+
+        // no image present, but dimensions at set so lets create the texture
+        glTexImage2D( GL_TEXTURE_2D, 0, _internalFormat,
+                     _textureWidth, _textureHeight, _borderWidth,
+                     _sourceFormat ? _sourceFormat : _internalFormat,
+                     _sourceType ? _sourceType : GL_UNSIGNED_BYTE,
+                     0);                
+                     
+        if (_readPBuffer.valid())
+        {
+            _readPBuffer->bindPBufferToTexture(GL_FRONT);
+        }
+        
     }
     else
     {
