@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -240,6 +240,29 @@ void Texture3D::apply(State& state) const
             non_const_this->_image = 0;
         }
 
+    }
+    else if ( (_textureWidth!=0) && (_textureHeight!=0) && (_textureDepth!=0) && (_internalFormat!=0) )
+    {
+        _textureObjectBuffer[contextID] = textureObject = generateTextureObject(
+                contextID,GL_TEXTURE_3D,_numMipmapLevels,_internalFormat,_textureWidth,_textureHeight,_textureDepth,0);
+        
+        textureObject->bind();
+
+        applyTexParameters(GL_TEXTURE_3D,state);
+
+        // no image present, but dimensions at set so lets create the texture
+        extensions->glTexImage3D( GL_TEXTURE_3D, 0, _internalFormat,
+                     _textureWidth, _textureHeight, _textureDepth,
+                     _borderWidth,
+                     _sourceFormat ? _sourceFormat : _internalFormat,
+                     _sourceType ? _sourceType : GL_UNSIGNED_BYTE,
+                     0);                
+                     
+        if (_readPBuffer.valid())
+        {
+            _readPBuffer->bindPBufferToTexture(GL_FRONT);
+        }
+        
     }
     else
     {
@@ -416,7 +439,7 @@ void Texture3D::setExtensions(unsigned int contextID,Extensions* extensions)
 
 Texture3D::Extensions::Extensions(unsigned int contextID)
 {
-    setupGLExtenions(contextID);
+    setupGLExtensions(contextID);
 }
 
 Texture3D::Extensions::Extensions(const Extensions& rhs):
@@ -446,7 +469,7 @@ void Texture3D::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._gluBuild3DMipmaps)                    _gluBuild3DMipmaps = 0;
 }
 
-void Texture3D::Extensions::setupGLExtenions(unsigned int contextID)
+void Texture3D::Extensions::setupGLExtensions(unsigned int contextID)
 {
     _isTexture3DFast = isGLExtensionSupported(contextID,"GL_EXT_texture3D");
 
