@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -121,7 +121,9 @@ void TextureRectangle::setImage(Image* image)
 
 void TextureRectangle::apply(State& state) const
 {
-    static bool s_rectangleSupported = isGLExtensionSupported(state.getContextID(),"GL_EXT_texture_rectangle") || isGLExtensionSupported(state.getContextID(),"GL_NV_texture_rectangle");
+    static bool s_rectangleSupported = isGLExtensionSupported(state.getContextID(),"GL_ARB_texture_rectangle")
+            || isGLExtensionSupported(state.getContextID(),"GL_EXT_texture_rectangle")
+            || isGLExtensionSupported(state.getContextID(),"GL_NV_texture_rectangle");
 
     if (!s_rectangleSupported)
     {
@@ -193,6 +195,28 @@ void TextureRectangle::apply(State& state) const
             non_const_this->_image = 0;
         }
     }
+    else if ( (_textureWidth!=0) && (_textureHeight!=0) && (_internalFormat!=0) )
+    {
+        _textureObjectBuffer[contextID] = textureObject = generateTextureObject(
+                contextID,GL_TEXTURE_RECTANGLE,0,_internalFormat,_textureWidth,_textureHeight,1,0);
+        
+        textureObject->bind();
+
+        applyTexParameters(GL_TEXTURE_RECTANGLE,state);
+
+        // no image present, but dimensions at set so lets create the texture
+        glTexImage2D( GL_TEXTURE_RECTANGLE, 0, _internalFormat,
+                     _textureWidth, _textureHeight, _borderWidth,
+                     _sourceFormat ? _sourceFormat : _internalFormat,
+                     _sourceType ? _sourceType : GL_UNSIGNED_BYTE,
+                     0);                
+                     
+        if (_readPBuffer.valid())
+        {
+            _readPBuffer->bindPBufferToTexture(GL_FRONT);
+        }
+        
+    }
     else
     {
         glBindTexture(GL_TEXTURE_RECTANGLE, 0);
