@@ -5,6 +5,7 @@
 #include <osg/Timer>
 #include <osg/Texture>
 #include <osg/Notify>
+#include <osg/ApplicationUsage>
 
 #include <OpenThreads/ScopedLock>
 
@@ -19,6 +20,10 @@
 using namespace osgDB;
 using namespace OpenThreads;
 
+static osg::ApplicationUsageProxy DatabasePager_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_DO_PRE_COMPILE <ON/OFF>","Switch on or off the pre compile of OpenGL object database pager.");
+static osg::ApplicationUsageProxy DatabasePager_e1(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_MINIMUM_COMPILE_TIME_PER_FRAME <float>","minimum compile time alloted to compiling OpenGL objects per frame in database pager.");
+static osg::ApplicationUsageProxy DatabasePager_e2(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_MAXIMUM_OBJECTS_TO_COMPILE_PER_FRAME <int>","maximum number of OpenGL objects to compile per frame in database pager.");
+static osg::ApplicationUsageProxy DatabasePager_e3(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_DATABASE_PAGER_DRAWABLE <mode>","Set the drawable policy for setting of loaded drawable to specified type.  mode can be one of DoNotModify, DisplayList, VBO or VertexArrays>.");
 DatabasePager::DatabasePager()
 {
     //osg::notify(osg::INFO)<<"Constructing DatabasePager()"<<std::endl;
@@ -32,12 +37,42 @@ DatabasePager::DatabasePager()
     _useFrameBlock = false;
     _numFramesActive = 0;
     _frameNumber = 0;
-    _frameBlock = new Block;
-    _databasePagerThreadBlock = new Block;
+    _frameBlock = new osg::Block;
+    _databasePagerThreadBlock = new osg::Block;
 
     _threadPriorityDuringFrame = THREAD_PRIORITY_MIN;
     _threadPriorityOutwithFrame = THREAD_PRIORITY_MIN;
 
+#if __APPLE__
+    // OSX really doesn't like compiling display lists, and performs poorly when they are used,
+    // so apply this hack to make up for its short comings.
+    _drawablePolicy = USE_VERTEX_ARRAYS;
+#else
+    _drawablePolicy = DO_NOT_MODIFY_DRAWABLE_SETTINGS;
+#endif    
+    
+    const char* str = getenv("OSG_DATABASE_PAGER_GEOMETRY");
+    if (!str) str = getenv("OSG_DATABASE_PAGER_DRAWABLE");
+    if (str)
+    {
+        if (strcmp(str,"DoNotModify")==0)
+        {
+            _drawablePolicy = DO_NOT_MODIFY_DRAWABLE_SETTINGS;
+        }
+        else if (strcmp(str,"DisplayList")==0 || strcmp(str,"DL")==0)
+        {
+            _drawablePolicy = USE_DISPLAY_LISTS;
+        }
+        else if (strcmp(str,"VBO")==0)
+        {
+            _drawablePolicy = USE_VERTEX_BUFFER_OBJECTS;
+        }
+        else if (strcmp(str,"VertexArrays")==0 || strcmp(str,"VA")==0 )
+        {
+            _drawablePolicy = USE_VERTEX_ARRAYS;
+        } 
+    }
+
     _changeAutoUnRef = true;
     _valueAutoUnRef = true;
     _changeAnisotropy = false;
@@ -52,9 +87,26 @@ DatabasePager::DatabasePager()
     
     _expiryDelay = 10;
 
+    const char* ptr=0;
+    _doPreCompile = true;
+    if( (ptr = getenv("OSG_DO_PRE_COMPILE")) != 0)
+    {
+        _doPreCompile = strcmp(ptr,"yes")==0 || strcmp(ptr,"YES")==0 ||
+                        strcmp(ptr,"on")==0 || strcmp(ptr,"ON")==0;
+    }
+
     _targetFrameRate = 100.0;
     _minimumTimeAvailableForGLCompileAndDeletePerFrame = 0.001; // 1ms.
-    _maximumNumOfObjectsToCompilePerFrame = 8;
+    _maximumNumOfObjectsToCompilePerFrame = 4;
+    if( (ptr = getenv("OSG_MINIMUM_COMPILE_TIME_PER_FRAME")) != 0)
+    {
+        _minimumTimeAvailableForGLCompileAndDeletePerFrame = atof(ptr);
+    }
+
+    if( (ptr = getenv("OSG_MAXIMUM_OBJECTS_TO_COMPILE_PER_FRAME")) != 0)
+    {
+        _maximumNumOfObjectsToCompilePerFrame = atoi(ptr);
+    }
 
     // make sure a SharedStateManager exists.
     //osgDB::Registry::instance()->getOrCreateSharedStateManager();
@@ -266,11 +318,13 @@ class DatabasePager::FindCompileableGLObjectsVisitor : public osg::NodeVisitor
 public:
     FindCompileableGLObjectsVisitor(DatabasePager::DataToCompile& dataToCompile, 
                                bool changeAutoUnRef, bool valueAutoUnRef,
-                               bool changeAnisotropy, float valueAnisotropy):
-                        osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN),
-                        _dataToCompile(dataToCompile),
-                        _changeAutoUnRef(changeAutoUnRef), _valueAutoUnRef(valueAutoUnRef),
-                        _changeAnisotropy(changeAnisotropy), _valueAnisotropy(valueAnisotropy)
+                               bool changeAnisotropy, float valueAnisotropy,
+                               DatabasePager::DrawablePolicy drawablePolicy):
+            osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN),
+            _dataToCompile(dataToCompile),
+            _changeAutoUnRef(changeAutoUnRef), _valueAutoUnRef(valueAutoUnRef),
+            _changeAnisotropy(changeAnisotropy), _valueAnisotropy(valueAnisotropy),
+            _drawablePolicy(drawablePolicy)
     {
     }
     
@@ -323,12 +377,31 @@ public:
     {
         apply(drawable->getStateSet());
         
-        // drawable->setUseDisplayList(false);
-        // drawable->setUseVertexBufferObjects(true);
-
+        switch(_drawablePolicy)
+        {
+        case DatabasePager::DO_NOT_MODIFY_DRAWABLE_SETTINGS: 
+             // do nothing, leave settings as they came in from loaded database.
+             // osg::notify(osg::NOTICE)<<"DO_NOT_MODIFY_DRAWABLE_SETTINGS"<<std::endl;
+             break;
+        case DatabasePager::USE_DISPLAY_LISTS: 
+             drawable->setUseDisplayList(true);
+             drawable->setUseVertexBufferObjects(false);
+             break;
+        case DatabasePager::USE_VERTEX_BUFFER_OBJECTS:
+             drawable->setUseDisplayList(true);
+             drawable->setUseVertexBufferObjects(true);
+             // osg::notify(osg::NOTICE)<<"USE_VERTEX_BUFFER_OBJECTS"<<std::endl;
+             break;
+        case DatabasePager::USE_VERTEX_ARRAYS:
+             drawable->setUseDisplayList(false);
+             drawable->setUseVertexBufferObjects(false);
+             // osg::notify(osg::NOTICE)<<"USE_VERTEX_ARRAYS"<<std::endl;
+             break;
+        }
+        
         if (drawable->getUseDisplayList() || drawable->getUseVertexBufferObjects())
         {
-            //osg::notify(osg::INFO)<<"Found compilable drawable"<<std::endl;
+            // osg::notify(osg::NOTICE)<<"  Found compilable drawable"<<std::endl;
             _dataToCompile.second.push_back(drawable);
         }
     }
@@ -338,6 +411,7 @@ public:
     bool                            _valueAutoUnRef;
     bool                            _changeAnisotropy;
     float                           _valueAnisotropy;
+    DatabasePager::DrawablePolicy   _drawablePolicy;
 };
 
 
@@ -447,7 +521,7 @@ void DatabasePager::run()
                 
                 bool loadedObjectsNeedToBeCompiled = false;
 
-                if (databaseRequest->_loadedModel.valid() && !_activeGraphicsContexts.empty())
+                if (_doPreCompile && databaseRequest->_loadedModel.valid() && !_activeGraphicsContexts.empty())
                 {
                     // force a compute of the loaded model's bounding volume, so that when the subgraph
                     // merged with the main scene graph and large computeBound() isn't incurred.
@@ -462,7 +536,8 @@ void DatabasePager::run()
                     // find all the compileable rendering objects
                     FindCompileableGLObjectsVisitor frov(dtc, 
                                                          _changeAutoUnRef, _valueAutoUnRef,
-                                                         _changeAnisotropy, _valueAnisotropy);
+                                                         _changeAnisotropy, _valueAnisotropy,
+                                                         _drawablePolicy);
 
                     databaseRequest->_loadedModel->accept(frov);
 
@@ -838,7 +913,6 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
         };
 
         unsigned int numObjectsCompiled = 0;
-        unsigned int maxNumObjectsToCompile = 4;
 
         // while there are valid databaseRequest's in the to compile list and there is
         // sufficient time left compile each databaseRequest's stateset and drawables.
@@ -846,7 +920,7 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
         {
             DataToCompileMap& dcm = databaseRequest->_dataToCompileMap;
             DataToCompile& dtc = dcm[state.getContextID()];
-            if (!dtc.first.empty() && (elapsedTime+estimatedTextureDuration)<availableTime && numObjectsCompiled<maxNumObjectsToCompile)
+            if (!dtc.first.empty() && (elapsedTime+estimatedTextureDuration)<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame)
             {
 
                 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_PRIORITY, 1.0);
@@ -856,7 +930,7 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
                 //osg::notify(osg::INFO)<<"Compiling statesets"<<std::endl;
                 StateSetList::iterator itr=sslist.begin();
                 for(;
-                    itr!=sslist.end() && (elapsedTime+estimatedTextureDuration)<availableTime && numObjectsCompiled<maxNumObjectsToCompile;
+                    itr!=sslist.end() && (elapsedTime+estimatedTextureDuration)<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame;
                     ++itr)
                 {
                     //osg::notify(osg::INFO)<<"    Compiling stateset "<<(*itr).get()<<std::endl;
@@ -880,14 +954,14 @@ void DatabasePager::compileGLObjects(osg::State& state, double& availableTime)
                 sslist.erase(sslist.begin(),itr);
             }
 
-            if (!dtc.second.empty() && (elapsedTime+estimatedDrawableDuration)<availableTime && numObjectsCompiled<maxNumObjectsToCompile)
+            if (!dtc.second.empty() && (elapsedTime+estimatedDrawableDuration)<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame)
             {
                 // we have Drawable's to compile
                 //osg::notify(osg::INFO)<<"Compiling drawables"<<std::endl;
                 DrawableList& dwlist = dtc.second;
                 DrawableList::iterator itr=dwlist.begin();
                 for(;
-                    itr!=dwlist.end() && (elapsedTime+estimatedDrawableDuration)<availableTime && numObjectsCompiled<maxNumObjectsToCompile;
+                    itr!=dwlist.end() && (elapsedTime+estimatedDrawableDuration)<availableTime && numObjectsCompiled<_maximumNumOfObjectsToCompilePerFrame;
                     ++itr)
                 {
                     //osg::notify(osg::INFO)<<"    Compiling drawable "<<(*itr).get()<<std::endl;
