@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -107,6 +107,14 @@ bool osgDB::makeDirectory( const std::string &path )
     {
         std::string dir = paths.top();
  
+        #if defined(WIN32)
+            //catch drive name
+            if (dir.size() == 2 && dir.c_str()[1] == ':') {
+                paths.pop();
+                continue;
+            }
+        #endif
+
         if( mkdir( dir.c_str(), 0755 )< 0 )
         {
             osg::notify(osg::DEBUG_INFO) << "osgDB::makeDirectory(): "  << strerror(errno) << std::endl;
@@ -455,7 +463,85 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
 
     // #define COMPILE_COCOA_VERSION
     #define COMPILE_CARBON_VERSION
+    // WARNING: Cocoa version is currently untested.
+    #ifdef COMPILE_COCOA_VERSION
+        #include <Foundation/Foundation.h>
+    #endif
+    #ifdef COMPILE_CARBON_VERSION
+        #include <CoreServices/CoreServices.h>
+        #include <CoreFoundation/CoreFoundation.h>
+        #include <Carbon/Carbon.h>
+    #endif
+    #include <iostream>
     
+    // These functions are local to FileUtils.cpp and not exposed to the API
+    // returns the path string except for numToShorten directories stripped off the end
+    std::string GetShortenedPath(std::string path, int numToShorten)
+    {
+        unsigned int i = path.length() - 1;
+        if(path[i] == '/') i--;
+        while(i > 1 && numToShorten)
+        {
+            if(path[i] == '/')
+                numToShorten--;
+            i--;
+        }
+        return path.substr(0,i + 1);
+    }
+
+    // returns an absolute (POSIX on MacOS X) path from a CFURLRef
+    std::string GetPathFromCFURLRef(CFURLRef urlRef)
+    {
+        char buffer[1024];
+        std::string path;
+        if(CFURLGetFileSystemRepresentation(urlRef, true, (UInt8*)buffer, 1024))
+            path = std::string(buffer);
+        return path;
+    }
+
+    // returns the absolute path to the main bundle
+    std::string GetApplicationBundlePath(CFBundleRef mainBundle)
+    {
+        std::string path;
+        CFURLRef urlRef = CFBundleCopyBundleURL(mainBundle);
+        if(urlRef)
+        {
+            path = GetPathFromCFURLRef(urlRef);
+            CFRelease(urlRef); // docs say we are responsible for releasing CFURLRef
+        }
+        return path;
+        
+    }
+
+    std::string GetApplicationParentPath(CFBundleRef mainBundle)
+    {
+        return GetShortenedPath(GetApplicationBundlePath(mainBundle), 1);
+    }
+
+    std::string GetApplicationPluginsPath(CFBundleRef mainBundle)
+    {
+        std::string path;
+        CFURLRef urlRef = CFBundleCopyBuiltInPlugInsURL(mainBundle);
+        if(urlRef)
+        {
+            path = GetPathFromCFURLRef(urlRef);
+            CFRelease(urlRef);
+        }
+        return path;
+        
+    }
+
+    std::string GetApplicationResourcesPath(CFBundleRef mainBundle)
+    {
+        std::string path;
+        CFURLRef urlRef = CFBundleCopyResourcesDirectoryURL(mainBundle);
+        if(urlRef)
+        {
+            path = GetPathFromCFURLRef(urlRef);
+            CFRelease(urlRef);
+        }
+        return path;
+    }
 
     // The Cocoa version is about 10 lines of code.
     // The Carbon version is noticably longer.
@@ -471,7 +557,6 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
     // And of course, you will need to link against Cocoa
 
     #ifdef COMPILE_COCOA_VERSION
-    #include <Foundation/Foundation.h>
     // OS X has preferred locations for where PlugIns should be located.
     // This function will set this as the order to search:
     // YourProgram.app/Contents/PlugIns
@@ -546,18 +631,12 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
 
     #elif defined(COMPILE_CARBON_VERSION)
 
-
-    #include <CoreServices/CoreServices.h>
     // OS X has preferred locations for where PlugIns should be located.
     // This function will set this as the order to search:
     // YourProgram.app/Contents/PlugIns
     // ~/Library/Application Support/OpenSceneGraph/PlugIns
     // /Library/Application Support/OpenSceneGraph/PlugIns
     // /Network/Library/Application Support/OpenSceneGraph/PlugIns
-    // 
-    // As a side effect of this function, if the application is not a 
-    // bundle, the first place searched becomes
-    // YourProgram/PlugIns
     //
     // In principle, these other directories should be searched:
     // ~/Library/Application Support/YourProgram/PlugIns
@@ -583,67 +662,27 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
             convertStringPathIntoFilePathList(ptr, filepath);
         }
 
-        const int MAX_OSX_PATH_SIZE = 1024;
         const std::string OSG_PLUGIN_PATH("/OpenSceneGraph/PlugIns");
-        char buffer[MAX_OSX_PATH_SIZE];
-        char bundlePathBuffer[MAX_OSX_PATH_SIZE];
         CFURLRef  url;
-        CFStringRef pathString;
         CFBundleRef myBundle;
-        CFStringRef bundlePathString;
         FSRef     f;
-        OSErr	errCode;
+        OSErr    errCode;
 
         // Start with the the Bundle PlugIns directory.
-        // Unlike the Cocoa API, it seems that the PlugIn path is relative
-        // and not absolute. So I will need to fetch both the bundle path
-        // (which is absolute) and the PlugIn path (which is relative),
-        // and combine the two to form a full path.
 
-        // Get the bundle first
+        // Get the main bundle first. No need to retain or release it since
+        //  we are not keeping a reference
         myBundle = CFBundleGetMainBundle();
+        
         if(myBundle != NULL)
         {
-            // Get the URL to the bundle
-            url = CFBundleCopyBundleURL( myBundle );
-
-            // Convert the URL to a CFString that looks like a Unix file path
-            bundlePathString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
-            // Convert the CFString to a C string
-            CFStringGetCString( bundlePathString, bundlePathBuffer, MAX_OSX_PATH_SIZE, kCFStringEncodingUTF8 );
-
-            CFRelease( url );
-
-            // Now find the PlugIns directory
-            // Get the URL to the bundle
-            url = CFBundleCopyBuiltInPlugInsURL( myBundle );
-            //pathString = CFURLCopyPath( url );
-            // Convert the URL to a CFString that looks like a Unix file path
-            pathString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
-            // Convert the CFString to a C string
-            CFStringGetCString( pathString, buffer, MAX_OSX_PATH_SIZE, kCFStringEncodingUTF8 );
-
-            // Combine the string and copy it into the FilePath list
-            filepath.push_back( 
-                std::string(bundlePathBuffer) 
-                + std::string("/")
-                + std::string(buffer)
-            );
-
-            CFRelease( pathString );
-            CFRelease( bundlePathString );
-            CFRelease( url );
-
-            // I was getting random crashes which I believe were caused by
-            // releasing the bundle. The documentation says I'm responsible
-            // for retaining and releasing which didn't immediately register
-            // in my head. I never retain the bundle, so I don't release it.
-            // CFRelease( myBundle );
-
-            pathString = NULL;
-            bundlePathString = NULL;
-            url = NULL;
-            // myBundle = NULL;
+            // CFBundleGetMainBundle will return a bundle ref even if 
+            //  the application isn't part of a bundle, so we need to check
+            //  if the path to the bundle ends in ".app" to see if it is a
+            //  proper application bundle. If it is, the plugins path is added
+            std::string bundlePath = GetApplicationBundlePath(myBundle);
+            if( bundlePath.substr(bundlePath.length() - 4, 4) == std::string(".app") )
+                filepath.push_back(GetApplicationPluginsPath(myBundle));
         }
         else
         {
@@ -656,19 +695,15 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
         {
             // Get the URL
             url = CFURLCreateFromFSRef( 0, &f );
-            // Convert the URL to a CFString that looks like a Unix file path
-            pathString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
-            // Convert the CFString to a C string
-            CFStringGetCString( pathString, buffer, MAX_OSX_PATH_SIZE, kCFStringEncodingUTF8 );
+            if(url)
+            {
+                filepath.push_back(GetPathFromCFURLRef(url) + OSG_PLUGIN_PATH);
+                CFRelease( url );
+            }
+            else
+                osg::notify( osg::DEBUG_INFO ) << "Couldn't create CFURLRef for User's application support Path" << std::endl;
 
-            // Add the directory to the FilePathList
-            filepath.push_back( 
-                std::string(buffer) 
-                + OSG_PLUGIN_PATH
-            );
-
-            CFRelease( url );
-            CFRelease( pathString );
+            url = NULL;
         }
         else
         {
@@ -681,19 +716,16 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
         {
             // Get the URL
             url = CFURLCreateFromFSRef( 0, &f );
-            // Convert the URL to a CFString that looks like a Unix file path
-            pathString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
-            // Convert the CFString to a C string
-            CFStringGetCString( pathString, buffer, MAX_OSX_PATH_SIZE, kCFStringEncodingUTF8 );
+            
+            if(url)
+            {
+                filepath.push_back(GetPathFromCFURLRef(url) + OSG_PLUGIN_PATH);
+                CFRelease( url );
+            }
+            else
+                osg::notify( osg::DEBUG_INFO ) << "Couldn't create CFURLRef for local System's ApplicationSupport Path" << std::endl;
 
-            // Add the directory to the FilePathList
-            filepath.push_back( 
-                std::string(buffer) 
-                + OSG_PLUGIN_PATH
-            );
-
-            CFRelease( url );
-            CFRelease( pathString );
+            url = NULL;
         }
         else
         {
@@ -708,23 +740,20 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
         {
             // Get the URL
             url = CFURLCreateFromFSRef( 0, &f );
-            // Convert the URL to a CFString that looks like a Unix file path
-            pathString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
-            // Convert the CFString to a C string
-            CFStringGetCString( pathString, buffer, MAX_OSX_PATH_SIZE, kCFStringEncodingUTF8 );
+            
+            if(url)
+            {
+                filepath.push_back(GetPathFromCFURLRef(url) + OSG_PLUGIN_PATH);
+                CFRelease( url );
+            }
+            else
+                osg::notify( osg::DEBUG_INFO ) << "Couldn't create CFURLRef for network Application Support Path" << std::endl;
 
-            // Add the directory to the FilePathList
-            filepath.push_back( 
-                std::string(buffer) 
-                + OSG_PLUGIN_PATH
-            );
-
-            CFRelease( url );
-            CFRelease( pathString );
+            url = NULL;
         }
         else
         {
-	    // had to comment out as it segfauls the OSX app otherwise
+        // had to comment out as it segfauls the OSX app otherwise
             // osg::notify( osg::DEBUG_INFO ) << "Couldn't find the Network Application Support Path" << std::endl;
         }
     }
@@ -750,7 +779,12 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
             convertStringPathIntoFilePathList(ptr,filepath);
         }
 
+
+#if defined(__ia64__) || defined(__x86_64__)
+        convertStringPathIntoFilePathList("/usr/lib/:/usr/lib64/:/usr/local/lib/:/usr/local/lib64/",filepath);
+#else
         convertStringPathIntoFilePathList("/usr/lib/:/usr/local/lib/",filepath);
+#endif
 
     }
 
@@ -759,6 +793,38 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
 
 
 
+#ifdef __APPLE__
+    void osgDB::appendPlatformSpecificResourceFilePaths(FilePathList& filepath)
+    {
+        // Get the main application bundle
+        CFBundleRef mainBundle = CFBundleGetMainBundle();
+        
+        if (mainBundle != NULL) {
+            // Get the parent directory and the resources directory
+            std::string bundlePath = GetApplicationBundlePath(mainBundle);
+            std::string resourcesPath = GetApplicationResourcesPath(mainBundle);
+            
+            // check if application is really part of a .app bundle
+            if(bundlePath.substr(bundlePath.length() - 4, 4) == std::string(".app"))
+            {
+                if(resourcesPath != std::string(""))
+                    filepath.push_back( resourcesPath );
+                
+                std::string parentPath = GetShortenedPath(bundlePath, 1);
+                if(parentPath != std::string(""))
+                    filepath.push_back( parentPath );
+            }
+        }
+        else
+        {
+            osg::notify( osg::DEBUG_INFO ) << "Couldn't find the Application Bundle." << std::endl;
+        }
+    }
+#else
+    void osgDB::appendPlatformSpecificResourceFilePaths(FilePathList& /*filepath*/)
+    {
+    }
+#endif
 
 
 
