@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -119,13 +119,12 @@ void PrintFilePathList(std::ostream& stream,const FilePathList& filepath)
 
 Registry* Registry::instance(bool erase)
 {
-    static ref_ptr<Registry> s_nodeFactory = new Registry;
+    static ref_ptr<Registry> s_registry = new Registry;
     if (erase) 
     {
-        s_nodeFactory->closeAllLibraries();
-        s_nodeFactory = 0;
+        s_registry = 0;
     }
-    return s_nodeFactory.get(); // will return NULL on erase
+    return s_registry.get(); // will return NULL on erase
 }
 
 
@@ -141,6 +140,21 @@ Registry::Registry()
     initFilePathLists();
 
     // register file extension alias.
+    const char* flt_str = getenv("OSG_OPEN_FLIGHT_PLUGIN");
+    if (flt_str)
+    {
+        if (strcmp(flt_str, "new")==0)
+        {
+            addFileExtensionAlias("flt", "OpenFlight");
+        }
+    }
+    else
+    {
+    #ifndef COMPILE_WITH_OLD_OPENFLIGHT_PLUGIN_AS_DEFAULT
+        addFileExtensionAlias("flt", "OpenFlight");
+    #endif
+    }
+
     addFileExtensionAlias("sgi",  "rgb");
     addFileExtensionAlias("rgba", "rgb");
     addFileExtensionAlias("int",  "rgb");
@@ -150,7 +164,6 @@ Registry::Registry()
     addFileExtensionAlias("ivz",   "gz");
     addFileExtensionAlias("ozg",   "gz");
     
-
 #if defined(DARWIN_QUICKTIME)
     addFileExtensionAlias("jpg",  "qt");
     addFileExtensionAlias("jpe",  "qt");
@@ -167,6 +180,7 @@ Registry::Registry()
     addFileExtensionAlias("mpv",  "qt");
     addFileExtensionAlias("dv",   "qt");
     addFileExtensionAlias("mp4",   "qt");
+    addFileExtensionAlias("m4v",   "qt");
 #else
     addFileExtensionAlias("jpg",  "jpeg");
     addFileExtensionAlias("jpe",  "jpeg");
@@ -239,131 +253,38 @@ Registry::~Registry()
     closeAllLibraries();
 }
 
-#ifndef WIN32
+#if !defined(WIN32) || defined(__CYGWIN__)
 static osg::ApplicationUsageProxy Registry_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_FILE_PATH <path>[:path]..","Paths for locating datafiles");
 #else
 static osg::ApplicationUsageProxy Registry_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_FILE_PATH <path>[;path]..","Paths for locating datafiles");
 #endif
 
-#if defined(__APPLE__)
-
-//Executable packages should be able to load file resources from inside the packages.
-//These resources should be stored in YourProgram.app/Contents/Resources, and so
-//should the path list for the data files should include that path by default.
-#include <CoreServices/CoreServices.h>
-
-void Registry::initDataFilePathList()
-{
-  
-  FilePathList filepath;
-  
-  //
-  // set up data file paths
-  //
-  char *ptr;
-  
-  if( (ptr = getenv( "OSG_FILE_PATH" )) )
-  {
-    //notify(DEBUG_INFO) << "OSG_FILE_PATH("<<ptr<<")"<<std::endl;
-    convertStringPathIntoFilePathList(ptr, filepath);
-  }
-  else if( (ptr = getenv( "OSGFILEPATH" )) )
-  {
-    //notify(DEBUG_INFO) << "OSGFILEPATH("<<ptr<<")"<<std::endl;
-    convertStringPathIntoFilePathList(ptr, filepath);
-  }
-  
-
-  char* buffer = NULL;
-  char* bundlePathBuffer = NULL;
-  CFURLRef  url;
-  CFStringRef pathString;
-  CFBundleRef myBundle;
-  CFStringRef bundlePathString;
-  CFIndex strLen; 
-  CFIndex maxBufferSize;
-  
-  myBundle = CFBundleGetMainBundle();
-  if (myBundle != NULL) {
-    // Get the URL to the bundle
-    url = CFBundleCopyBundleURL( myBundle );
-    
-    // Convert the URL to a CFString that looks like a Unix file path
-    bundlePathString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
-    
-    // Convert the CFString to a C string
-    strLen = CFStringGetLength( bundlePathString );
-    maxBufferSize = CFStringGetMaximumSizeForEncoding( strLen, kCFStringEncodingUTF8 );
-    bundlePathBuffer = (char*) malloc(maxBufferSize);
-    CFStringGetCString( bundlePathString, bundlePathBuffer, maxBufferSize, kCFStringEncodingUTF8 );
-    
-    CFRelease( url );
-
-    // Now find the resources folder
-    url = CFBundleCopyResourcesDirectoryURL( myBundle );
-    //pathString = CFURLCopyPath( url );
-    // Convert the URL to a CFString that looks like a Unix file path
-    pathString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
-    
-    // Convert the CFString to a C string
-    strLen = CFStringGetLength( pathString );
-    maxBufferSize = CFStringGetMaximumSizeForEncoding( strLen, kCFStringEncodingUTF8 );
-    buffer = (char*) malloc(maxBufferSize);
-    CFStringGetCString( pathString, buffer, maxBufferSize, kCFStringEncodingUTF8 );
-    
-    // Combine the string and copy it into the FilePath list
-    filepath.push_back( std::string(bundlePathBuffer) 
-                         + std::string("/")
-                         + std::string(buffer)
-                         );
-    
-    CFRelease( pathString );
-    CFRelease( bundlePathString );
-    CFRelease( url );
-    
-    if (bundlePathBuffer != NULL) free(bundlePathBuffer);
-    if (buffer != NULL) free(buffer);
-    
-    pathString = NULL;
-    bundlePathString = NULL;
-    url = NULL;
-  }
-  else
-  {
-    osg::notify( osg::DEBUG_INFO ) << "Couldn't find the Application Bundle" << std::endl;
-  }
-  
-  setDataFilePathList(filepath);
-
-  //osg::notify(INFO)<<"Data FilePathList"<<std::endl;
-  //PrintFilePathList(osg::notify(INFO),getDataFilePathList());
-}
-
-#else
+#include <iostream>
 
 void Registry::initDataFilePathList()
 {
+    FilePathList filepath;
     //
     // set up data file paths
     //
     char *ptr;
+  
     if( (ptr = getenv( "OSG_FILE_PATH" )) )
     {
         //notify(DEBUG_INFO) << "OSG_FILE_PATH("<<ptr<<")"<<std::endl;
-        setDataFilePathList(ptr);
+        convertStringPathIntoFilePathList(ptr, filepath);
     }
     else if( (ptr = getenv( "OSGFILEPATH" )) )
     {
         //notify(DEBUG_INFO) << "OSGFILEPATH("<<ptr<<")"<<std::endl;
-        setDataFilePathList(ptr);
+        convertStringPathIntoFilePathList(ptr, filepath);
     }
 
-    //osg::notify(INFO)<<"Data FilePathList"<<std::endl;
-    //PrintFilePathList(osg::notify(INFO),getDataFilePathList());
+    osgDB::appendPlatformSpecificResourceFilePaths(filepath);
+    setDataFilePathList(filepath);
+    
 }
 
-#endif
-
 void Registry::setDataFilePathList(const std::string& paths)
 {
     _dataFilePath.clear(); 
@@ -398,9 +319,6 @@ void Registry::initLibraryFilePathList()
     
     appendPlatformSpecificLibraryFilePaths(_libraryFilePath);
 
-    //osg::notify(INFO)<<"Library FilePathList"<<std::endl;
-    //PrintFilePathList(osg::notify(INFO),getLibraryFilePathList());
-
 }
 
 
@@ -552,7 +470,7 @@ void Registry::removeReaderWriter(ReaderWriter* rw)
 
 void Registry::addFileExtensionAlias(const std::string mapExt, const std::string toExt)
 {
-    if (mapExt!=toExt) _extAliasMap[mapExt] = toExt;
+    _extAliasMap[mapExt] = toExt;
 }
 
 std::string Registry::createLibraryNameForFile(const std::string& fileName)
@@ -566,12 +484,12 @@ std::string Registry::createLibraryNameForExtension(const std::string& ext)
 {
 
     ExtensionAliasMap::iterator itr=_extAliasMap.find(ext);
-    if (itr!=_extAliasMap.end()) return createLibraryNameForExtension(itr->second);
+    if (itr!=_extAliasMap.end() && ext != itr->second) return createLibraryNameForExtension(itr->second);
 
 #ifdef OSG_JAVA_BUILD
-	static std::string prepend = "java";
+    static std::string prepend = "java";
 #else
-	static std::string prepend = "";
+    static std::string prepend = "";
 #endif
 
 #if defined(WIN32)
@@ -709,6 +627,7 @@ ReaderWriter* Registry::getReaderWriterForExtension(const std::string& ext)
 
 struct concrete_wrapper: basic_type_wrapper 
 {
+    virtual ~concrete_wrapper() {}
     concrete_wrapper(const osg::Object *myobj) : myobj_(myobj) {}
     bool matches(const osg::Object *proto) const
     {
@@ -1146,11 +1065,6 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
     // try composite name first
     DotOsgWrapperMap::iterator itr = _classNameWrapperMap.find(compositeName);
 
-    // composite name not found, try simple class name
-    if (itr == _classNameWrapperMap.end()) {
-        itr = _classNameWrapperMap.find(classname);
-    }
-
     if (itr==_classNameWrapperMap.end())
     {
         // first try the standard nodekit library.
@@ -1160,11 +1074,14 @@ bool Registry::writeObject(const osg::Object& obj,Output& fw)
         // otherwise try the osgdb_ plugin library.
         std::string pluginLibraryName = createLibraryNameForExtension(obj.libraryName());
         if (loadLibrary(pluginLibraryName)) return writeObject(obj,fw);
+
+        // otherwise try simple class name
+        if (itr == _classNameWrapperMap.end()) 
+            itr = _classNameWrapperMap.find(classname);
     }
-    else
-    {
-    
-    
+
+    if (itr!=_classNameWrapperMap.end())
+    { 
         DotOsgWrapper* wrapper = itr->second.get();
         const DotOsgWrapper::Associates& assoc = wrapper->getAssociates();
 
@@ -1410,16 +1327,18 @@ ReaderWriter::ReadResult Registry::read(const ReadFunctor& readFunctor)
             if (num_ERROR_IN_READING_FILE)
             {
                 osg::notify(osg::NOTICE)<<"Warning: error reading file \""<<readFunctor._filename<<"\""<<std::endl;
-                return NULL;
+                return ReaderWriter::ReadResult(ReaderWriter::ReadResult::ERROR_IN_READING_FILE);
             }
             else if (num_FILE_NOT_FOUND)
             {
                 osg::notify(osg::NOTICE)<<"Warning: could not find file \""<<readFunctor._filename<<"\""<<std::endl;
-                return NULL;
+                return ReaderWriter::ReadResult(ReaderWriter::ReadResult::FILE_NOT_FOUND);
             }
         }
     }
 
+    results.clear();
+
     // now look for a plug-in to load the file.
     std::string libraryName = createLibraryNameForFile(readFunctor._filename);
     if (loadLibrary(libraryName))
@@ -1432,11 +1351,40 @@ ReaderWriter::ReadResult Registry::read(const ReadFunctor& readFunctor)
         }
     }
     
-    if (results.empty())
+    if (!results.empty())
+    {
+        unsigned int num_FILE_NOT_HANDLED = 0;
+        unsigned int num_FILE_NOT_FOUND = 0;
+        unsigned int num_ERROR_IN_READING_FILE = 0;
+
+        for(Results::iterator ritr=results.begin();
+            ritr!=results.end();
+            ++ritr)
+        {
+            if (ritr->status()==ReaderWriter::ReadResult::FILE_NOT_HANDLED) ++num_FILE_NOT_HANDLED;
+            else if (ritr->status()==ReaderWriter::ReadResult::FILE_NOT_FOUND) ++num_FILE_NOT_FOUND;
+            else if (ritr->status()==ReaderWriter::ReadResult::ERROR_IN_READING_FILE) ++num_ERROR_IN_READING_FILE;
+        }
+        
+        if (num_FILE_NOT_HANDLED!=results.size())
+        {
+            // we've come across a file not found or error in reading file.
+            if (num_ERROR_IN_READING_FILE)
+            {
+                osg::notify(osg::NOTICE)<<"Warning: error reading file \""<<readFunctor._filename<<"\""<<std::endl;
+                return ReaderWriter::ReadResult(ReaderWriter::ReadResult::ERROR_IN_READING_FILE);
+            }
+            else if (num_FILE_NOT_FOUND)
+            {
+                osg::notify(osg::NOTICE)<<"Warning: could not find file \""<<readFunctor._filename<<"\""<<std::endl;
+                return ReaderWriter::ReadResult(ReaderWriter::ReadResult::FILE_NOT_FOUND);
+            }
+        }
+    }
+    else
     {
         return ReaderWriter::ReadResult("Warning: Could not find plugin to read objects from file \""+readFunctor._filename+"\".");
     }
-    
 
     return results.front();
 }
