@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -12,7 +12,7 @@
 */
 
 #if defined(_MSC_VER)
-	#pragma warning( disable : 4786 )
+    #pragma warning( disable : 4786 )
 #endif
 
 #include <osgGA/DriveManipulator>
@@ -341,7 +341,7 @@ bool DriveManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
 
         case(GUIEventAdapter::KEYDOWN):
         {
-            if (ea.getKey()==' ')
+            if (ea.getKey()==GUIEventAdapter::KEY_Space)
             {
                 flushMouseEventStack();
                 home(ea,us);
@@ -361,13 +361,15 @@ bool DriveManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
             }
 #ifdef KEYBOARD_PITCH
             else if (ea.getKey()==osgGA::GUIEventAdapter::KEY_Up ||
-                     ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Up)
+                     ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Up ||
+                     ea.getKey()=='9')
             {
                 _pitchUpKeyPressed = true;
                 return true;
             }
             else if (ea.getKey()==osgGA::GUIEventAdapter::KEY_Down ||
-                     ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Down)
+                     ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Down ||
+                     ea.getKey()=='6')
             {
                 _pitchDownKeyPressed = true;
                 return true;
@@ -380,13 +382,15 @@ bool DriveManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
         {
 #ifdef KEYBOARD_PITCH
             if (ea.getKey()==osgGA::GUIEventAdapter::KEY_Up ||
-                     ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Up)
+                     ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Up ||
+                     ea.getKey()=='9')
             {
                 _pitchUpKeyPressed = false;
                 return true;
             }
             else if (ea.getKey()==osgGA::GUIEventAdapter::KEY_Down ||
-                     ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Down)
+                     ea.getKey()==osgGA::GUIEventAdapter::KEY_KP_Down ||
+                     ea.getKey()=='6')
             {
                 _pitchDownKeyPressed = false;
                 return true;
@@ -482,7 +486,7 @@ bool DriveManipulator::calcMovement()
     // return if less then two events have been added.
     if (_ga_t0.get()==NULL || _ga_t1.get()==NULL) return false;
 
-    double dt = _ga_t0->time()-_ga_t1->time();
+    double dt = _ga_t0->getTime()-_ga_t1->getTime();
 
     if (dt<0.0f)
     {
