@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -17,7 +17,9 @@
 using namespace osg;
 using namespace osgGA;
 
-EventVisitor::EventVisitor():NodeVisitor(UPDATE_VISITOR,TRAVERSE_ALL_CHILDREN)
+EventVisitor::EventVisitor()
+:    NodeVisitor(EVENT_VISITOR,TRAVERSE_ACTIVE_CHILDREN),
+    _handled(false)
 {
 }
 
@@ -41,4 +43,5 @@ void EventVisitor::removeEvent(GUIEventAdapter* event)
 void EventVisitor::reset()
 {
     _events.clear();
+    _handled = false;
 }
