@@ -1,3 +1,16 @@
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+ *
+ * This library is open source and may be redistributed and/or modified under  
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * (at your option) any later version.  The full license is in LICENSE file
+ * included with this distribution, and on the openscenegraph.org website.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * OpenSceneGraph Public License for more details.
+*/
+
 #include <osgGA/FlightManipulator>
 #include <osg/Notify>
 
@@ -118,7 +131,8 @@ bool FlightManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
         }
 
         case(GUIEventAdapter::KEYDOWN):
-            if (ea.getKey()==' ')
+        {
+            if (ea.getKey()==GUIEventAdapter::KEY_Space)
             {
                 flushMouseEventStack();
                 home(ea,us);
@@ -137,7 +151,7 @@ bool FlightManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
                 return true;
             }
             return false;
-
+        }
         case(GUIEventAdapter::FRAME):
             addMouseEvent(ea);
             if (calcMovement()) us.requestRedraw();
@@ -220,7 +234,7 @@ bool FlightManipulator::calcMovement()
     if (_ga_t0.get()==NULL || _ga_t1.get()==NULL) return false;
 
 
-    double dt = _ga_t0->time()-_ga_t1->time();
+    double dt = _ga_t0->getTime()-_ga_t1->getTime();
 
     if (dt<0.0f)
     {
@@ -264,7 +278,7 @@ bool FlightManipulator::calcMovement()
     osg::Vec3d sv = lv^up;
     sv.normalize();
 
-    double pitch = -inDegrees(dy*75.0f*dt);
+    double pitch = -inDegrees(dy*50.0f*dt);
     double roll = inDegrees(dx*50.0f*dt);
 
     osg::Quat delta_rotate;
