@@ -1,3 +1,16 @@
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+ *
+ * This library is open source and may be redistributed and/or modified under  
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * (at your option) any later version.  The full license is in LICENSE file
+ * included with this distribution, and on the openscenegraph.org website.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * OpenSceneGraph Public License for more details.
+*/
+
 #include <osgGA/NodeTrackerManipulator>
 #include <osg/Quat>
 #include <osg/Notify>
@@ -44,6 +57,34 @@ NodeTrackerManipulator::~NodeTrackerManipulator()
 {
 }
 
+osg::NodePath NodeTrackerManipulator::getNodePath() const
+{
+    osg::NodePath nodePath;
+    for(ObserveredNodePath::const_iterator itr = _trackNodePath.begin();
+        itr != _trackNodePath.end();
+        ++itr)
+    {
+        nodePath.push_back(const_cast<osg::Node*>(itr->get()));
+    }
+    return nodePath;
+}
+
+bool NodeTrackerManipulator::validateNodePath() const
+{
+    for(ObserveredNodePath::const_iterator itr = _trackNodePath.begin();
+        itr != _trackNodePath.begin();
+        ++itr)
+    {
+        if (*itr==0) 
+        {
+            osg::notify(osg::NOTICE)<<"Warning: tracked node path has been invalidated by changes in the scene graph."<<std::endl;
+            const_cast<ObserveredNodePath&>(_trackNodePath).clear();
+            return false;
+        }
+    }
+    return true;
+}
+
 void NodeTrackerManipulator::setTrackerMode(TrackerMode mode)
 {
     _trackerMode = mode;
@@ -85,13 +126,21 @@ void NodeTrackerManipulator::setTrackNode(osg::Node* node)
 
     if (!cpp._nodePaths.empty())
     {
-        osg::notify(osg::INFO)<<"NodeTrackerManipulator::setTrackNode(Node*): Path set"<<std::endl;
-        _trackNodePath = cpp._nodePaths[0];
+        osg::notify(osg::INFO)<<"NodeTrackerManipulator::setTrackNode(Node*"<<node<<" "<<node->getName()<<"): Path set"<<std::endl;
+        _trackNodePath.clear();
+        setTrackNodePath( cpp._nodePaths[0] );
     }
     else
     {
         osg::notify(osg::NOTICE)<<"NodeTrackerManipulator::setTrackNode(Node*): Unable to set tracked node due to empty parental path."<<std::endl;
     }
+    
+    osg::notify(osg::NOTICE)<<"setTrackNode("<<node->getName()<<")"<<std::endl;
+    for(unsigned int i=0; i<_trackNodePath.size(); ++i)
+    {
+        osg::notify(osg::NOTICE)<<"  "<<_trackNodePath[i]->className()<<" '"<<_trackNodePath[i]->getName()<<"'"<<std::endl;
+    }
+
 }
 
 const osg::Node* NodeTrackerManipulator::getNode() const
@@ -141,7 +190,7 @@ void NodeTrackerManipulator::getUsage(osg::ApplicationUsage& usage) const
 {
     usage.addKeyboardMouseBinding("NodeTracker: Space","Reset the viewing position to home");
     usage.addKeyboardMouseBinding("NodeTracker: +","When in stereo, increase the fusion distance");
-    usage.addKeyboardMouseBinding("NodeTracker: -","When in stereo, reduse the fusion distance");
+    usage.addKeyboardMouseBinding("NodeTracker: -","When in stereo, reduce the fusion distance");
 }
 
 bool NodeTrackerManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
@@ -239,7 +288,7 @@ bool NodeTrackerManipulator::isMouseMoving()
     float dx = _ga_t0->getXnormalized()-_ga_t1->getXnormalized();
     float dy = _ga_t0->getYnormalized()-_ga_t1->getYnormalized();
     float len = sqrtf(dx*dx+dy*dy);
-    float dt = _ga_t0->time()-_ga_t1->time();
+    float dt = _ga_t0->getTime()-_ga_t1->getTime();
 
     return (len>dt*velocity);
 }
@@ -269,7 +318,7 @@ void NodeTrackerManipulator::computeNodeWorldToLocal(osg::Matrixd& worldToLocal)
 {
     if (validateNodePath())
     {
-        worldToLocal = osg::computeWorldToLocal(_trackNodePath);
+        worldToLocal = osg::computeWorldToLocal(getNodePath());
     }
 }
 
@@ -277,7 +326,7 @@ void NodeTrackerManipulator::computeNodeLocalToWorld(osg::Matrixd& localToWorld)
 {
     if (validateNodePath())
     {
-        localToWorld = osg::computeLocalToWorld(_trackNodePath);
+        localToWorld = osg::computeLocalToWorld(getNodePath());
     }
 
 }
