@@ -1,3 +1,16 @@
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
+ *
+ * This library is open source and may be redistributed and/or modified under  
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * (at your option) any later version.  The full license is in LICENSE file
+ * included with this distribution, and on the openscenegraph.org website.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * OpenSceneGraph Public License for more details.
+*/
+
 #include <osgGA/TerrainManipulator>
 #include <osg/Quat>
 #include <osg/Notify>
@@ -76,9 +89,9 @@ void TerrainManipulator::init(const GUIEventAdapter& ,GUIActionAdapter& )
 
 void TerrainManipulator::getUsage(osg::ApplicationUsage& usage) const
 {
-    usage.addKeyboardMouseBinding("Trackball: Space","Reset the viewing position to home");
-    usage.addKeyboardMouseBinding("Trackball: +","When in stereo, increase the fusion distance");
-    usage.addKeyboardMouseBinding("Trackball: -","When in stereo, reduse the fusion distance");
+    usage.addKeyboardMouseBinding("Terrain: Space","Reset the viewing position to home");
+    usage.addKeyboardMouseBinding("Terrain: +","When in stereo, increase the fusion distance");
+    usage.addKeyboardMouseBinding("Terrain: -","When in stereo, reduce the fusion distance");
 }
 
 bool TerrainManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
@@ -145,7 +158,7 @@ bool TerrainManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
         }
 
         case(GUIEventAdapter::KEYDOWN):
-            if (ea.getKey()==' ')
+            if (ea.getKey()== GUIEventAdapter::KEY_Space)
             {
                 flushMouseEventStack();
                 _thrown = false;
@@ -176,7 +189,7 @@ bool TerrainManipulator::isMouseMoving()
     float dx = _ga_t0->getXnormalized()-_ga_t1->getXnormalized();
     float dy = _ga_t0->getYnormalized()-_ga_t1->getYnormalized();
     float len = sqrtf(dx*dx+dy*dy);
-    float dt = _ga_t0->time()-_ga_t1->time();
+    float dt = _ga_t0->getTime()-_ga_t1->getTime();
 
     return (len>dt*velocity);
 }
