@@ -54,7 +54,7 @@ void TrackballManipulator::home(double /*currentTime*/)
 
 void TrackballManipulator::home(const GUIEventAdapter& ea ,GUIActionAdapter& us)
 {
-    home(ea.time());
+    home(ea.getTime());
     us.requestRedraw();
 }
 
@@ -69,7 +69,7 @@ void TrackballManipulator::getUsage(osg::ApplicationUsage& usage) const
 {
     usage.addKeyboardMouseBinding("Trackball: Space","Reset the viewing position to home");
     usage.addKeyboardMouseBinding("Trackball: +","When in stereo, increase the fusion distance");
-    usage.addKeyboardMouseBinding("Trackball: -","When in stereo, reduse the fusion distance");
+    usage.addKeyboardMouseBinding("Trackball: -","When in stereo, reduce the fusion distance");
 }
 
 bool TrackballManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us)
@@ -136,7 +136,7 @@ bool TrackballManipulator::handle(const GUIEventAdapter& ea,GUIActionAdapter& us
         }
 
         case(GUIEventAdapter::KEYDOWN):
-            if (ea.getKey()==' ')
+            if (ea.getKey()== GUIEventAdapter::KEY_Space)
             {
                 flushMouseEventStack();
                 _thrown = false;
@@ -167,7 +167,7 @@ bool TrackballManipulator::isMouseMoving()
     float dx = _ga_t0->getXnormalized()-_ga_t1->getXnormalized();
     float dy = _ga_t0->getYnormalized()-_ga_t1->getYnormalized();
     float len = sqrtf(dx*dx+dy*dy);
-    float dt = _ga_t0->time()-_ga_t1->time();
+    float dt = _ga_t0->getTime()-_ga_t1->getTime();
 
     return (len>dt*velocity);
 }
