@@ -305,11 +305,11 @@ osg::Node* ReaderWriter3DS::ReaderObject::processNode(StateSetMap drawStateMap,L
     for (p=node->childs; p!=0; p=p->next) {
         if (!group) {
             group =new osg::Group;
-	    if (strcmp(node->name, "$$$DUMMY") == 0) {
-		group->setName(node->data.object.instance);
-	    } else {
-            	group->setName(node->name);
-	    }
+            if (strcmp(node->name, "$$$DUMMY") == 0) {
+                group->setName(node->data.object.instance);
+            } else {
+                group->setName(node->name);
+            }
         }
         group->addChild(processNode(drawStateMap,f,p));
     }
@@ -570,7 +570,7 @@ osg::Drawable*   ReaderWriter3DS::ReaderObject::createDrawable(Lib3dsMesh *m,Fac
     // create normals.
     if (_usePerVertexNormals)
     {
-        osg::Vec3Array* osg_normals=osg_normals = new osg::Vec3Array(noVertex);
+        osg::Vec3Array* osg_normals = new osg::Vec3Array(noVertex);
         
         // initialize normal list to zero's.
         for (i=0; i<noVertex; ++i)
@@ -615,7 +615,7 @@ osg::Drawable*   ReaderWriter3DS::ReaderObject::createDrawable(Lib3dsMesh *m,Fac
         geom->setNormalBinding(osg::Geometry::BIND_PER_PRIMITIVE);
     }
     
-    osg::UByte4Array* osg_colors = new osg::UByte4Array(1);
+    osg::Vec4ubArray* osg_colors = new osg::Vec4ubArray(1);
     (*osg_colors)[0].set(255,255,255,255);
     geom->setColorArray(osg_colors);
     geom->setColorBinding(osg::Geometry::BIND_OVERALL);
@@ -692,7 +692,8 @@ osg::Texture2D*  ReaderWriter3DS::ReaderObject::createTexture(Lib3dsTextureMap *
 
         // what is the wrap mode of the texture.
         osg::Texture2D::WrapMode wm = ((texture->flags)&LIB3DS_NO_TILE) ?
-        osg::Texture2D::CLAMP : wm=osg::Texture2D::REPEAT;
+                osg::Texture2D::CLAMP :
+                osg::Texture2D::REPEAT;
         osg_texture->setWrap(osg::Texture2D::WRAP_S,wm);
         osg_texture->setWrap(osg::Texture2D::WRAP_T,wm);
         osg_texture->setWrap(osg::Texture2D::WRAP_R,wm);
