@@ -98,6 +98,15 @@ lwosg::Converter::Options ReaderWriterLWO::parse_options(const Options *options)
             if (opt == "FORCE_ARB_COMPRESSION")    conv_options.force_arb_compression = true;
             if (opt == "USE_OSGFX")                conv_options.use_osgfx = true;
             if (opt == "NO_LIGHTMODEL_ATTRIBUTE")  conv_options.apply_light_model = false;
+            if (opt == "BIND_TEXTURE_MAP")
+            {
+                std::string mapname;
+                int unit;
+                if (iss >> mapname >> unit)
+                {
+                    conv_options.texturemap_bindings.insert(std::make_pair(mapname,  unit));
+                }
+            }
             if (opt == "MAX_TEXTURE_UNITS") {
                 int n;
                 if (iss >> n) {
@@ -121,7 +130,7 @@ osgDB::ReaderWriter::ReadResult ReaderWriterLWO::readNode_LWO2(const std::string
     lwosg::Converter converter(conv_options, options);
     osg::ref_ptr<osg::Node> node = converter.convert(fileName);
     if (node.valid()) {
-        return node.take();
+        return node.release();
     }
 
     return ReadResult::FILE_NOT_HANDLED;
@@ -134,7 +143,7 @@ osgDB::ReaderWriter::ReadResult ReaderWriterLWO::readNode_old_LWO2(const std::st
     if (lwo2->ReadFile(fileName))
     {
         osg::ref_ptr<Group> group = new osg::Group();
-        if (lwo2->GenerateGroup(*group)) return group.take();
+        if (lwo2->GenerateGroup(*group)) return group.release();
     }
     return ReadResult::FILE_NOT_HANDLED;
 }
