@@ -48,7 +48,7 @@
 class ReaderWriterOBJ : public osgDB::ReaderWriter
 {
 public:
-    ReaderWriterOBJ() { }
+    ReaderWriterOBJ():_fixBlackMaterials(true) {}
 
     virtual const char* className() const { return "Wavefront OBJ Reader"; }
     virtual bool acceptsExtension(const std::string& extension) const {
@@ -66,21 +66,86 @@ protected:
     
     void buildMaterialToStateSetMap(obj::Model& model, MaterialToStateSetMap& materialToSetSetMap) const;
     
-    osg::Geometry* convertElementListToGeometry(obj::Model& model, obj::Model::ElementList& elementList) const;
+    osg::Geometry* convertElementListToGeometry(obj::Model& model, obj::Model::ElementList& elementList, bool& rotate) const;
     
-    osg::Node* convertModelToSceneGraph(obj::Model& model) const;
+    osg::Node* convertModelToSceneGraph(obj::Model& model, bool& rotate) const;
 
-    inline osg::Vec3 transformVertex(const osg::Vec3& vec) const { return osg::Vec3(vec.x(),-vec.z(),vec.y()); }
-    inline osg::Vec3 transformNormal(const osg::Vec3& vec) const { return osg::Vec3(vec.x(),-vec.z(),vec.y()); }
+    inline osg::Vec3 transformVertex(const osg::Vec3& vec, const bool rotate) const ;
+    inline osg::Vec3 transformNormal(const osg::Vec3& vec, const bool rotate) const ;
     
+    bool _fixBlackMaterials;
+
 };
 
+inline osg::Vec3 ReaderWriterOBJ::transformVertex(const osg::Vec3& vec, const bool rotate) const
+{
+    if(rotate==true)
+    {
+        return osg::Vec3(vec.x(),-vec.z(),vec.y());
+    }
+    else
+    {
+        return vec;
+    }
+}
+
+inline osg::Vec3 ReaderWriterOBJ::transformNormal(const osg::Vec3& vec, const bool rotate) const
+{
+    if(rotate==true)
+    {
+        return osg::Vec3(vec.x(),-vec.z(),vec.y());
+    }
+    else
+    {
+        return vec;
+    }
+}
+
 
 // register with Registry to instantiate the above reader/writer.
 osgDB::RegisterReaderWriterProxy<ReaderWriterOBJ> g_objReaderWriterProxy;
 
 void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToStateSetMap& materialToStateSetMap) const
 {
+    if (_fixBlackMaterials)
+    {
+        // hack to fix Maya exported models that contian all black materials.
+        int numBlack = 0;
+        int numNotBlack = 0;
+        obj::Model::MaterialMap::iterator itr;
+        for(itr = model.materialMap.begin();
+            itr != model.materialMap.end();
+            ++itr)
+        {
+            obj::Material& material = itr->second;
+            if (material.ambient==osg::Vec4(0.0f,0.0f,0.0f,1.0f) &&
+                material.diffuse==osg::Vec4(0.0f,0.0f,0.0f,1.0f))
+            {
+                ++numBlack;
+            }
+            else
+            {
+                ++numNotBlack;
+            }
+        }
+        
+        if (numNotBlack==0 && numBlack!=0)
+        {
+            for(itr = model.materialMap.begin();
+                itr != model.materialMap.end();
+                ++itr)
+            {
+                obj::Material& material = itr->second;
+                if (material.ambient==osg::Vec4(0.0f,0.0f,0.0f,1.0f) &&
+                    material.diffuse==osg::Vec4(0.0f,0.0f,0.0f,1.0f))
+                {
+                    material.ambient.set(0.3f,0.3f,0.3f,1.0f);
+                    material.diffuse.set(1.0f,1.0f,1.0f,1.0f);
+                }
+            }
+        }
+    }
+    
     for(obj::Model::MaterialMap::iterator itr = model.materialMap.begin();
         itr != model.materialMap.end();
         ++itr)
@@ -157,7 +222,7 @@ void ReaderWriterOBJ::buildMaterialToStateSetMap(obj::Model& model, MaterialToSt
     }
 }
 
-osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model, obj::Model::ElementList& elementList) const
+osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model, obj::Model::ElementList& elementList, bool& rotate) const
 {
     
     unsigned int numVertexIndices = 0;
@@ -235,7 +300,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                     index_itr != element.vertexIndices.end();
                     ++index_itr)
                 {
-                    vertices->push_back(transformVertex(model.vertices[*index_itr]));
+                    vertices->push_back(transformVertex(model.vertices[*index_itr],rotate));
                     ++numPoints;
                 }
                 if (numNormalIndices)
@@ -244,7 +309,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                         index_itr != element.normalIndices.end();
                         ++index_itr)
                     {
-                        normals->push_back(transformNormal(model.normals[*index_itr]));
+                        normals->push_back(transformNormal(model.normals[*index_itr],rotate));
                     }
                 }
                 if (numTexCoordIndices)
@@ -281,7 +346,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                     index_itr != element.vertexIndices.end();
                     ++index_itr)
                 {
-                    vertices->push_back(transformVertex(model.vertices[*index_itr]));
+                    vertices->push_back(transformVertex(model.vertices[*index_itr],rotate));
                 }
                 if (numNormalIndices)
                 {
@@ -289,7 +354,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                         index_itr != element.normalIndices.end();
                         ++index_itr)
                     {
-                        normals->push_back(transformNormal(model.normals[*index_itr]));
+                        normals->push_back(transformNormal(model.normals[*index_itr],rotate));
                     }
                 }
                 if (numTexCoordIndices)
@@ -352,7 +417,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                         index_itr != element.vertexIndices.rend();
                         ++index_itr)
                     {
-                        vertices->push_back(transformVertex(model.vertices[*index_itr]));
+                        vertices->push_back(transformVertex(model.vertices[*index_itr],rotate));
                     }
                     if (numNormalIndices)
                     {
@@ -360,7 +425,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                             index_itr != element.normalIndices.rend();
                             ++index_itr)
                         {
-                            normals->push_back(transformNormal(model.normals[*index_itr]));
+                            normals->push_back(transformNormal(model.normals[*index_itr],rotate));
                         }
                     }
                     if (numTexCoordIndices)
@@ -380,7 +445,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                         index_itr != element.vertexIndices.end();
                         ++index_itr)
                     {
-                        vertices->push_back(transformVertex(model.vertices[*index_itr]));
+                        vertices->push_back(transformVertex(model.vertices[*index_itr],rotate));
                     }
                     if (numNormalIndices)
                     {
@@ -388,7 +453,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
                             index_itr != element.normalIndices.end();
                             ++index_itr)
                         {
-                            normals->push_back(transformNormal(model.normals[*index_itr]));
+                            normals->push_back(transformNormal(model.normals[*index_itr],rotate));
                         }
                     }
                     if (numTexCoordIndices)
@@ -410,7 +475,7 @@ osg::Geometry* ReaderWriterOBJ::convertElementListToGeometry(obj::Model& model,
     return geometry;
 }
 
-osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model) const
+osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model, bool& rotate) const
 {
 
     if (model.elementStateMap.empty()) return 0;
@@ -430,7 +495,7 @@ osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model) const
         const obj::ElementState& es = itr->first;
         obj::Model::ElementList& el = itr->second;
 
-        osg::Geometry* geometry = convertElementListToGeometry(model,el);
+        osg::Geometry* geometry = convertElementListToGeometry(model,el,rotate);
 
         if (geometry)
         {
@@ -453,9 +518,22 @@ osg::Node* ReaderWriterOBJ::convertModelToSceneGraph(obj::Model& model) const
                 tsv.smooth(*geometry);
             }
 
+
             osg::Geode* geode = new osg::Geode;
             geode->addDrawable(geometry);
-            geode->setName(es.objectName);
+            
+            if (es.objectName.empty())
+            {
+                geode->setName(es.groupName);
+            }
+            else if (es.groupName.empty())
+            {
+                geode->setName(es.objectName);
+            }
+            else
+            {
+                geode->setName(es.groupName + std::string(":") + es.objectName);
+            }
 
             group->addChild(geode);
 
@@ -484,12 +562,18 @@ osgDB::ReaderWriter::ReadResult ReaderWriterOBJ::readNode(const std::string& fil
         osg::ref_ptr<Options> local_opt = options ? static_cast<Options*>(options->clone(osg::CopyOp::SHALLOW_COPY)) : new Options;
         local_opt->setDatabasePath(osgDB::getFilePath(fileName));
 
-
         obj::Model model;
         model.setDatabasePath(osgDB::getFilePath(fileName.c_str()));
         model.readOBJ(fin, local_opt.get());
         
-        osg::Node* node = convertModelToSceneGraph(model);
+        // code for checking the nonRotation
+        bool rotate = true;
+        if ((options!=NULL) && (options->getOptionString() == "noRotation"))
+        {
+            rotate = false;
+        }
+
+        osg::Node* node = convertModelToSceneGraph(model,rotate);
         return node;
     }
     
@@ -503,7 +587,14 @@ osgDB::ReaderWriter::ReadResult ReaderWriterOBJ::readNode(std::istream& fin, con
         obj::Model model;
         model.readOBJ(fin, options);
         
-        osg::Node* node = convertModelToSceneGraph(model);
+        // code for checking the nonRotation
+        bool rotate = true;
+        if ((options!=NULL) && (options->getOptionString() == "noRotation"))
+        {
+            rotate = false;
+        }
+
+        osg::Node* node = convertModelToSceneGraph(model,rotate);
         return node;
     }
     
