@@ -173,7 +173,7 @@ bool Geometry_readLocalData(Object& obj, Input& fr)
         {
             // post 0.9.3 releases.
             ++fr;
-            Vec3Array* normals = dynamic_cast<Vec3Array*>(Array_readLocalData(fr));
+            Array* normals = Array_readLocalData(fr);
             if (normals)
             {
                 geom.setNormalArray(normals);
@@ -513,9 +513,9 @@ Array* Array_readLocalData(Input& fr)
         ++fr;
         return_array = array;
     }
-    else if (strcmp(arrayName,"UByte4Array")==0)
+    else if (strcmp(arrayName,"UVec4bArray")==0 || strcmp(arrayName,"Vec4ubArray")==0)
     {
-        UByte4Array* array = new UByte4Array;
+        Vec4ubArray* array = new Vec4ubArray;
         array->reserve(capacity);
         while (!fr.eof() && fr[0].getNoNestedBrackets()>entry)
         {
@@ -526,7 +526,7 @@ Array* Array_readLocalData(Input& fr)
                 fr[3].getUInt(a))
             {
                 fr+=4;
-                array->push_back(osg::UByte4(r,g,b,a));
+                array->push_back(osg::Vec4ub(r,g,b,a));
             }
             else ++fr;
         }
@@ -601,6 +601,120 @@ Array* Array_readLocalData(Input& fr)
         ++fr;
         return_array = array;
     }
+    else if (strcmp(arrayName,"Vec2bArray")==0)
+    {
+        Vec2bArray* array = new Vec2bArray;
+        array->reserve(capacity);
+        while (!fr.eof() && fr[0].getNoNestedBrackets()>entry)
+        {
+            unsigned int r,g;
+            if (fr[0].getUInt(r) &&
+                fr[1].getUInt(g))
+            {
+                fr+=2;
+                array->push_back(osg::Vec2b(r,g));
+            }
+            else ++fr;
+        }
+        ++fr;
+        return_array = array;
+    }
+    else if (strcmp(arrayName,"Vec3bArray")==0)
+    {
+        Vec3bArray* array = new Vec3bArray;
+        array->reserve(capacity);
+        while (!fr.eof() && fr[0].getNoNestedBrackets()>entry)
+        {
+            unsigned int r,g,b;
+            if (fr[0].getUInt(r) &&
+                fr[1].getUInt(g) &&
+                fr[2].getUInt(b))
+            {
+                fr+=3;
+                array->push_back(osg::Vec3b(r,g,b));
+            }
+            else ++fr;
+        }
+        ++fr;
+        return_array = array;
+    }
+    else if (strcmp(arrayName,"Vec4bArray")==0)
+    {
+        Vec4bArray* array = new Vec4bArray;
+        array->reserve(capacity);
+        while (!fr.eof() && fr[0].getNoNestedBrackets()>entry)
+        {
+            unsigned int r,g,b,a;
+            if (fr[0].getUInt(r) &&
+                fr[1].getUInt(g) &&
+                fr[2].getUInt(b) &&
+                fr[3].getUInt(a))
+            {
+                fr+=4;
+                array->push_back(osg::Vec4b(r,g,b,a));
+            }
+            else ++fr;
+        }
+        ++fr;
+        return_array = array;
+    }
+    else if (strcmp(arrayName,"Vec2sArray")==0)
+    {
+        Vec2sArray* array = new Vec2sArray;
+        array->reserve(capacity);
+        while (!fr.eof() && fr[0].getNoNestedBrackets()>entry)
+        {
+            unsigned int r,g;
+            if (fr[0].getUInt(r) &&
+                fr[1].getUInt(g))
+            {
+                fr+=2;
+                array->push_back(osg::Vec2s(r,g));
+            }
+            else ++fr;
+        }
+        ++fr;
+        return_array = array;
+    }
+    else if (strcmp(arrayName,"Vec3sArray")==0)
+    {
+        Vec3sArray* array = new Vec3sArray;
+        array->reserve(capacity);
+        while (!fr.eof() && fr[0].getNoNestedBrackets()>entry)
+        {
+            unsigned int r,g,b;
+            if (fr[0].getUInt(r) &&
+                fr[1].getUInt(g) &&
+                fr[2].getUInt(b))
+            {
+                fr+=3;
+                array->push_back(osg::Vec3s(r,g,b));
+            }
+            else ++fr;
+        }
+        ++fr;
+        return_array = array;
+    }
+    else if (strcmp(arrayName,"Vec4sArray")==0)
+    {
+        Vec4sArray* array = new Vec4sArray;
+        array->reserve(capacity);
+        while (!fr.eof() && fr[0].getNoNestedBrackets()>entry)
+        {
+            unsigned int r,g,b,a;
+            if (fr[0].getUInt(r) &&
+                fr[1].getUInt(g) &&
+                fr[2].getUInt(b) &&
+                fr[3].getUInt(a))
+            {
+                fr+=4;
+                array->push_back(osg::Vec4s(r,g,b,a));
+            }
+            else ++fr;
+        }
+        ++fr;
+        return_array = array;
+    }
     
     if (return_array) 
     {
@@ -681,9 +795,9 @@ bool Array_writeLocalData(const Array& array,Output& fw)
                 return true;
             }
             break;
-        case(Array::UByte4ArrayType):
+        case(Array::Vec4ubArrayType):
             {
-                const UByte4Array& carray = static_cast<const UByte4Array&>(array);
+                const Vec4ubArray& carray = static_cast<const Vec4ubArray&>(array);
                 fw<<array.className()<<" "<<carray.size()<<" ";
                 writeArray(fw,carray.begin(),carray.end(),1);
                 return true;
@@ -721,6 +835,54 @@ bool Array_writeLocalData(const Array& array,Output& fw)
                 return true;
             }
             break;
+        case(Array::Vec2sArrayType):
+            {
+                const Vec2sArray& carray = static_cast<const Vec2sArray&>(array);
+                fw<<array.className()<<" "<<carray.size()<<std::endl;
+                writeArray(fw,carray.begin(),carray.end(), 3);
+                return true;
+            }
+            break;
+        case(Array::Vec3sArrayType):
+            {
+                const Vec3sArray& carray = static_cast<const Vec3sArray&>(array);
+                fw<<array.className()<<" "<<carray.size()<<std::endl;
+                writeArray(fw,carray.begin(),carray.end(), 2);
+                return true;
+            }
+            break;
+        case(Array::Vec4sArrayType):
+            {
+                const Vec4sArray& carray = static_cast<const Vec4sArray&>(array);
+                fw<<array.className()<<" "<<carray.size()<<std::endl;
+                writeArray(fw,carray.begin(),carray.end(), 1);
+                return true;
+            }
+            break;
+        case(Array::Vec2bArrayType):
+            {
+                const Vec2bArray& carray = static_cast<const Vec2bArray&>(array);
+                fw<<array.className()<<" "<<carray.size()<<" ";
+                writeArray(fw,carray.begin(),carray.end(),1);
+                return true;
+            }
+            break;
+        case(Array::Vec3bArrayType):
+            {
+                const Vec3bArray& carray = static_cast<const Vec3bArray&>(array);
+                fw<<array.className()<<" "<<carray.size()<<" ";
+                writeArray(fw,carray.begin(),carray.end(),1);
+                return true;
+            }
+            break;
+        case(Array::Vec4bArrayType):
+            {
+                const Vec4bArray& carray = static_cast<const Vec4bArray&>(array);
+                fw<<array.className()<<" "<<carray.size()<<" ";
+                writeArray(fw,carray.begin(),carray.end(),1);
+                return true;
+            }
+            break;
         case(Array::ArrayType):
         default:
             return false;
