@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -22,6 +22,12 @@
 #include <osgDB/FileUtils>
 
 #include <osgProducer/OsgCameraGroup>
+#include <Producer/CameraConfig>
+
+#ifdef WIN32
+#define        WGL_SAMPLE_BUFFERS_ARB        0x2041
+#define        WGL_SAMPLES_ARB                0x2042
+#endif
 
 using namespace Producer;
 using namespace osgProducer;
@@ -93,6 +99,13 @@ std::string extractCameraConfigFile(osg::ArgumentParser& arguments)
 
 static osg::ApplicationUsageProxy OsgCameraGroup_e1(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"PRODUCER_CAMERA_CONFIG_FILE <filename>","specify the default producer camera config to use when opening osgProducer based applications.");
 
+static osg::ApplicationUsageProxy OsgCameraGroup_e4(
+    osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,
+    "PRODUCER_CAMERA_BLOCK_ON_VSYNC", 
+    "After the CPU finishes generating the scene for one frame,"
+    " block the CPU until the start of the next frame. Same as pressing 'v'"
+    " within an osgProducer-based application.");
+
 
 OsgCameraGroup::OsgCameraGroup() : Producer::CameraGroup() 
 {
@@ -134,33 +147,33 @@ class QuitImageStreamVisitor : public osg::NodeVisitor
     public:
 
         QuitImageStreamVisitor():
-		osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN) {}
+            osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN) {}
        
 
         /** Simply traverse using standard NodeVisitor traverse method.*/
         virtual void apply(osg::Node& node)
-	{
-	    if (node.getStateSet())
-        	apply(*(node.getStateSet()));
-
-	    traverse(node);
-	}
-        
-        virtual void apply(osg::Geode& node)
-	{
-	    if (node.getStateSet())
+        {
+            if (node.getStateSet())
                 apply(*(node.getStateSet()));
 
-	    for(unsigned int i=0;i<node.getNumDrawables();++i)
-	    {
-        	osg::Drawable* drawable = node.getDrawable(i);
-        	if (drawable && drawable->getStateSet())
+            traverse(node);
+        }
+
+        virtual void apply(osg::Geode& node)
+        {
+            if (node.getStateSet())
+                apply(*(node.getStateSet()));
+
+            for(unsigned int i=0;i<node.getNumDrawables();++i)
+            {
+                osg::Drawable* drawable = node.getDrawable(i);
+                if (drawable && drawable->getStateSet())
                     apply(*(drawable->getStateSet()));
-	    }
-	}
-	
+            }
+        }
+
         void apply(osg::StateSet& stateset)
-	{
+        {
             for(unsigned int i=0;i<stateset.getTextureAttributeList().size();++i)
             {
                 osg::StateAttribute* texture = stateset.getTextureAttribute(i,osg::StateAttribute::TEXTURE);
@@ -168,26 +181,26 @@ class QuitImageStreamVisitor : public osg::NodeVisitor
                 {
                     osg::TextureRectangle* textureRect = dynamic_cast<osg::TextureRectangle*>(texture);
                     if (textureRect)
-		    {
-		        osg::ImageStream* imageStream = dynamic_cast<osg::ImageStream*>(textureRect->getImage());
-		        if (imageStream)
-		        {
-		    	    imageStream->quit();
-		        }
-		    }
+                    {
+                        osg::ImageStream* imageStream = dynamic_cast<osg::ImageStream*>(textureRect->getImage());
+                        if (imageStream)
+                        {
+                            imageStream->quit();
+                        }
+                    }
 
                     osg::Texture2D* texture2D = dynamic_cast<osg::Texture2D*>(texture);
                     if (texture2D)
-		    {
-		        osg::ImageStream* imageStream = dynamic_cast<osg::ImageStream*>(texture2D->getImage());
-		        if (imageStream)
-		        {
-		    	    imageStream->quit();
-		        }
-		    }
+                    {
+                        osg::ImageStream* imageStream = dynamic_cast<osg::ImageStream*>(texture2D->getImage());
+                        if (imageStream)
+                        {
+                            imageStream->quit();
+                        }
+                    }
                 }
-	    }
-	}
+            }
+        }
 
 };
 
@@ -206,22 +219,66 @@ OsgCameraGroup::~OsgCameraGroup()
 }
 
 
+static osg::ApplicationUsageProxy OsgCameraGroup_e2(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_CAMERA_THREADING <value>","Set the threading model using by osgProducer::Viewer/OsgCameraGroup based apps. <value> can be SingleThreaded or ThreadPerCamera");
+static osg::ApplicationUsageProxy OsgCameraGroup_e3(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_SHARE_GRAPHICS_CONTEXTS <value>","Set whether to share graphics contexts. <value> can be ON or OFF");
+
 void OsgCameraGroup::_init()
 {
+    if (_cfg.valid())
+    {
+        // By default select ThreadPerCamera when we have multiple cameras.
+        _threadModel = (_cfg->getNumberOfCameras()>1) ? ThreadPerCamera : SingleThreaded;
+    }
+
     const char* str = getenv("OSG_CAMERA_THREADING");
     if (str)
     {
-        if (strcmp(str,"SingleThreaded")==0) _thread_model = SingleThreaded;
-        else if (strcmp(str,"ThreadPerCamera")==0) _thread_model = ThreadPerCamera;
+        if (strcmp(str,"SingleThreaded")==0) _threadModel = SingleThreaded;
+        else if (strcmp(str,"ThreadPerCamera")==0) _threadModel = ThreadPerCamera;
     }
     
+
+    // work out how many graphics context we have    
+    std::set<Producer::RenderSurface*> renderSurfaces;
+    for(unsigned int i=0; i<getNumberOfCameras(); ++i)
+    {
+        if (getCamera(i) && getCamera(i)->getRenderSurface())
+        {
+            renderSurfaces.insert(getCamera(i)->getRenderSurface());
+        }
+    }
+    unsigned int numContexts = renderSurfaces.size();
+    
     str = getenv("OSG_SHARE_GRAPHICS_CONTEXTS");
     if (str)
     {
-        if (strcmp(str,"ON")==0) Producer::RenderSurface::shareAllGLContexts(true);
-        else if (strcmp(str,"OFF")==0) Producer::RenderSurface::shareAllGLContexts(false);
+        if (strcmp(str,"ON")==0) 
+        {
+            Producer::RenderSurface::shareAllGLContexts(true);
+
+            // if we are sharing graphics contexts then we just treat them all as one.
+            numContexts = 1;
+        }
+        else if (strcmp(str,"OFF")==0) 
+        {
+            Producer::RenderSurface::shareAllGLContexts(false);
+        }
+        
     }
-    
+
+    // initialize the number of graphics contexts to set up if there arn't already enough.
+    if (osg::DisplaySettings::instance()->getMaxNumberOfGraphicsContexts() < numContexts)
+    {    
+        osg::DisplaySettings::instance()->setMaxNumberOfGraphicsContexts(numContexts);
+    }
+
+
+    if ((_threadModel==ThreadPerCamera || _threadModel==ThreadPerRenderSurface) && _cfg->getNumberOfCameras()>1)
+    {
+        // switch on thread safe reference counting by default when running multi-threaded.
+        osg::Referenced::setThreadSafeReferenceCounting(true);
+    }
+
 
     _scene_data = NULL;
     _global_stateset = NULL;
@@ -239,10 +296,6 @@ void OsgCameraGroup::_init()
     _start_tick = _timer.tick();
 
     if (!_frameStamp) _frameStamp = new osg::FrameStamp;
-
-    // set up the maximum number of graphics contexts, before loading the scene graph
-    // to ensure that texture objects and display buffers are configured to the correct size.
-    osg::DisplaySettings::instance()->setMaxNumberOfGraphicsContexts( getNumberOfCameras() );
     
     _applicationUsage = osg::ApplicationUsage::instance();
 
@@ -368,7 +421,10 @@ void OsgCameraGroup::advance()
 bool OsgCameraGroup::realize( ThreadingModel thread_model )
 {
     if( _realized ) return _realized;
-    _thread_model = thread_model;
+    
+    if (_cfg.valid()) _cfg->setThreadModelDirective(thread_model);
+    _threadModel = thread_model;
+    
     return realize();
 }
 
@@ -388,7 +444,7 @@ public:
     
     virtual void apply(osg::Node& node)
     {
-        if (strcmp(node.libraryName(),"osgParticle")==0) _foundParticles = true;
+        if (strcmp(node.libraryName(),"osgParticle")==0 && strcmp(node.className(),"PrecipitationEffect")!=0) _foundParticles = true;
         
         if (!_foundParticles ||
             !_foundPagedLOD) traverse(node);
@@ -410,8 +466,6 @@ bool OsgCameraGroup::realize()
 
     if (!_ds) _ds = osg::DisplaySettings::instance();
 
-    _ds->setMaxNumberOfGraphicsContexts( _cfg->getNumberOfCameras() );
-    
     _shvec.clear();
     
     osg::Node* node = getTopMostSceneData();
@@ -422,13 +476,6 @@ bool OsgCameraGroup::realize()
         drv.setDisplaySettings(_ds.get());
         node->accept(drv);
     }
-    
-    unsigned int numMultiSamples = 0;
-
-    #ifdef __sgi
-    // switch on anti-aliasing by default, just in case we have an Onyx :-)
-    numMultiSamples = 4;
-    #endif
 
     // set up each render stage to clear the appropriate buffers.
     GLbitfield clear_mask=0;
@@ -440,6 +487,8 @@ bool OsgCameraGroup::realize()
     // use a std::map to keep track of what render surfaces are associated with what state.
     typedef std::map<Producer::RenderSurface*,osg::State*> RenderSurfaceStateMap;
     RenderSurfaceStateMap _renderSurfaceStateMap;
+
+    bool needToCreatedNewContextID = true;
     unsigned int contextID = 0;
 
     for(unsigned int i = 0; i < _cfg->getNumberOfCameras(); i++ )
@@ -461,11 +510,21 @@ bool OsgCameraGroup::realize()
         if (_renderSurfaceStateMap.count(rs)==0)
         {
             _renderSurfaceStateMap[rs] = sv->getState();
+
+            if (needToCreatedNewContextID)
+            {
+                // create a unique contextID for this graphics context.
+                contextID = osg::GraphicsContext::createNewContextID();
+                
+                // if we are sharing our graphics context then when needn't create any more.
+                needToCreatedNewContextID = !Producer::RenderSurface::allGLContextsAreShared();
+            }
+            else
+            {
+                osg::GraphicsContext::incrementContextIDUsageCount(contextID);
+            }
+
             sv->getState()->setContextID(contextID);
-            
-            // if we aren't share OpenGL objects between graphics contexts then need to increment the contextID
-            // to ensure that the OSG generates seperate objects for each graphics context. 
-            if (!Producer::RenderSurface::allGLContextsAreShared()) ++contextID;
         }
         else
         {
@@ -483,7 +542,7 @@ bool OsgCameraGroup::realize()
         rs->setRealizeCallback( new RenderSurfaceRealizeCallback(this, sh));
         
         // set up the visual chooser.
-        if (_ds.valid() || numMultiSamples!=0)
+        if (_ds.valid())
         {
         
             Producer::VisualChooser* rs_vc = rs->getVisualChooser();
@@ -493,24 +552,64 @@ bool OsgCameraGroup::realize()
                 rs_vc->setSimpleConfiguration();
                 rs->setVisualChooser(rs_vc);
             }
-            if (_ds->getStereo() && _ds->getStereoMode()==osg::DisplaySettings::QUAD_BUFFER) rs_vc->useStereo();
-            if (_ds->getStencilBuffer()) rs_vc->setStencilSize(_ds->getMinimumNumStencilBits());
+
+            unsigned int numStencilBits = 0;
+            if (_ds->getStereo())
+            {
+                switch(_ds->getStereoMode())
+                {
+                    case(osg::DisplaySettings::QUAD_BUFFER): 
+                        rs_vc->useStereo();
+                        break;
+                    case(osg::DisplaySettings::HORIZONTAL_INTERLACE):
+                    case(osg::DisplaySettings::VERTICAL_INTERLACE):
+                        numStencilBits = 8;
+                        break;
+                    default:
+                        break;
+                }
+            }
+            
+            // set up stencil buffer if required.            
+            numStencilBits = osg::maximum(numStencilBits,_ds->getMinimumNumStencilBits());
+            if (numStencilBits > 0)
+            {
+                rs_vc->setStencilSize(numStencilBits);
+            }
+            
             if (_ds->getAlphaBuffer()) rs_vc->setAlphaSize(_ds->getMinimumNumAlphaBits());
 
             rs_vc->setDepthSize(24);
 
-            if (numMultiSamples)
+            if (_ds->getAccumBuffer())
             {
-                #if defined( GLX_SAMPLES_SGIS )
-                    rs_vc->addExtendedAttribute( GLX_SAMPLES_SGIS,  numMultiSamples);
-                #endif
-                #if defined( GLX_SAMPLES_BUFFER_SGIS )
-                    rs_vc->addExtendedAttribute( GLX_SAMPLES_BUFFER_SGIS,  1);
-                #endif
+                rs_vc->setAccumRedSize(_ds->getMinimumNumAccumRedBits());
+                rs_vc->setAccumGreenSize(_ds->getMinimumNumAccumGreenBits());
+                rs_vc->setAccumBlueSize(_ds->getMinimumNumAccumBlueBits());
+                rs_vc->setAccumAlphaSize(_ds->getMinimumNumAccumAlphaBits());
+            }
+            
+            if (_ds->getMultiSamples())
+            {
+                rs_vc->addAttribute(Producer::VisualChooser::SampleBuffers, 1);
+                rs_vc->addAttribute(Producer::VisualChooser::Samples, _ds->getNumMultiSamples());
             }
         }        
     }
 
+    // now set up GraphicsContext wrappers for each of the render surfaces
+    // to all core OSG classes to keep track of the graphics context.
+    for(RenderSurfaceStateMap::iterator ritr = _renderSurfaceStateMap.begin();
+        ritr != _renderSurfaceStateMap.end();
+        ++ritr)
+    {
+        Producer::RenderSurface* rs = ritr->first;
+        osg::State* state = ritr->second;
+        GraphicsContextImplementation* gc = new GraphicsContextImplementation(rs);
+        gc->setState(state);
+        state->setGraphicsContext(gc);
+        _gcList.push_back(gc);
+    }
     
     if( _global_stateset == NULL && _shvec.size() > 0 )
     {
@@ -519,39 +618,65 @@ bool OsgCameraGroup::realize()
     }
 
     setUpSceneViewsWithData();
-    
-    if (getTopMostSceneData())
+
+    // if we are multi-threaded check to see if particle exists in the scene
+    // if so we need to disable multi-threading of cameras.
+    if (_threadModel == Producer::CameraGroup::ThreadPerCamera)
     {
-        SearchForSpecialNodes sfsn;
-        getTopMostSceneData()->accept(sfsn);
-        if (sfsn._foundParticles)
+        if (getTopMostSceneData())
         {
-            osg::notify(osg::INFO)<<"Warning: disabling multi-threading of cull and draw"<<std::endl;
-            osg::notify(osg::INFO)<<"         to avoid threading problems in osgParticle."<<std::endl;
-            _thread_model = Producer::CameraGroup::SingleThreaded;
-        }
-        else
-        {
-            std::set<RenderSurface*> renderSurfaceSet;
-            for( unsigned int i = 0; i < _cfg->getNumberOfCameras(); i++ )
+            SearchForSpecialNodes sfsn;
+            getTopMostSceneData()->accept(sfsn);
+            if (sfsn._foundParticles)
             {
-                Producer::Camera *cam = _cfg->getCamera(i);
-                renderSurfaceSet.insert(cam->getRenderSurface());
+                osg::notify(osg::INFO)<<"Warning: disabling multi-threading of cull and draw"<<std::endl;
+                osg::notify(osg::INFO)<<"         to avoid threading problems in osgParticle."<<std::endl;
+                _threadModel = Producer::CameraGroup::SingleThreaded;
             }
-            if (renderSurfaceSet.size()!=_cfg->getNumberOfCameras())
-            {
-                // camera's must be sharing a RenderSurface, so we need to ensure that we're
-                // running single threaded, to avoid OpenGL threading issues.
-                osg::notify(osg::INFO)<<"Warning: disabling multi-threading of cull and draw to avoid"<<std::endl;
-                osg::notify(osg::INFO)<<"         threading problems when camera's share a RenderSurface."<<std::endl;
-                _thread_model = Producer::CameraGroup::SingleThreaded;
-            }
-            
-            
         }
         
     }
+
+    // if we are still multi-thread check to make sure that no render surfaces
+    // are shared and don't use shared contexts, if they do we need to 
+    // disable multi-threading of cameras.
+    if (_threadModel == Producer::CameraGroup::ThreadPerCamera)
+    {
+        std::set<RenderSurface*> renderSurfaceSet;
+        for( unsigned int i = 0; i < _cfg->getNumberOfCameras(); i++ )
+        {
+            Producer::Camera *cam = _cfg->getCamera(i);
+            renderSurfaceSet.insert(cam->getRenderSurface());
+        }
+        if (renderSurfaceSet.size()!=_cfg->getNumberOfCameras())
+        {
+            // camera's must be sharing a RenderSurface, so we need to ensure that we're
+            // running single threaded, to avoid OpenGL threading issues.
+            osg::notify(osg::INFO)<<"Warning: disabling multi-threading of cull and draw to avoid"<<std::endl;
+            osg::notify(osg::INFO)<<"         threading problems when camera's share a RenderSurface."<<std::endl;
+            _threadModel = Producer::CameraGroup::SingleThreaded;
+        }
+        else if (renderSurfaceSet.size()>1 && RenderSurface::allGLContextsAreShared())
+        {
+            // we have multiple RenderSurface, but with share contexts, which is dangerous for multi-threaded usage,
+            // so need to disable multi-threading to prevent problems.
+            osg::notify(osg::INFO)<<"Warning: disabling multi-threading of cull and draw to avoid"<<std::endl;
+            osg::notify(osg::INFO)<<"         threading problems when sharing graphics contexts within RenderSurface."<<std::endl;
+            _threadModel = Producer::CameraGroup::SingleThreaded;
+        }
+
+    }
     
+    if (_threadModel==Producer::CameraGroup::SingleThreaded)
+    {
+        osg::notify(osg::INFO)<<"OsgCameraGroup::realize() _threadModel==Producer::CameraGroup::SingleThreaded"<<std::endl;
+    }
+    else
+    {
+        osg::notify(osg::INFO)<<"OsgCameraGroup::realize() _threadModel==Producer::CameraGroup::ThreadPerCamera"<<std::endl;
+    }
+
+    if (_cfg.valid()) _cfg->setThreadModelDirective(_threadModel);
 
     _initialized = CameraGroup::realize();
     
@@ -653,7 +778,8 @@ void OsgCameraGroup::cleanup_frame()
         ++itr)
     {
         (*itr)->getSceneView()->releaseAllGLObjects();
-        (*itr)->setFlushOfAllDeletedGLObjectsOnNextFrame(true);
+        (*itr)->getSceneView()->setRenderStage(0);
+        (*itr)->setCleanUpOnNextFrame(true);
     }
     
     // make sure that the registry all flushes all its texture objects.
