@@ -2,7 +2,7 @@
 
 const char* osgProducerGetVersion()
 {
-    return "0.9.9";
+    return "1.1";
 }
 
 
