@@ -30,130 +30,6 @@ using namespace osg;
 #define SINGLE_THREAD_KEYBOARDMOUSE
 #endif
 
-//////////////////////////////////////////////////////////////////////////////
-//
-// Picking intersection visitor.
-//
-
-class PickIntersectVisitor : public osgUtil::IntersectVisitor
-{
-public:
-    PickIntersectVisitor()
-    { 
-    }
-    virtual ~PickIntersectVisitor() {}
-    
-    HitList& getIntersections(osg::Node *scene, osg::Vec3 nr, osg::Vec3 fr)
-    { 
-        // option for non-sceneView users: you need to get the screen perp line and call getIntersections
-        // if you are using Projection nodes you should also call setxy to define the xp,yp positions for use with
-        // the ray transformed by Projection
-        _lineSegment = new osg::LineSegment;
-        _lineSegment->set(nr,fr); // make a line segment
-        
-        //std::cout<<"near "<<nr<<std::endl;
-        //std::cout<<"far "<<fr<<std::endl;
-        
-        addLineSegment(_lineSegment.get());
-
-        scene->accept(*this);
-        return getHitList(_lineSegment.get());
-    }
-private:
-    osg::ref_ptr<osg::LineSegment> _lineSegment;
-    friend class osgUtil::IntersectVisitor;
-};
-
-// PickVisitor traverses whole scene and checks below all Projection nodes
-class PickVisitor : public osg::NodeVisitor
-{
-public:
-    PickVisitor()
-    { 
-        xp=yp=0;    
-        setTraversalMode(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN);
-    }
-    ~PickVisitor() {}
-
-    void setTraversalMask(osg::Node::NodeMask traversalMask)
-    {
-        NodeVisitor::setTraversalMask(traversalMask);
-        _piv.setTraversalMask(traversalMask);   
-    }
-
-	// Aug 2003 added to pass the nodemaskOverride to the PickIntersectVisitor
-     //   may be used make the visitor override the nodemask to visit invisible actions
-    inline void setNodeMaskOverride(osg::Node::NodeMask mask) {
-		_piv.setNodeMaskOverride(mask);
-		_nodeMaskOverride = mask; }
-
-
-    virtual void apply(osg::Projection& pr)
-    { // stack the intersect rays, transform to new projection, traverse
-        // Assumes that the Projection is an absolute projection
-        osg::Matrixd mt;
-        mt.invert(pr.getMatrix());
-        osg::Vec3 npt=osg::Vec3(xp,yp,-1.0f) * mt, farpt=osg::Vec3(xp,yp,1.0f) * mt;
-
-        // traversing the nodes children, using the projection direction
-        for (unsigned int i=0; i<pr.getNumChildren(); i++) 
-        {
-            osg::Node *nodech=pr.getChild(i);
-            osgUtil::IntersectVisitor::HitList &hli=_piv.getIntersections(nodech,npt, farpt);
-            for(osgUtil::IntersectVisitor::HitList::iterator hitr=hli.begin();
-                hitr!=hli.end();
-                ++hitr)
-            { // add the projection hits to the scene hits.
-                    // This is why _lineSegment is retained as a member of PickIntersectVisitor
-                _PIVsegHitList.push_back(*hitr);
-            }
-            traverse(*nodech);
-        }
-    }
-
-    osgUtil::IntersectVisitor::HitList& getHits(osg::Node *node, const osg::Vec3& near_point, const osg::Vec3& far_point)
-    {
-        // High level get intersection with sceneview using a ray from x,y on the screen
-        // sets xp,yp as pixels scaled to a mapping of (-1,1, -1,1); needed for Projection from x,y pixels
-
-        // first get the standard hits in un-projected nodes
-        _PIVsegHitList=_piv.getIntersections(node,near_point,far_point); // fill hitlist
-
-        // then get hits in projection nodes
-        traverse(*node); // check for projection nodes
-        return _PIVsegHitList;
-    }
-
-    osgUtil::IntersectVisitor::HitList& getHits(osg::Node *node, const osg::Matrixd &projm, const float x, const float y)
-    { 
-        // utility for non=sceneview viewers
-        // x,y are values returned by 
-        osg::Matrixd inverseMVPW;
-        inverseMVPW.invert(projm);
-        osg::Vec3 near_point = osg::Vec3(x,y,-1.0f)*inverseMVPW;
-        osg::Vec3 far_point = osg::Vec3(x,y,1.0f)*inverseMVPW;
-        
-        if (!near_point.valid() || !far_point.valid())
-        {
-            osg::notify(osg::INFO)<<"Warning: PickVisitor caught invalid nea/far poiints."<<std::endl;
-            return _PIVsegHitList;
-        }
-        
-        setxy(x,y);    
-        getHits(node,near_point,far_point);
-        return _PIVsegHitList;
-    }
-
-    inline void setxy(float xpt, float ypt) { xp=xpt; yp=ypt; }
-    inline bool hits() const { return _PIVsegHitList.size()>0;}
-    
-private:
-
-    PickIntersectVisitor _piv;
-    float xp, yp; // start point in viewport fraction coordiantes
-    osgUtil::IntersectVisitor::HitList       _PIVsegHitList;
-};
-
 
 class CollectedCoordinateSystemNodesVisitor : public osg::NodeVisitor
 {
@@ -201,8 +77,7 @@ public:
     {
         osg::notify(osg::INFO)<<"getCoordinateFrame("<<position<<")"<<std::endl;
 
-        // do automatic conversion between RefNodePath and NodePath.
-        osg::NodePath tmpPath = _viewer->getCoordindateSystemNodePath();
+        osg::NodePath tmpPath = _viewer->getCoordinateSystemNodePath();
         
         if (!tmpPath.empty())
         {        
@@ -265,10 +140,11 @@ Viewer::Viewer():
     _setDoneAtFrameNumber(0),
     _done(false),
     _writeImageWhenDone(false),
-    _writeImageFileName("saved_image.jpg"),
+    _writeImageFileName(getDefaultImageFileName()),
     _recordingAnimationPath(false),
     _recordingStartTime(0.0)
 {
+    _eventQueue = new osgGA::EventQueue(osgGA::GUIEventAdapter::Y_INCREASING_UPWARDS);
     _position[0] = 0.0;
     _position[1] = 0.0;
     _position[2] = 0.0;
@@ -283,10 +159,11 @@ Viewer::Viewer(Producer::CameraConfig *cfg):
     _setDoneAtFrameNumber(0),
     _done(false),
     _writeImageWhenDone(false),
-    _writeImageFileName("saved_image.jpg"),
+    _writeImageFileName(getDefaultImageFileName()),
     _recordingAnimationPath(false),
     _recordingStartTime(0.0)
 {
+    _eventQueue = new osgGA::EventQueue(osgGA::GUIEventAdapter::Y_INCREASING_UPWARDS);
     _position[0] = 0.0;
     _position[1] = 0.0;
     _position[2] = 0.0;
@@ -302,10 +179,11 @@ Viewer::Viewer(const std::string& configFile):
     _setDoneAtFrameNumber(0),
     _done(false),
     _writeImageWhenDone(false),
-    _writeImageFileName("saved_image.jpg"),
+    _writeImageFileName(getDefaultImageFileName()),
     _recordingAnimationPath(false),
     _recordingStartTime(0.0)
 {
+    _eventQueue = new osgGA::EventQueue(osgGA::GUIEventAdapter::Y_INCREASING_UPWARDS);
     _position[0] = 0.0;
     _position[1] = 0.0;
     _position[2] = 0.0;
@@ -320,10 +198,11 @@ Viewer::Viewer(osg::ArgumentParser& arguments):
     _setDoneAtFrameNumber(0),
     _done(false),
     _writeImageWhenDone(false),
-    _writeImageFileName("saved_image.jpg"),
+    _writeImageFileName(getDefaultImageFileName()),
     _recordingAnimationPath(false),
     _recordingStartTime(0.0)
 {
+    _eventQueue = new osgGA::EventQueue(osgGA::GUIEventAdapter::Y_INCREASING_UPWARDS);
     _position[0] = 0.0;
     _position[1] = 0.0;
     _position[2] = 0.0;
@@ -334,7 +213,7 @@ Viewer::Viewer(osg::ArgumentParser& arguments):
     {
         arguments.getApplicationUsage()->addCommandLineOption("-p <filename>","Specify camera path file to animate the camera through the loaded scene");
         arguments.getApplicationUsage()->addCommandLineOption("--run-till-frame-number <integer>","Specify the number of frame to run");
-        arguments.getApplicationUsage()->addCommandLineOption("--run-till-elapsed-time","Specify the about of time to run");
+        arguments.getApplicationUsage()->addCommandLineOption("--run-till-elapsed-time","Specify the amount of time to run");
         arguments.getApplicationUsage()->addCommandLineOption("--clear-color <float>,<float>,<float>[,<float>]","Specify the clear color as RGB or RGBA");
     }
 
@@ -344,8 +223,8 @@ Viewer::Viewer(osg::ArgumentParser& arguments):
     std::string pathfile;
     while (arguments.read("-p",pathfile))
     {
-	osg::ref_ptr<osgGA::AnimationPathManipulator> apm = new osgGA::AnimationPathManipulator(pathfile);
-	if( apm.valid() && apm->valid() ) 
+        osg::ref_ptr<osgGA::AnimationPathManipulator> apm = new osgGA::AnimationPathManipulator(pathfile);
+        if( apm.valid() && apm->valid() ) 
         {
             unsigned int num = addCameraManipulator(apm.get());
             selectCameraManipulator(num);
@@ -386,6 +265,18 @@ Viewer::~Viewer()
 {
 }
 
+osg::NodePath Viewer::getCoordinateSystemNodePath() const
+{
+    osg::NodePath nodePath;
+    for(ObserveredNodePath::const_iterator itr = _coordinateSystemNodePath.begin();
+        itr != _coordinateSystemNodePath.end();
+        ++itr)
+    {
+        nodePath.push_back(const_cast<osg::Node*>(itr->get()));
+    }
+    return nodePath;
+}
+
 void Viewer::setWriteImageFileName(const std::string& filename)
 {
     _writeImageFileName = filename;
@@ -406,8 +297,16 @@ const std::string& Viewer::getWriteImageFileName() const
     return _writeImageFileName;
 }
 
+static osg::ApplicationUsageProxy Viewer_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE, "OSG_IMAGE_FILE_NAME <filename>", "name of snapshot image file" );
 
-void Viewer::setCoordindateSystemNodePath(const osg::NodePath& nodePath)
+const char* Viewer::getDefaultImageFileName()
+{
+    const char* name = getenv( "OSG_IMAGE_FILE_NAME" );
+    return name ? name : "saved_image.jpg";
+}
+
+
+void Viewer::setCoordinateSystemNodePath(const osg::NodePath& nodePath)
 {
     _coordinateSystemNodePath.clear();
     std::copy(nodePath.begin(),
@@ -415,7 +314,7 @@ void Viewer::setCoordindateSystemNodePath(const osg::NodePath& nodePath)
               std::back_inserter(_coordinateSystemNodePath));
 }
 
-void Viewer::computeActiveCoordindateSystemNodePath()
+void Viewer::computeActiveCoordinateSystemNodePath()
 {
     // now search for CoordinateSystemNode's for which we want to track.
     osg::Node* subgraph = getTopMostSceneData();
@@ -427,12 +326,12 @@ void Viewer::computeActiveCoordindateSystemNodePath()
 
         if (!ccsnv._pathToCoordinateSystemNode.empty())
         {
-           setCoordindateSystemNodePath(ccsnv._pathToCoordinateSystemNode);
+           setCoordinateSystemNodePath(ccsnv._pathToCoordinateSystemNode);
            return;
         }
     }  
     // otherwise no node path found so reset to empty.
-    setCoordindateSystemNodePath(osg::NodePath());
+    setCoordinateSystemNodePath(osg::NodePath());
 }
 
 void Viewer::updatedSceneData()
@@ -440,7 +339,7 @@ void Viewer::updatedSceneData()
     OsgCameraGroup::updatedSceneData();
 
     // refresh the coordinate system node path.
-    computeActiveCoordindateSystemNodePath();
+    computeActiveCoordinateSystemNodePath();
 
     // refresh the camera manipulators    
     if (_keyswitchManipulator.valid()) _keyswitchManipulator->setNode(getTopMostSceneData());
@@ -455,7 +354,11 @@ void Viewer::setKeyboardMouse(Producer::KeyboardMouse* kbm)
 void Viewer::setKeyboardMouseCallback(osgProducer::KeyboardMouseCallback* kbmcb)
 {
     _kbmcb = kbmcb;
-    if (_kbm.valid() && _kbmcb.valid()) _kbm->setCallback(_kbmcb.get());
+    if (_kbm.valid() && _kbmcb.valid())
+    {
+        _kbm->setCallback(_kbmcb.get());
+        _kbmcb->setEventQueue(_eventQueue.get());
+    }
 }
 
 void Viewer::setUpViewer(unsigned int options)
@@ -466,17 +369,19 @@ void Viewer::setUpViewer(unsigned int options)
     
     if (!_kbm)
     {
-        _kbm = ia ?
+        setKeyboardMouse(ia ?
                    (new Producer::KeyboardMouse(ia)) : 
-                   (new Producer::KeyboardMouse(getCamera(0)->getRenderSurface()));
+                   (new Producer::KeyboardMouse(getCamera(0)->getRenderSurface())) );
                    
     }
     
     // set the keyboard mouse callback to catch the events from the windows.
     if (!_kbmcb)
-        _kbmcb = new osgProducer::KeyboardMouseCallback( _kbm.get(), _done, (options & ESCAPE_SETS_DONE)!=0 );
+    {
+        setKeyboardMouseCallback(new osgProducer::KeyboardMouseCallback( _kbm.get(), _done, (options & ESCAPE_SETS_DONE)!=0 ));
+    }
         
-    _kbmcb->setStartTick(_start_tick);
+    getEventQueue()->setStartTick(_start_tick);
     
     // register the callback with the keyboard mouse manger.
     _kbm->setCallback( _kbmcb.get() );
@@ -493,11 +398,12 @@ void Viewer::setUpViewer(unsigned int options)
         // enable depth testing by default.
         globalStateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::ON);
 
-
+#if 0
         // set up an alphafunc by default to speed up blending operations.
         osg::AlphaFunc* alphafunc = new osg::AlphaFunc;
         alphafunc->setFunction(osg::AlphaFunc::GREATER,0.0f);
         globalStateSet->setAttributeAndModes(alphafunc, osg::StateAttribute::ON);
+#endif
     }
     
     if (options & HEAD_LIGHT_SOURCE || options & SKY_LIGHT_SOURCE)
@@ -593,7 +499,7 @@ void Viewer::setViewByMatrix( const Producer::Matrix & pm)
 bool Viewer::realize( ThreadingModel thread_model )
 {
     if( _realized ) return _realized;
-    _thread_model = thread_model;
+    _threadModel = thread_model;
     return realize();
 }
 
@@ -615,10 +521,27 @@ bool Viewer::realize()
 
     OsgCameraGroup::realize();
 
+    // force a sync before we intialize the keyswitch manipulator to home
+    // so that Producer has a chance to set up the windows before we do
+    // any work on them.
+    OsgCameraGroup::sync();
+
 #ifndef SINGLE_THREAD_KEYBOARDMOUSE
     // kick start the keyboard mouse if needed.
-    if (_kbm.valid() && !_kbm->isRunning()) _kbm->startThread();
+    if (_kbm.valid() && !_kbm->isRunning())
+    {
+        _kbm->startThread();
+
+        while (!_kbm->isRunning())
+        {
+            // osg::notify(osg::NOTICE)<<"Waiting"<<std::endl;
+            OpenThreads::Thread::YieldCurrentThread();
+        }
+    }
 #endif
+
+    if (_kbmcb.valid()) _kbmcb->updateWindowSize();
+
     // by default set up the DatabasePager.
     {    
         osgDB::DatabasePager* databasePager = osgDB::Registry::instance()->getOrCreateDatabasePager();
@@ -645,18 +568,13 @@ bool Viewer::realize()
 
     }
     
-    // force a sync before we intialize the keyswitch manipulator to home
-    // so that Producer has a chance to set up the windows before we do
-    // any work on them.
-    OsgCameraGroup::sync();
  
-    if (_keyswitchManipulator.valid() && _keyswitchManipulator->getCurrentMatrixManipulator())
+    if (_keyswitchManipulator.valid() && _keyswitchManipulator->getCurrentMatrixManipulator() && _eventQueue.valid())
     {
         _keyswitchManipulator->setCoordinateFrameCallback(new ViewerCoordinateFrameCallback(this));
 
-        osg::ref_ptr<osgProducer::EventAdapter> init_event = _kbmcb->createEventAdapter();
-        init_event->adaptFrame(0.0);
-    
+        osg::ref_ptr<osgGA::GUIEventAdapter> init_event = _eventQueue->createEvent();
+
         _keyswitchManipulator->setNode(getTopMostSceneData());
         _keyswitchManipulator->home(*init_event,*this);
     }
@@ -678,14 +596,12 @@ void Viewer::update()
     if (_kbm.valid() && !_kbm->isRunning()) _kbm->update(*(_kbm->getCallback()));
 #endif
 
-    // get the event since the last frame.
-    osgProducer::KeyboardMouseCallback::EventQueue queue;
-    if (_kbmcb.valid()) _kbmcb->getEventQueue(queue);
-
     // create an event to signal the new frame.
-    osg::ref_ptr<osgProducer::EventAdapter> frame_event = new osgProducer::EventAdapter;
-    frame_event->adaptFrame(_frameStamp->getReferenceTime());
-    queue.push_back(frame_event);
+    getEventQueue()->frame(_frameStamp->getReferenceTime());
+
+    // get the event since the last frame.
+    osgGA::EventQueue::Events events;
+    getEventQueue()->takeEvents(events);
 
     if (_eventVisitor.valid())
     {
@@ -693,22 +609,26 @@ void Viewer::update()
     }
 
     // dispatch the events in order of arrival.
-    for(osgProducer::KeyboardMouseCallback::EventQueue::iterator event_itr=queue.begin();
-        event_itr!=queue.end();
+    for(osgGA::EventQueue::Events::iterator event_itr=events.begin();
+        event_itr!=events.end();
         ++event_itr)
     {
         bool handled = false;
-        for(EventHandlerList::iterator handler_itr=_eventHandlerList.begin();
-            handler_itr!=_eventHandlerList.end() && !handled;
-            ++handler_itr)
-        {   
-            handled = (*handler_itr)->handle(*(*event_itr),*this,0,0);
-        }
+
         if (_eventVisitor.valid())
         {
             _eventVisitor->reset();
             _eventVisitor->addEvent(event_itr->get());
             getTopMostSceneData()->accept(*_eventVisitor);
+            if (_eventVisitor->getEventHandled())
+                handled = true;
+        }
+
+        for(EventHandlerList::iterator handler_itr=_eventHandlerList.begin();
+            handler_itr!=_eventHandlerList.end() && !handled;
+            ++handler_itr)
+        {   
+            handled = (*handler_itr)->handle(*(*event_itr),*this,0,0);
         }
         
     }
@@ -797,6 +717,12 @@ void Viewer::frame()
 
     OsgCameraGroup::frame();
 
+
+    if (osg::Referenced::getDeleteHandler()) 
+    {
+        osg::Referenced::getDeleteHandler()->flush();
+    }
+
 }
 
 bool Viewer::computePixelCoords(float x,float y,unsigned int cameraNum,float& pixel_x,float& pixel_y)
@@ -849,55 +775,74 @@ bool Viewer::computeNearFarPoints(float x,float y,unsigned int cameraNum,osg::Ve
     float pixel_x,pixel_y;
     if (computePixelCoords(x,y,cameraNum,pixel_x,pixel_y))
     {
+        pixel_x-=sv->getViewport()->x();
+        pixel_y-=sv->getViewport()->y();
+
         return sv->projectWindowXYIntoObject((int)(pixel_x+0.5f),(int)(pixel_y+0.5f),near_point,far_point);
     }
     return false;
 
 }
 
-bool Viewer::computeIntersections(float x,float y,unsigned int cameraNum,osg::Node *node,osgUtil::IntersectVisitor::HitList& hits,osg::Node::NodeMask traversalMask)
+bool Viewer::computeIntersections(float x,float y,unsigned int cameraNum,osg::Node* node,osgUtil::IntersectVisitor::HitList& hits,osg::Node::NodeMask traversalMask)
 {
     float pixel_x,pixel_y;
-    if (computePixelCoords(x,y,cameraNum,pixel_x,pixel_y))
+    if (node && computePixelCoords(x,y,cameraNum,pixel_x,pixel_y))
     {
 
         Producer::Camera* camera=getCamera(cameraNum);
 
-        int pr_wx, pr_wy;
-        unsigned int pr_width, pr_height;
-        camera->getProjectionRectangle( pr_wx, pr_wy, pr_width, pr_height );
-
-        // convert into clip coords.
-        float rx = 2.0f*(pixel_x - (float)pr_wx)/(float)pr_width-1.0f;
-        float ry = 2.0f*(pixel_y - (float)pr_wy)/(float)pr_height-1.0f;
-
-        //std::cout << "    rx "<<rx<<"  "<<ry<<std::endl;
-
         osgProducer::OsgSceneHandler* sh = dynamic_cast<osgProducer::OsgSceneHandler*>(camera->getSceneHandler());
-        osgUtil::SceneView* sv = sh?sh->getSceneView():0;
-        osg::Matrixd vum;
+        osgUtil::SceneView* sv = sh ? sh->getSceneView() : 0;
+        osg::Matrixd proj;
+        osg::Matrixd view;
+        const osg::Viewport* viewport = 0;
+        osg::Node* rootNode = 0; 
         if (sv!=0)
         {
-            vum.set(sv->getViewMatrix() *
-                    sv->getProjectionMatrix());
+            viewport = sv->getViewport();
+            proj = sv->getProjectionMatrix();
+            view = sv->getViewMatrix();
+            rootNode = sv->getSceneData();
         }
         else
         {
-            vum.set(osg::Matrixd(camera->getViewMatrix()) *
-                    osg::Matrixd(camera->getProjectionMatrix()));
+            viewport = 0;
+            proj = osg::Matrixd(camera->getProjectionMatrix());
+            view = osg::Matrixd(camera->getViewMatrix());
         }
 
-        PickVisitor iv;
-        iv.setTraversalMask(traversalMask);
+        unsigned int numHitsBefore = hits.size();
+
+        osg::NodePathList parentNodePaths = node->getParentalNodePaths(rootNode);
+        for(unsigned int i=0;i<parentNodePaths.size();++i)
+        {
+            osg::NodePath& nodePath = parentNodePaths[i];
+            
+            // remove the intersection node from the nodePath as it'll be accounted for
+            // in the PickVisitor traversal, so we don't double account for its transform.
+            if (!nodePath.empty()) nodePath.pop_back();  
+            
+            osg::Matrixd modelview(view);
+            // modify the view matrix so that it accounts for this nodePath's accumulated transform
+            if (!nodePath.empty()) modelview.preMult(computeLocalToWorld(nodePath));
+            
+            osgUtil::PickVisitor pick(viewport, proj, modelview, pixel_x, pixel_y);
+            pick.setTraversalMask(traversalMask);
+            node->accept(pick);
+
+            // copy all the hits across to the external hits list
+            for(osgUtil::PickVisitor::LineSegmentHitListMap::iterator itr = pick.getSegHitList().begin();
+                itr != pick.getSegHitList().end();
+                ++itr)
+            {
+                hits.insert(hits.end(),itr->second.begin(), itr->second.end());
+            }
+
+        }
         
-        osgUtil::IntersectVisitor::HitList localHits;        
-        localHits = iv.getHits(node, vum, rx,ry);
-        
-        if (localHits.empty()) return false;
-        
-        hits.insert(hits.begin(),localHits.begin(),localHits.end());
-        
-        return true;
+        // return true if we now have more hits than before
+        return hits.size()>numHitsBefore;
     }
     return false;
 }
@@ -925,7 +870,20 @@ bool Viewer::computeIntersections(float x,float y,osgUtil::IntersectVisitor::Hit
 
 void Viewer::selectCameraManipulator(unsigned int no)
 {
-    if (_keyswitchManipulator.valid()) _keyswitchManipulator->selectMatrixManipulator(no);
+   if (_keyswitchManipulator.valid())
+   {
+      _keyswitchManipulator->selectMatrixManipulator(no);
+      
+      // keyswitch manipulator doesn't yet force manipulators to init themselves
+      // so we'll do this mannually.  Note pretty, and needs replacing by a refactor
+      // of MatrixMinpulators in the longer term.
+      osg::ref_ptr<osgGA::GUIEventAdapter> ea = new osgGA::GUIEventAdapter;
+      double time = _kbmcb.valid() ? _kbmcb->getTime() : 0.0;
+      ea->setTime(time);
+      ea->setEventType(osgGA::GUIEventAdapter::KEYDOWN);
+      ea->setKey(osgGA::GUIEventAdapter::KEY_KP_1+no);
+      _keyswitchManipulator->init(*ea, *this);
+   }
 }
 
 void Viewer::getCameraManipulatorNameList( std::list<std::string> &nameList )
@@ -989,12 +947,11 @@ void Viewer::requestContinuousUpdate(bool)
 
 void Viewer::requestWarpPointer(float x,float y)
 {
-    if (_kbmcb.valid())
+    if (_kbmcb.valid() && isRealized())
     {
         osg::notify(osg::INFO) << "requestWarpPointer x= "<<x<<" y="<<y<<std::endl;
     
-        EventAdapter::_s_mx = x;
-        EventAdapter::_s_my = y;
+        getEventQueue()->mouseWarp(x,y);
         _kbmcb->getKeyboardMouse()->positionPointer(x,y);
         return;
     }   
@@ -1014,3 +971,4 @@ void Viewer::getUsage(osg::ApplicationUsage& usage) const
         (*itr)->getUsage(usage);
     }
 }
+
