@@ -4,12 +4,65 @@
 #include <osgDB/FileNameUtils>
 #include <osgText/Text>
 #include <osg/BlendFunc>
+#include <osg/PolygonOffset>
+#include <osgUtil/Statistics>
+#include <OpenThreads/Barrier>
+
 
 #include <algorithm>
 #include <fstream>
 
 using namespace osgProducer;
 
+class ViewerEventHandler::CameraBarrierCallback : public Producer::Camera::Callback, public OpenThreads::Barrier
+{
+public:
+    CameraBarrierCallback(unsigned int numThreads):
+        OpenThreads::Barrier(numThreads),
+        _doBlock(false) {}
+        
+    virtual ~CameraBarrierCallback()
+    {
+        release();
+    }
+    
+    void setDoBlock(bool block)
+    {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+        if (_doBlock != block)
+        {
+            if (_doBlock) release();
+            
+            _doBlock = block;
+            
+            osg::notify(osg::NOTICE)<<"setDoBlock("<<block<<")"<<std::endl;
+        }
+    }
+    
+    bool getDoBlock() const { return _doBlock; } 
+    
+    virtual void operator() (const Producer::Camera&)
+    {
+        bool doBlock = false;
+        {
+            OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
+            doBlock = _doBlock;
+        }
+        
+        if (doBlock)
+        {
+            osg::notify(osg::NOTICE)<<"Entering block()"<<std::endl;
+            block();
+        }
+    };
+
+protected:
+
+    bool                _doBlock;
+    OpenThreads::Mutex  _mutex;
+    
+};
+
 
 class ViewerEventHandler::SnapImageDrawCallback : public Producer::Camera::Callback
 {
@@ -77,7 +130,7 @@ public:
         _stateset->setMode(GL_DEPTH_TEST,osg::StateAttribute::OFF);
         _stateset->setMode(GL_BLEND,osg::StateAttribute::ON);
         
-        _projection = new osg::RefMatrix(osg::Matrix::ortho2D(0.0,1280,0,1024));
+        _projection = new osg::RefMatrix(osg::Matrix::ortho2D(0.0,1280,0.0,1024));
         _modelview = new osg::RefMatrix();
     
         //createHelpText();
@@ -144,6 +197,12 @@ protected:
     osg::ref_ptr<osgText::Text> _positionText;
     osg::ref_ptr<osgText::Text> _orientationText;
     osg::ref_ptr<osgText::Text> _speedText;
+    
+    TextList                    _sceneStatsLabelList;
+    osg::ref_ptr<osgText::Text> _numVerticesText;
+    osg::ref_ptr<osgText::Text> _numPrimitivesText;
+    osg::ref_ptr<osgText::Text> _numDrawablesText;
+    
 
     std::vector <Producer::CameraGroup::FrameStats> _fs;
     unsigned int _index;
@@ -165,7 +224,14 @@ void ViewerEventHandler::StatsAndHelpDrawCallback::operator()( const Producer::C
     osgUtil::SceneView* sv = osh->getSceneView();
     osg::State& state = *(sv->getState());
     
-    
+
+    if (!_projection) _projection = new osg::RefMatrix(osg::Matrix::ortho2D(0.0,width/height*1024.0f,0.0,1024.0));
+    else 
+    {
+        _projection->makeOrtho2D(0.0,width/height*1024.0f,0.0,1024.0);
+    }
+
+
     state.applyProjectionMatrix(_projection.get());
     state.applyModelViewMatrix(_modelview.get());
     
@@ -379,6 +445,8 @@ void ViewerEventHandler::StatsAndHelpDrawCallback::displayStats()
     OsgSceneHandler* osh = _veh->getOsgCameraGroup()->getSceneHandlerList()[_cameraNumber].get();
     osgUtil::SceneView* sv = osh->getSceneView();
 
+    char tmpText[128];
+
     // render graphs
     if (_veh->getFrameStatsMode()>=ViewerEventHandler::CAMERA_STATS)
     {
@@ -387,7 +455,7 @@ void ViewerEventHandler::StatsAndHelpDrawCallback::displayStats()
         glMatrixMode( GL_PROJECTION );
         glPushMatrix();
         glLoadIdentity();
-        glOrtho( -.025, .128, 600.0, -10.0, -1.0, 1.0 ); 
+        glOrtho( -.03, .128, 600.0, -20.0, -1.0, 1.0 ); 
 
 
         unsigned int lindex = (_index + 1) % _fs.size();
@@ -486,10 +554,8 @@ void ViewerEventHandler::StatsAndHelpDrawCallback::displayStats()
     {
         // update and draw the frame rate text.
         
-        char tmpText[128];
 
         _frameRateLabelText->draw(*(sv->getState()));
-
         if (_fs.size()>1)
         {
             unsigned int lindex = (_index + 1) % _fs.size();
@@ -554,6 +620,44 @@ void ViewerEventHandler::StatsAndHelpDrawCallback::displayStats()
             }
         }
         
+        if (_veh->getFrameStatsMode()>=ViewerEventHandler::SCENE_STATS)
+        {
+            osgUtil::Statistics stats;
+
+            for(osgProducer::OsgCameraGroup::SceneHandlerList::iterator shitr = _veh->getOsgCameraGroup()->getSceneHandlerList().begin();
+                shitr != _veh->getOsgCameraGroup()->getSceneHandlerList().end();
+                ++shitr)
+            {
+                (*shitr)->getSceneView()->getStats(stats); 
+            }
+
+            unsigned int primitives = 0;
+            for(osgUtil::Statistics::PrimitiveCountMap::iterator pcmitr = stats.GetPrimitivesBegin();
+                pcmitr != stats.GetPrimitivesEnd();
+                ++pcmitr)
+            {
+                primitives += pcmitr->second;
+            }
+            
+            for(TextList::iterator itr = _sceneStatsLabelList.begin();
+                itr != _sceneStatsLabelList.end();
+                ++itr)
+            {
+                (*itr)->draw(*(sv->getState()));
+            }
+            
+            sprintf(tmpText,"%d",stats._vertexCount);
+            _numVerticesText->setText(tmpText);
+            _numVerticesText->draw(*(sv->getState()));
+
+            sprintf(tmpText,"%d",primitives);
+            _numPrimitivesText->setText(tmpText);
+            _numPrimitivesText->draw(*(sv->getState()));
+
+            sprintf(tmpText,"%d",stats.numDrawables);
+            _numDrawablesText->setText(tmpText);
+            _numDrawablesText->draw(*(sv->getState()));
+        } 
     }
 
 }
@@ -593,7 +697,7 @@ void ViewerEventHandler::StatsAndHelpDrawCallback::createStatsText()
 
 
     pos.x() = leftPos;
-    pos.y() -= characterSize;
+    pos.y() -= characterSize*1.5f;
 
     {
         osgText::Text* text = new osgText::Text;
@@ -693,7 +797,91 @@ void ViewerEventHandler::StatsAndHelpDrawCallback::createStatsText()
         pos.y() -= characterSize;
     }
 
+    pos.y() -= characterSize*0.5f;
 
+    // scene stats section
+    {
+        pos.x() = leftPos;
+
+        // num drawables
+        osgText::Text* drawLabel = new osgText::Text;
+        drawLabel->setFont("fonts/arial.ttf");
+        drawLabel->setColor(colorDraw);
+        drawLabel->setFontResolution((unsigned int)characterSize,(unsigned int)characterSize);
+        drawLabel->setCharacterSize(characterSize);
+        drawLabel->setPosition(pos);
+        drawLabel->setAlignment(osgText::Text::BASE_LINE);
+        drawLabel->setText("Number of Drawables: ");
+
+        _sceneStatsLabelList.push_back(drawLabel);
+        
+        pos.x() = drawLabel->getBound().xMax();
+
+        _numDrawablesText = new osgText::Text;
+        _numDrawablesText->setFont("fonts/arial.ttf");
+        _numDrawablesText->setColor(colorDraw);
+        _numDrawablesText->setFontResolution((unsigned int)characterSize,(unsigned int)characterSize);
+        _numDrawablesText->setCharacterSize(characterSize);
+        _numDrawablesText->setPosition(pos);
+        _numDrawablesText->setAlignment(osgText::Text::BASE_LINE);
+        _numDrawablesText->setText("100000  ");
+        
+        pos.x() = _numDrawablesText->getBound().xMax();
+
+        // num vertices
+        drawLabel = new osgText::Text;
+        drawLabel->setFont("fonts/arial.ttf");
+        drawLabel->setColor(colorDraw);
+        drawLabel->setFontResolution((unsigned int)characterSize,(unsigned int)characterSize);
+        drawLabel->setCharacterSize(characterSize);
+        drawLabel->setPosition(pos);
+        drawLabel->setAlignment(osgText::Text::BASE_LINE);
+        drawLabel->setText("Number of Vertices: ");
+
+        _sceneStatsLabelList.push_back(drawLabel);
+        
+        pos.x() = drawLabel->getBound().xMax();
+
+        _numVerticesText = new osgText::Text;
+        _numVerticesText->setFont("fonts/arial.ttf");
+        _numVerticesText->setColor(colorDraw);
+        _numVerticesText->setFontResolution((unsigned int)characterSize,(unsigned int)characterSize);
+        _numVerticesText->setCharacterSize(characterSize);
+        _numVerticesText->setPosition(pos);
+        _numVerticesText->setAlignment(osgText::Text::BASE_LINE);
+        _numVerticesText->setText("1000000  ");
+        
+        pos.x() = _numVerticesText->getBound().xMax();
+
+    
+        // num primitives
+        drawLabel = new osgText::Text;
+        drawLabel->setFont("fonts/arial.ttf");
+        drawLabel->setColor(colorDraw);
+        drawLabel->setFontResolution((unsigned int)characterSize,(unsigned int)characterSize);
+        drawLabel->setCharacterSize(characterSize);
+        drawLabel->setPosition(pos);
+        drawLabel->setAlignment(osgText::Text::BASE_LINE);
+        drawLabel->setText("Number of Primitives: ");
+
+        _sceneStatsLabelList.push_back(drawLabel);
+        
+        pos.x() = drawLabel->getBound().xMax();
+
+        _numPrimitivesText = new osgText::Text;
+        _numPrimitivesText->setFont("fonts/arial.ttf");
+        _numPrimitivesText->setColor(colorDraw);
+        _numPrimitivesText->setFontResolution((unsigned int)characterSize,(unsigned int)characterSize);
+        _numPrimitivesText->setCharacterSize(characterSize);
+        _numPrimitivesText->setPosition(pos);
+        _numPrimitivesText->setAlignment(osgText::Text::BASE_LINE);
+        _numPrimitivesText->setText("1000000  ");
+        
+        pos.x() = _numPrimitivesText->getBound().xMax();
+        
+
+
+    }
 }
 
 void ViewerEventHandler::StatsAndHelpDrawCallback::displayInfo()
@@ -714,6 +902,20 @@ ViewerEventHandler::ViewerEventHandler(OsgCameraGroup* cg):
     _firstTimeTogglingFullScreen(true)
 {
     Producer::CameraConfig* cfg = _cg->getCameraConfig();
+
+    _cameraBarrierCallback = 0;
+#if 0    
+    if (cfg->getNumberOfCameras()>1) 
+    {
+        // use a barrier to make that stats only runs once all the threads have done their drawing.
+        _cameraBarrierCallback = new CameraBarrierCallback(cfg->getNumberOfCameras());
+        for(unsigned int i=0;i<cfg->getNumberOfCameras();++i)
+        {
+            cfg->getCamera(i)->addPostDrawCallback(_cameraBarrierCallback);
+        }
+    }
+#endif
+        
     Producer::Camera *cam = cfg->getCamera(0);
     
     _statsAndHelpDrawCallback = new StatsAndHelpDrawCallback(this,0);
@@ -729,8 +931,7 @@ ViewerEventHandler::ViewerEventHandler(OsgCameraGroup* cg):
     
     Viewer* viewer = dynamic_cast<Viewer*>(cg);
     if (viewer) setWriteImageFileName(viewer->getWriteImageFileName());
-    else setWriteImageFileName("saved_image.jpg");
-    
+    else setWriteImageFileName( Viewer::getDefaultImageFileName() );
 }
 
 void ViewerEventHandler::setWriteImageOnNextFrame(bool writeImageOnNextFrame)
@@ -778,7 +979,12 @@ void ViewerEventHandler::setFrameStatsMode(FrameStatsMode mode)
     else
     {
         _cg->setInstrumentationMode(true);
-    }    
+    }
+    
+    if (_cameraBarrierCallback)
+    {
+        _cameraBarrierCallback->setDoBlock(_frameStatsMode>=SCENE_STATS);
+    }
 }
 
 bool ViewerEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& aa)
@@ -792,7 +998,7 @@ bool ViewerEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActio
         {
             case 's' :
             {
-                FrameStatsMode newFrameStatsMode = (FrameStatsMode)((_frameStatsMode+1)%3);
+                FrameStatsMode newFrameStatsMode = (FrameStatsMode)((_frameStatsMode+1)%4);
                 setFrameStatsMode(newFrameStatsMode);
                 return true;
             }
@@ -885,17 +1091,36 @@ bool ViewerEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActio
                 setWriteImageOnNextFrame(true);                
                 return true;
             }
+#if 0
+// Adjust osg::PolygonOffset
             case '+' :
             {
-                //_camera->setFusionDistanceRatio(_camera->getFusionDistanceRatio()*1.25f);
+                osg::PolygonOffset::setFactorMultiplier(osg::PolygonOffset::getFactorMultiplier()*1.1f);
+                osg::notify(osg::NOTICE)<<"osg::PolygonOffset::getFactorMultiplier()="<<osg::PolygonOffset::getFactorMultiplier()<<std::endl;
                 return true;
             }
             case '-' :
             {
-                //_camera->setFusionDistanceRatio(_camera->getFusionDistanceRatio()/1.25f);
+                osg::PolygonOffset::setFactorMultiplier(osg::PolygonOffset::getFactorMultiplier()/1.1f);
+                osg::notify(osg::NOTICE)<<"osg::PolygonOffset::getFactorMultiplier()="<<osg::PolygonOffset::getFactorMultiplier()<<std::endl;
                 return true;
             }
 
+            case '*' :
+            {
+                osg::PolygonOffset::setUnitsMultiplier(osg::PolygonOffset::getUnitsMultiplier()*1.1f);
+                osg::notify(osg::NOTICE)<<"osg::PolygonOffset::getUnitsMultiplier()="<<osg::PolygonOffset::getUnitsMultiplier()<<std::endl;
+                return true;
+            }
+            case '/' :
+            {
+                osg::PolygonOffset::setUnitsMultiplier(osg::PolygonOffset::getUnitsMultiplier()/1.1f);
+                osg::notify(osg::NOTICE)<<"osg::PolygonOffset::getUnitsMultiplier()="<<osg::PolygonOffset::getUnitsMultiplier()<<std::endl;
+                return true;
+            }
+#else
+// Adjust osg::LODScale
+
             case '*' :
             case '/' :
             {
@@ -904,7 +1129,7 @@ bool ViewerEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActio
                 std::cout<<"New LOD Scale = "<<_cg->getLODScale()<<std::endl;
                 return true;
             }
-
+#endif
             case osgGA::GUIEventAdapter::KEY_Help :
             case 'h' :
             {
@@ -951,9 +1176,10 @@ bool ViewerEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActio
                         osgGA::KeySwitchMatrixManipulator* kscm = viewer->getKeySwitchMatrixManipulator();
                         if (kscm)
                         {
-                            for(apmNo=0;apmNo<kscm->getNumMatrixManipulators() && apm==0;++apmNo)
+                            for(apmNo=0;apmNo<kscm->getNumMatrixManipulators();++apmNo)
                             {
-                                apm = dynamic_cast<osgGA::AnimationPathManipulator*>(kscm->getMatrixManipulator(apmNo));
+                                apm = dynamic_cast<osgGA::AnimationPathManipulator*>(kscm->getMatrixManipulatorWithIndex(apmNo));
+                                if (apm) break;
                             }
                         }
 
@@ -997,7 +1223,5 @@ void ViewerEventHandler::getUsage(osg::ApplicationUsage& usage) const
     usage.addKeyboardMouseBinding("s","Toggle instrumention");
     usage.addKeyboardMouseBinding("v","Toggle block and vsync");
     usage.addKeyboardMouseBinding("z","Start recording camera path.");
-    usage.addKeyboardMouseBinding("Z","If recording camera path stop recording camera path, save to \"saved_animation.path\"\nThen start viewing from being on animation path");
-//    usage.addKeyboardMouseBinding("+","When in stereo, increase the fusion distance");
-//    usage.addKeyboardMouseBinding("-","When in stereo, reduse the fusion distance");
+    usage.addKeyboardMouseBinding("Z","If recording camera path stop recording camera path, save to \"saved_animation.path\"\nThen restart camera from beginning on animation path");
 }
