@@ -1,5 +1,5 @@
 ;
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -21,16 +21,17 @@
 
 #include <osgDB/ReadFile>
 #include <osgDB/FileUtils>
+#include <osgDB/FileNameUtils>
 #include <osg/GLU>
 
 using namespace osgText;
 using namespace std;
 
-std::string findFontFile(const std::string& str)
+std::string osgText::findFontFile(const std::string& str)
 {
     // try looking in OSGFILEPATH etc first for fonts.
     std::string filename = osgDB::findDataFile(str);
-    if (!filename.empty()) return std::string(filename);
+    if (!filename.empty()) return filename;
 
 
     static osgDB::FilePathList s_FontFilePath;
@@ -46,7 +47,9 @@ std::string findFontFile(const std::string& str)
         char *ptr;
         if ((ptr = getenv( "windir" )))
         {
-            s_FontFilePath.push_back(ptr);
+            std::string winFontPath = ptr;
+            winFontPath += "\\fonts";
+            s_FontFilePath.push_back(winFontPath);
         }
     #else
         osgDB::convertStringPathIntoFilePathList(
@@ -58,12 +61,28 @@ std::string findFontFile(const std::string& str)
     filename = osgDB::findFileInPath(str,s_FontFilePath);
     if (!filename.empty()) return filename;
 
+    // Try filename without pathname, if it has a path
+    filename = osgDB::getSimpleFileName(str);
+    if(filename!=str)
+    {
+        filename = osgDB::findFileInPath(filename,s_FontFilePath);
+        if (!filename.empty()) return filename;
+    }
+    else
+    {
+        filename = osgText::findFontFile(std::string("fonts/")+filename);
+        if (!filename.empty()) return filename;
+    }
+
+    // Not found, return empty string
     osg::notify(osg::WARN)<<"Warning: font file \""<<str<<"\" not found."<<std::endl;    
     return std::string();
 }
 
 osgText::Font* osgText::readFontFile(const std::string& filename)
 {
+    if (filename=="") return 0;
+
     std::string foundFile = findFontFile(filename);
     if (foundFile.empty()) return 0;
     
@@ -81,6 +100,33 @@ osgText::Font* osgText::readFontFile(const std::string& filename)
     return 0;
 }
 
+osgText::Font* osgText::readFontStream(std::istream& stream)
+{
+    osg::ref_ptr<osgDB::ReaderWriter::Options> options = new osgDB::ReaderWriter::Options;
+    options->setObjectCacheHint(osgDB::ReaderWriter::Options::CACHE_OBJECTS);
+
+    // there should be a better way to get the FreeType ReaderWriter by name...
+    osgDB::ReaderWriter *reader = osgDB::Registry::instance()->getReaderWriterForExtension("ttf");
+    if (reader == 0) return 0;
+    osgDB::ReaderWriter::ReadResult rr = reader->readObject(stream, options.get());
+    if (rr.error())
+    {
+        osg::notify(osg::WARN) << rr.message() << std::endl;
+        return 0;
+    }
+    if (!rr.validObject()) return 0;
+    
+    osg::Object *object = rr.takeObject();
+
+    // if the object is a font then return it.
+    osgText::Font* font = dynamic_cast<osgText::Font*>(object);
+    if (font) return font;
+
+    // otherwise if the object has zero references then delete it by doing another unref().
+    if (object && object->referenceCount()==0) object->unref();
+    return 0;
+}
+
 
 Font::Font(FontImplementation* implementation):
     _width(16),
@@ -93,6 +139,9 @@ Font::Font(FontImplementation* implementation):
 {
     setImplementation(implementation);
     _texEnv = new osg::TexEnv(osg::TexEnv::BLEND);
+
+    _stateset = new osg::StateSet;
+    _stateset->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
 }
 
 Font::~Font()
