@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -25,9 +25,9 @@
 #include <osg/LineSegment>
 #include <osg/TriangleFunctor>
 #include <osg/Geometry>
+#include <osg/io_utils>
 
 #include <osgUtil/CullVisitor>
-#include <osgUtil/RenderToTextureStage>
 
 #include <float.h>
 #include <algorithm>
@@ -91,11 +91,12 @@ class PrintVisitor : public NodeVisitor
 
 CullVisitor::CullVisitor():
     NodeVisitor(CULL_VISITOR,TRAVERSE_ACTIVE_CHILDREN),
-    _currentRenderGraph(NULL),
+    _currentStateGraph(NULL),
     _currentRenderBin(NULL),
     _computed_znear(FLT_MAX),
     _computed_zfar(-FLT_MAX),
-    _currentReuseRenderLeafIndex(0)
+    _currentReuseRenderLeafIndex(0),
+    _numberOfEncloseOverrideRenderBinDetails(0)
 {
     // _nearFarRatio = 0.000005f;     
 }
@@ -116,6 +117,8 @@ void CullVisitor::reset()
     
     CullStack::reset();
 
+    _numberOfEncloseOverrideRenderBinDetails = 0;
+
     // reset the calculated near far planes.
     _computed_znear = FLT_MAX;
     _computed_zfar = -FLT_MAX;
@@ -129,18 +132,18 @@ void CullVisitor::reset()
 
     _bbCornerNear = (~_bbCornerFar)&7;
     
-    // reset the resuse lists.
-    _currentReuseMatrixIndex = 0;
-    _currentReuseRenderLeafIndex = 0;
-    
-    for(RenderLeafList::iterator itr=_reuseRenderLeafList.begin();
-        itr!=_reuseRenderLeafList.end();
+    // Only reset the RenderLeaf objects used last frame.
+    for(RenderLeafList::iterator itr=_reuseRenderLeafList.begin(),
+        iter_end=_reuseRenderLeafList.begin()+_currentReuseRenderLeafIndex;
+        itr!=iter_end;
         ++itr)
     {
         (*itr)->reset();
     }
     
-    
+    // reset the resuse lists.
+    _currentReuseMatrixIndex = 0;
+    _currentReuseRenderLeafIndex = 0;
 
     _nearPlaneCandidateMap.clear();
 }
@@ -222,72 +225,79 @@ void CullVisitor::popProjectionMatrix()
 template<class matrix_type, class value_type>
 bool _clampProjectionMatrix(matrix_type& projection, double& znear, double& zfar, value_type nearFarRatio)
 {
-    if (zfar>=znear)
+    double epsilon = 1e-6;
+    if (zfar<znear-epsilon)
+    {
+        osg::notify(osg::INFO)<<"_clampProjectionMatrix not applied, invalid depth range, znear = "<<znear<<"  zfar = "<<zfar<<std::endl;
+        return false;
+    }
+    
+    if (zfar<znear+epsilon)
+    {
+        // znear and zfar are too close together and could cause divide by zero problems
+        // late on in the clamping code, so move the znear and zfar apart.
+        double average = (znear+zfar)*0.5;
+        znear = average-epsilon;
+        zfar = average+epsilon;
+        // osg::notify(osg::INFO) << "_clampProjectionMatrix widening znear and zfar to "<<znear<<" "<<zfar<<std::endl;
+    }
+
+    if (fabs(projection(0,3))<epsilon  && fabs(projection(1,3))<epsilon  && fabs(projection(2,3))<epsilon )
+    {
+        // osg::notify(osg::INFO) << "Orthographic matrix before clamping"<<projection<<std::endl;
+
+        value_type delta_span = (zfar-znear)*0.02;
+        if (delta_span<1.0) delta_span = 1.0;
+        value_type desired_znear = znear - delta_span;
+        value_type desired_zfar = zfar + delta_span;
+
+        // assign the clamped values back to the computed values.
+        znear = desired_znear;
+        zfar = desired_zfar;
+
+        projection(2,2)=-2.0f/(desired_zfar-desired_znear);
+        projection(3,2)=-(desired_zfar+desired_znear)/(desired_zfar-desired_znear);
+
+        // osg::notify(osg::INFO) << "Orthographic matrix after clamping "<<projection<<std::endl;
+    }
+    else
     {
 
-        double epsilon = 1e-6;
-        if (fabs(projection(0,3))<epsilon  && fabs(projection(1,3))<epsilon  && fabs(projection(2,3))<epsilon )
-        {
-            // osg::notify(osg::INFO) << "Orthographic matrix before clamping"<<projection<<std::endl;
+        // osg::notify(osg::INFO) << "Persepective matrix before clamping"<<projection<<std::endl;
 
-            value_type delta_span = (zfar-znear)*0.02;
-            if (delta_span<1.0) delta_span = 1.0;
-            value_type desired_znear = znear - delta_span;
-            value_type desired_zfar = zfar + delta_span;
+        //std::cout << "_computed_znear"<<_computed_znear<<std::endl;
+        //std::cout << "_computed_zfar"<<_computed_zfar<<std::endl;
 
-            // assign the clamped values back to the computed values.
-            znear = desired_znear;
-            zfar = desired_zfar;
+        value_type zfarPushRatio = 1.02;
+        value_type znearPullRatio = 0.98;
 
-            projection(2,2)=-2.0f/(desired_zfar-desired_znear);
-            projection(3,2)=-(desired_zfar+desired_znear)/(desired_zfar-desired_znear);
-            
-            // osg::notify(osg::INFO) << "Orthographic matrix after clamping "<<projection<<std::endl;
+        //znearPullRatio = 0.99; 
 
-            return true;
+        value_type desired_znear = znear * znearPullRatio;
+        value_type desired_zfar = zfar * zfarPushRatio;
 
-        }
-        else
-        {
+        // near plane clamping.
+        double min_near_plane = zfar*nearFarRatio;
+        if (desired_znear<min_near_plane) desired_znear=min_near_plane;
 
-            // osg::notify(osg::INFO) << "Persepective matrix before clamping"<<projection<<std::endl;
+        // assign the clamped values back to the computed values.
+        znear = desired_znear;
+        zfar = desired_zfar;
 
-            //std::cout << "_computed_znear"<<_computed_znear<<std::endl;
-            //std::cout << "_computed_zfar"<<_computed_zfar<<std::endl;
+        value_type trans_near_plane = (-desired_znear*projection(2,2)+projection(3,2))/(-desired_znear*projection(2,3)+projection(3,3));
+        value_type trans_far_plane = (-desired_zfar*projection(2,2)+projection(3,2))/(-desired_zfar*projection(2,3)+projection(3,3));
 
-            value_type zfarPushRatio = 1.02;
-            value_type znearPullRatio = 0.98;
-            
-            //znearPullRatio = 0.99; 
-            
-            value_type desired_znear = znear * znearPullRatio;
-            value_type desired_zfar = zfar * zfarPushRatio;
+        value_type ratio = fabs(2.0/(trans_near_plane-trans_far_plane));
+        value_type center = -(trans_near_plane+trans_far_plane)/2.0;
 
-            // near plane clamping.
-            double min_near_plane = zfar*nearFarRatio;
-            if (desired_znear<min_near_plane) desired_znear=min_near_plane;
+        projection.postMult(osg::Matrix(1.0f,0.0f,0.0f,0.0f,
+                                        0.0f,1.0f,0.0f,0.0f,
+                                        0.0f,0.0f,ratio,0.0f,
+                                        0.0f,0.0f,center*ratio,1.0f));
 
-            // assign the clamped values back to the computed values.
-            znear = desired_znear;
-            zfar = desired_zfar;
-
-            value_type trans_near_plane = (-desired_znear*projection(2,2)+projection(3,2))/(-desired_znear*projection(2,3)+projection(3,3));
-            value_type trans_far_plane = (-desired_zfar*projection(2,2)+projection(3,2))/(-desired_zfar*projection(2,3)+projection(3,3));
-
-            value_type ratio = fabs(2.0/(trans_near_plane-trans_far_plane));
-            value_type center = -(trans_near_plane+trans_far_plane)/2.0;
-            
-            projection.postMult(osg::Matrix(1.0f,0.0f,0.0f,0.0f,
-                                            0.0f,1.0f,0.0f,0.0f,
-                                            0.0f,0.0f,ratio,0.0f,
-                                            0.0f,0.0f,center*ratio,1.0f));
-                                            
-            // osg::notify(osg::INFO) << "Persepective matrix after clamping"<<projection<<std::endl;
-
-            return true;
-        }
+        // osg::notify(osg::INFO) << "Persepective matrix after clamping"<<projection<<std::endl;
     }
-    return false;
+    return true;
 }
 
 
@@ -597,7 +607,7 @@ bool CullVisitor::updateCalculatedNearFar(const osg::Matrix& matrix,const osg::D
 
     if (d_far<0.0)
     {
-        // whole object behind the eye point so disguard
+        // whole object behind the eye point so discard
         return false;
     }
 
@@ -746,14 +756,40 @@ void CullVisitor::apply(Geode& node)
             if (!updateCalculatedNearFar(matrix,*drawable,false)) continue;
         }
 
+        // need to track how push/pops there are, so we can unravel the stack correctly.
+        unsigned int numPopStateSetRequired = 0;
+
         // push the geoset's state on the geostate stack.    
         StateSet* stateset = drawable->getStateSet();
-        if (stateset) pushStateSet(stateset);
+        if (stateset)
+        {
+            ++numPopStateSetRequired;
+            pushStateSet(stateset);
+        }
+
+        CullingSet& cs = getCurrentCullingSet();
+        if (!cs.getStateFrustumList().empty())
+        {
+            osg::CullingSet::StateFrustumList& sfl = cs.getStateFrustumList();
+            for(osg::CullingSet::StateFrustumList::iterator itr = sfl.begin();
+                itr != sfl.end();
+                ++itr)
+            {
+                if (itr->second.contains(bb))
+                {
+                    ++numPopStateSetRequired;
+                    pushStateSet(itr->first.get());
+                }
+            }
+        }
 
         if (bb.valid()) addDrawableAndDepth(drawable,&matrix,distance(bb.center(),matrix));
-	else addDrawableAndDepth(drawable,&matrix,0.0f);
+        else addDrawableAndDepth(drawable,&matrix,0.0f);
 
-        if (stateset) popStateSet();
+        for(unsigned int i=0;i< numPopStateSetRequired; ++i)
+        {
+            popStateSet();
+        }
 
     }
 
@@ -873,10 +909,17 @@ void CullVisitor::apply(TexGenNode& node)
     StateSet* node_state = node.getStateSet();
     if (node_state) pushStateSet(node_state);
 
-    RefMatrix& matrix = getModelViewMatrix();
-
-    addPositionedTextureAttribute(node.getTextureUnit(), &matrix,node.getTexGen());
 
+    if (node.getReferenceFrame()==osg::TexGenNode::RELATIVE_RF)
+    {
+        RefMatrix& matrix = getModelViewMatrix();
+        addPositionedTextureAttribute(node.getTextureUnit(), &matrix ,node.getTexGen());
+    }
+    else
+    {
+        addPositionedTextureAttribute(node.getTextureUnit(), 0 ,node.getTexGen());
+    }
+    
     handle_cull_callbacks_and_traverse(node);
 
     // pop the node's state off the geostate stack.    
@@ -952,7 +995,7 @@ void CullVisitor::apply(Projection& node)
     _computed_zfar = -FLT_MAX;
 
 
-    ref_ptr<osg::RefMatrix> matrix = createOrReuseMatrix(node.getMatrix());
+    ref_ptr<RefMatrix> matrix = createOrReuseMatrix(node.getMatrix());
     pushProjectionMatrix(matrix.get());
     
     //osg::notify(osg::INFO)<<"Push projection "<<*matrix<<std::endl;
@@ -1023,6 +1066,173 @@ void CullVisitor::apply(osg::ClearNode& node)
 
 }
 
+void CullVisitor::apply(osg::CameraNode& camera)
+{
+    // push the node's state.
+    StateSet* node_state = camera.getStateSet();
+    if (node_state) pushStateSet(node_state);
+    
+    // set the compute near far mode.
+    ComputeNearFarMode saved_compute_near_far_mode = getComputeNearFarMode();
+    setComputeNearFarMode( camera.getComputeNearFarMode());
+
+
+    RefMatrix& originalModelView = getModelViewMatrix();
+
+    if (camera.getReferenceFrame()==osg::Transform::ABSOLUTE_RF)
+    {
+        pushProjectionMatrix(createOrReuseMatrix(camera.getProjectionMatrix()));
+        pushModelViewMatrix(createOrReuseMatrix(camera.getViewMatrix()));
+    }
+    else if (camera.getTransformOrder()==osg::CameraNode::POST_MULTIPLY)
+    {
+        pushProjectionMatrix(createOrReuseMatrix(getProjectionMatrix()*camera.getProjectionMatrix()));
+        pushModelViewMatrix(createOrReuseMatrix(getModelViewMatrix()*camera.getViewMatrix()));
+    }
+    else // pre multiply 
+    {
+        pushProjectionMatrix(createOrReuseMatrix(camera.getProjectionMatrix()*getProjectionMatrix()));
+        pushModelViewMatrix(createOrReuseMatrix(camera.getViewMatrix()*getModelViewMatrix()));
+    }
+
+
+    if (camera.getRenderOrder()==osg::CameraNode::NESTED_RENDER)
+    {
+        handle_cull_callbacks_and_traverse(camera);
+    }
+    else
+    {
+        // set up lighting.
+        // currently ignore lights in the scene graph itself..
+        // will do later.
+        osgUtil::RenderStage* previous_stage = getCurrentRenderBin()->getStage();
+
+        unsigned int contextID = getState() ? getState()->getContextID() : 0;
+
+        // use render to texture stage.
+        // create the render to texture stage.
+        osg::ref_ptr<osgUtil::RenderStage> rtts = dynamic_cast<osgUtil::RenderStage*>(camera.getRenderingCache(contextID));
+        if (!rtts)
+        {
+            OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*(camera.getDataChangeMutex()));
+        
+            rtts = new osgUtil::RenderStage;
+            rtts->setCameraNode(&camera);
+
+            if (camera.getDrawBuffer() != GL_NONE)
+            {
+                rtts->setDrawBuffer(camera.getDrawBuffer());
+            }
+            else
+            {
+                // inherit draw buffer from above.
+                rtts->setDrawBuffer(previous_stage->getDrawBuffer());
+            }
+
+            if (camera.getReadBuffer() != GL_NONE)
+            {
+                rtts->setReadBuffer(camera.getReadBuffer());
+            }
+            else
+            {
+                // inherit read buffer from above.
+                rtts->setReadBuffer(previous_stage->getReadBuffer());
+            }
+
+            camera.setRenderingCache(contextID, rtts.get());
+        }
+        else
+        {
+            // reusing render to texture stage, so need to reset it to empty it from previous frames contents.
+            rtts->reset();
+        }
+        
+
+
+        // set up the background color and clear mask.
+        rtts->setClearColor(camera.getClearColor());
+        rtts->setClearMask(camera.getClearMask());
+        
+        // set the color mask.
+        osg::ColorMask* colorMask = camera.getColorMask()!=0 ? camera.getColorMask() : previous_stage->getColorMask();
+        rtts->setColorMask(colorMask);
+
+        // set up the viewport.
+        osg::Viewport* viewport = camera.getViewport()!=0 ? camera.getViewport() : previous_stage->getViewport();
+        rtts->setViewport( viewport );
+        
+
+        // set up to charge the same PositionalStateContainer is the parent previous stage.
+        osg::Matrix inhertiedMVtolocalMV;
+        inhertiedMVtolocalMV.invert(originalModelView);
+        inhertiedMVtolocalMV.postMult(getModelViewMatrix());
+        rtts->setInheritedPositionalStateContainerMatrix(inhertiedMVtolocalMV);
+        rtts->setInheritedPositionalStateContainer(previous_stage->getPositionalStateContainer());
+
+        // record the render bin, to be restored after creation
+        // of the render to text
+        osgUtil::RenderBin* previousRenderBin = getCurrentRenderBin();
+
+        // set the current renderbin to be the newly created stage.
+        setCurrentRenderBin(rtts.get());
+
+        // set the cull traversal mask of camera node
+        osg::Node::NodeMask saved_mask = getCullMask();
+        if (camera.getInheritanceMask() & CULL_MASK)
+        {
+             setTraversalMask(camera.getCullMask());
+        }
+
+        // traverse the subgraph
+        {
+            handle_cull_callbacks_and_traverse(camera);
+        }
+
+        // restore the cull traversal mask of camera node
+        if (camera.getInheritanceMask() & CULL_MASK)
+        {
+            setTraversalMask(saved_mask);
+        }
+
+        // restore the previous renderbin.
+        setCurrentRenderBin(previousRenderBin);
+     
+
+        if (rtts->getStateGraphList().size()==0 && rtts->getRenderBinList().size()==0)
+        {
+            // getting to this point means that all the subgraph has been
+            // culled by small feature culling or is beyond LOD ranges.
+        }
+
+
+        // and the render to texture stage to the current stages
+        // dependancy list.
+        switch(camera.getRenderOrder())
+        {
+            case osg::CameraNode::PRE_RENDER :
+                getCurrentRenderBin()->getStage()->addPreRenderStage(rtts.get());
+                break;
+            default :
+                getCurrentRenderBin()->getStage()->addPostRenderStage(rtts.get());
+                break;
+        }
+
+    }
+
+    // restore the previous model view matrix.
+    popModelViewMatrix();
+
+    // restore the previous model view matrix.
+    popProjectionMatrix();
+
+    // restore the previous compute near far mode
+    setComputeNearFarMode(saved_compute_near_far_mode);
+
+    // pop the node's state off the render graph stack.    
+    if (node_state) popStateSet();
+
+}
+
 void CullVisitor::apply(osg::OccluderNode& node)
 {
     // need to check if occlusion node is in the occluder
