@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -35,8 +35,6 @@ Tesselator::~Tesselator()
     reset();
 }
 
-
-
 void Tesselator::beginTesselation()
 {
     reset();
@@ -48,6 +46,7 @@ void Tesselator::beginTesselation()
     gluTessCallback(_tobj, GLU_TESS_END_DATA,    (GLU_TESS_CALLBACK) endCallback);
     gluTessCallback(_tobj, GLU_TESS_COMBINE_DATA,(GLU_TESS_CALLBACK) combineCallback);
     gluTessCallback(_tobj, GLU_TESS_ERROR_DATA,  (GLU_TESS_CALLBACK) errorCallback);
+    if (tessNormal.length()>0.0) gluTessNormal(_tobj, tessNormal.x(), tessNormal.y(), tessNormal.z());
 
     gluTessBeginPolygon(_tobj,this);
 }    
@@ -145,7 +144,7 @@ class InsertNewVertices : public osg::ArrayVisitor
         virtual void apply(osg::UByteArray& ba) { apply_imp(ba,GLubyte(0)); }
         virtual void apply(osg::UShortArray& ba) { apply_imp(ba,GLushort(0)); }
         virtual void apply(osg::UIntArray& ba) { apply_imp(ba,GLuint(0)); }
-        virtual void apply(osg::UByte4Array& ba) { apply_imp(ba,UByte4()); }
+        virtual void apply(osg::Vec4ubArray& ba) { apply_imp(ba,Vec4ub()); }
         virtual void apply(osg::FloatArray& ba) { apply_imp(ba,float(0)); }
         virtual void apply(osg::Vec2Array& ba) { apply_imp(ba,Vec2()); }
         virtual void apply(osg::Vec3Array& ba) { apply_imp(ba,Vec3()); }
@@ -425,7 +424,7 @@ void Tesselator::handleNewVertices(osg::Geometry& geom,VertexPtrToIndexMap &vert
         osg::Vec3Array* normals = NULL;
         if (geom.getNormalBinding()==osg::Geometry::BIND_PER_VERTEX)
         {
-            normals = geom.getNormalArray();
+            normals = dynamic_cast<osg::Vec3Array*>(geom.getNormalArray());
         }
         
         typedef std::vector<osg::Array*> ArrayList;
@@ -591,7 +590,7 @@ void Tesselator::reduceArray(osg::Array * cold, const unsigned int nnu)
                                         }
             break;
         default: // should also handle:ArrayType' ByteArrayType' ShortArrayType' IntArrayType' 
-        // `UShortArrayType'  `UIntArrayType'  `UByte4ArrayType'  `FloatArrayType' 
+        // `UShortArrayType'  `UIntArrayType'  `Vec4ubArrayType'  `FloatArrayType' 
             break;
         }
     }
@@ -668,7 +667,7 @@ void Tesselator::collectTesselation(osg::Geometry &geom)
         if (geom.getNormalBinding()==osg::Geometry::BIND_PER_PRIMITIVE ||
             geom.getNormalBinding()==osg::Geometry::BIND_PER_PRIMITIVE_SET)
         {
-            normals = geom.getNormalArray(); // GWM Sep 2002
+            normals = dynamic_cast<osg::Vec3Array*>(geom.getNormalArray()); // GWM Sep 2002
         }
         // GWM Dec 2003 - needed to add colours for extra facets
         osg::Vec4Array* cols4 = NULL; // GWM Dec 2003 colours are vec4
