@@ -1,3 +1,4 @@
+#include <iostream>
 #include <string>
 #include <Producer/CameraConfig>
 #include <Producer/CameraGroup>
@@ -19,7 +20,10 @@ int main( int argc, char **argv )
 	{
         Producer::ref_ptr<Producer::CameraConfig> cfg = new Producer::CameraConfig;
 		if( cfg->parseFile( configFileName.c_str() ) == false )
-			return -1;
+        {
+            std::cerr << argv[0] << ": Failed to parse config file " << configFileName << std::endl;
+			return 1;
+        }
 		cg  = new Producer::CameraGroup( cfg.get() );
 	}
 
