@@ -86,10 +86,28 @@ void Camera::frame( bool doSwap )
     if( !_enabled ) 
         return;
 
+    if( _updateCallback.valid() )
+        (*((_updateCallback).get()))(*this);
+
+
+    if( preFrameCallbacks.size() )
+    {
+        std::vector <ref_ptr<Callback> >::iterator p;
+        for( p = preFrameCallbacks.begin(); p != preFrameCallbacks.end(); p++ )
+            (*((*p).get()))(*this);
+    }
+
     if( _instrumented )
         _frameInstrumented( doSwap );
     else
         _frame(doSwap);
+
+    if( postFrameCallbacks.size() )
+    {
+        std::vector <ref_ptr<Callback> >::iterator p;
+        for( p = postFrameCallbacks.begin(); p != postFrameCallbacks.end(); p++ )
+            (*((*p).get()))(*this);
+    }
 }
 
 
@@ -109,12 +127,29 @@ void Camera::_frameInstrumented( bool doSwap )
     memset( stamps, 0, sizeof(stamps));
 
     stamps[BeginCameraFrame] = _timer.tick();
+
+    if( _sh->useAutoView() )
+    {
+        if( _lens->getAutoAspect() )
+        {
+            int x, y;
+            unsigned int width, height;
+            _rs->getWindowRectangle(x,y,width,height);
+            double ar = (float(width) * (_projrectRight - _projrectLeft))/
+                        (float(height) * (_projrectTop - _projrectBottom ));
+            _lens->setAspectRatio( ar );
+        }
+    
+        _lens->apply( _offset._xshear , _offset._yshear);
+    }
+
     if( _sh->frame( *this ) == true )
     {
         stamps[EndCameraFrame] = _timer.tick();
         return;
     }
 
+
     stamps[BeginCull] = _timer.tick();
     if( preCullCallbacks.size() )
     {
@@ -154,23 +189,13 @@ void Camera::_frameInstrumented( bool doSwap )
     
     
     stamps[BeginDraw] = _timer.tick();
+
     stamps[BeginClear] = _timer.tick();
     _sh->clear( *this );
     stamps[EndClear] = _timer.tick();
 
     if( _sh->useAutoView() )
     {
-        if( _lens->getAutoAspect() )
-        {
-            int x, y;
-            unsigned int width, height;
-            _rs->getWindowRectangle(x,y,width,height);
-            double ar = (float(width) * (_projrectRight - _projrectLeft))/
-                        (float(height) * (_projrectTop - _projrectBottom ));
-            _lens->setAspectRatio( ar );
-        }
-    
-        _lens->apply( _offset._xshear , _offset._yshear);
         applyView();
     }
 
@@ -230,6 +255,21 @@ void Camera::_frame( bool doSwap )
 
     ++_frameCount;
 
+    if( _sh->useAutoView() )
+    {
+        if( _lens->getAutoAspect() )
+        {
+            int x, y;
+            unsigned int width, height;
+            _rs->getWindowRectangle(x,y,width,height);
+            double ar = (float(width) * (_projrectRight - _projrectLeft))/
+                        (float(height) * (_projrectTop - _projrectBottom ));
+            _lens->setAspectRatio( ar );
+        }
+    
+        _lens->apply( _offset._xshear , _offset._yshear);
+    }
+
     if( _sh->frame( *this ) == true )
     {
         return;
@@ -251,25 +291,17 @@ void Camera::_frame( bool doSwap )
     }
 
     _rs->makeCurrent();
+
     if( _block_on_vsync ) 
     {
         _rs->sync();
         _syncTick = _timer.tick();
     }
+
     _sh->clear( *this );
+
     if( _sh->useAutoView() )
     {
-        if( _lens->getAutoAspect() )
-        {
-            int x, y;
-            unsigned int width, height;
-            _rs->getWindowRectangle(x,y,width,height);
-            double ar = (float(width) * (_projrectRight - _projrectLeft))/
-                        (float(height) * (_projrectTop - _projrectBottom ));
-            _lens->setAspectRatio( ar );
-        }
-    
-        _lens->apply( _offset._xshear, _offset._yshear );
         applyView();
     }
 
@@ -809,25 +841,25 @@ void Camera::Lens::generateMatrix(float xshear, float yshear, Matrix::value_type
 
         break;
 
-	case Manual:
+    case Manual:
 
-		memcpy( matrix, _matrix, sizeof(Matrix::value_type[16]));
+        memcpy( matrix, _matrix, sizeof(Matrix::value_type[16]));
 
-		if(xshear || yshear)
-		{
-			if (matrix[3]!=0.0 || matrix[7]!=0.0 || matrix[11]!=0.0 || matrix[15]!=1.0)
-			{
-				// It's not an orthographic matrix so just assume a perspective shear
-				matrix[ 8] += -xshear;
-				matrix[ 9] += -yshear;
-			} 
+        if(xshear || yshear)
+        {
+            if (matrix[3]!=0.0 || matrix[7]!=0.0 || matrix[11]!=0.0 || matrix[15]!=1.0)
+            {
+                // It's not an orthographic matrix so just assume a perspective shear
+                matrix[ 8] += -xshear;
+                matrix[ 9] += -yshear;
+            } 
             else
-			{
-				 matrix[12] += xshear;
-				 matrix[13] += yshear;
-			}
-		}
-		break;
+            {
+                 matrix[12] += xshear;
+                 matrix[13] += yshear;
+            }
+        }
+        break;
     }
 }
 
@@ -872,6 +904,12 @@ int Camera::cancel()
     return 0;    
 }
 
+void Camera::advance()
+{
+    _rs->makeCurrent();
+    _rs->swapBuffers();
+}
+
 void Camera::run( void )
 {
     if( !_syncBarrier.valid() || !_frameBarrier.valid() )
@@ -954,3 +992,4 @@ bool Camera::_removeCallback( std::vector < ref_ptr<Callback> > &callbackList, C
     callbackList.erase( p );
     return true;
 }
+
