@@ -71,7 +71,7 @@ class KeyboardMouseImplementationBase : public Referenced
         virtual void setAutoRepeatMode( bool ) {}
         virtual bool getAutoRepeatMode() {return true;}
 
-        void cancel()
+        virtual void cancel()
         {
             _canceled = true;
         }
@@ -203,7 +203,7 @@ class KeyboardMouseImplementationBase : public Referenced
 #endif
 
                 RenderSurface::InputRectangle ir = _rs->getInputRectangle();
-                _mx = ir.left() + (float(wx)/float(windowWidth))*ir.width();
+                _mx = ir.left() + (float(wx-positionLeft)/float(windowWidth))*ir.width();
                 _my = ir.bottom() + (float(windowHeight-(wy-positionTop+1))/float(windowHeight-1))*ir.height();
 
             }
@@ -327,7 +327,7 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
 
             if( _inputArea != NULL )
             {
-                 for( int i = 0; i < _inputArea->getNumWindows(); i++ )
+                 for( unsigned int i = 0; i < _inputArea->getNumWindows(); i++ )
                  {
                      Window win = _inputArea->getWindow(i);
                      XSelectInput( dpy, win,  _inputMask );
@@ -363,9 +363,13 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
 
             // This should only be called from the run() thread if 
             // we are multi-threaded
+            /*
+             * commenting out these calls as they are suspect 
+             * 
             XSync(dpy,0);
             XCloseDisplay(dpy);
             cancel();
+            */
         }
 
 
@@ -566,16 +570,14 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
 
 
         bool update( KeyboardMouseCallback &cb, bool block=false)
-        {/* no blocking for now
-            if( block ) 
-            {
-                q->waitWhileEmpty();
-            }
-            */
-            //HACK!!! - This is a work-around for background threads that continuously
-            // poll the input queue as the above "waitWhileEmpty()" is not fully functional.
+        {
             if( block )
-                Sleep(16);
+            {
+                //Wait until at least one event queue has an event or cancel signal is set
+                DWORD waitResult = WaitForMultipleObjects(signals.size(),&signals.front(),FALSE,INFINITE);
+                //Check if cancel event was signaled
+                if(waitResult - WAIT_OBJECT_0 == signals.size()-1) return true;
+            }
             std::vector< Producer::ref_ptr<EventQueue> >::iterator p;
             for( p = qs.begin(); p != qs.end(); p++ )
             {
@@ -583,106 +585,119 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
                 while( ! q->empty() )
                 {
                     Event *ev = q->front().get();
-                    switch( ev->type() )
+                    if(!cb.handle(*ev)) //Allow callback to handle raw event
                     {
-                    case Event::WindowConfig :
-                      {
-                        WindowConfigEvent *wc = dynamic_cast<WindowConfigEvent *>(ev);
-                        if( wc != NULL )
+                        switch( ev->type() )
                         {
-                            // NOt using these now...
-                        }
-                      }
-                      break;
-        
-                    case Event::KeyPress :
-                      {
-                        KeyPressEvent *kp = dynamic_cast<KeyPressEvent *>(ev);
-                        if( kp != NULL )
-                        {
-                            KeyCharacter kchar = KeyCharacter(kp->key());
-                            if( kchar & 0xFF00 )
-                                cb.specialKeyPress( kchar );
-                            else
-                                cb.keyPress( kchar );
-                        }
-                      }
-                      break;
-
-                    case Event::KeyRelease :
-                      {
-                        KeyReleaseEvent *kr = dynamic_cast<KeyReleaseEvent *>(ev);
-                        if( kr != NULL )
-                        {
-                            KeyCharacter kchar = KeyCharacter(kr->key());
-                            if( kchar & 0xFF00 )
-                                cb.specialKeyRelease(kchar);
-                            else
-                                cb.keyRelease(kchar);
-                        }
-                      }
-                      break;
-
-                    case Event::ButtonPress :
-                      {
-                        ButtonPressEvent *bp = dynamic_cast<ButtonPressEvent *>(ev);
-                        transformMouseMotion( bp->window(), bp->mx(), bp->my() );
-                        _mbutton |= (1<<(bp->button()-1));
-
-                        if( bp != NULL )
-                            cb.buttonPress( _mx, _my, bp->button() );
-                      }
-                      break;
-
-                    case Event::ButtonRelease :
-                      {
-                        ButtonReleaseEvent *br = dynamic_cast<ButtonReleaseEvent *>(ev);
-                        if( br != NULL )
-                        {
-                            transformMouseMotion( br->window(), br->mx(), br->my() );
-                            _mbutton &= ~(1<<(br->button()-1));
-
-                            cb.buttonRelease( _mx, _my, br->button() );
-                        }
-                      }
-                      break;
-
-                    case Event::MouseScroll:
-                        {
-                            MouseScrollEvent *mse = dynamic_cast<MouseScrollEvent *>(ev);
-                            if( mse != NULL )
+                        case Event::WindowConfig :
+                          {
+                            WindowConfigEvent *wc = dynamic_cast<WindowConfigEvent *>(ev);
+                            if( wc != NULL )
                             {
-                                cb.mouseScroll(mse->scrollingMotion() );
+                                // NOt using these now...
                             }
+                          }
+                          break;
+            
+                        case Event::KeyPress :
+                          {
+                            KeyPressEvent *kp = dynamic_cast<KeyPressEvent *>(ev);
+                            if( kp != NULL )
+                            {
+                                KeyCharacter kchar = KeyCharacter(kp->key());
+                                if( kchar & 0xFF00 )
+                                    cb.specialKeyPress( kchar );
+                                else
+                                    cb.keyPress( kchar );
+                            }
+                          }
+                          break;
+    
+                        case Event::KeyRelease :
+                          {
+                            KeyReleaseEvent *kr = dynamic_cast<KeyReleaseEvent *>(ev);
+                            if( kr != NULL )
+                            {
+                                KeyCharacter kchar = KeyCharacter(kr->key());
+                                if( kchar & 0xFF00 )
+                                    cb.specialKeyRelease(kchar);
+                                else
+                                    cb.keyRelease(kchar);
+                            }
+                          }
+                          break;
+    
+                        case Event::ButtonPress :
+                          {
+                            ButtonPressEvent *bp = dynamic_cast<ButtonPressEvent *>(ev);
+                            transformMouseMotion( bp->window(), bp->mx(), bp->my() );
+                            _mbutton |= (1<<(bp->button()-1));
+    
+                            if( bp != NULL )
+                                cb.buttonPress( _mx, _my, bp->button() );
+                          }
+                          break;
+    
+                        case Event::ButtonRelease :
+                          {
+                            ButtonReleaseEvent *br = dynamic_cast<ButtonReleaseEvent *>(ev);
+                            if( br != NULL )
+                            {
+                                transformMouseMotion( br->window(), br->mx(), br->my() );
+                                _mbutton &= ~(1<<(br->button()-1));
+    
+                                cb.buttonRelease( _mx, _my, br->button() );
+                            }
+                          }
+                          break;
+    
+                        case Event::MouseScroll:
+                            {
+                                MouseScrollEvent *mse = dynamic_cast<MouseScrollEvent *>(ev);
+                                if( mse != NULL )
+                                {
+                                    cb.mouseScroll(mse->scrollingMotion() );
+                                }
+                            }
+                            break;
+    
+                        case Event::MouseMotion :
+                          {
+                            MouseMotionEvent *mm = dynamic_cast<MouseMotionEvent *>(ev);
+                            transformMouseMotion( mm->window(), mm->mx(), mm->my() );
+                            if( mm != NULL )
+                            {
+                                if( _mbutton )
+                                    cb.mouseMotion(_mx, _my);
+                                else
+                                    cb.passiveMouseMotion(_mx, _my);
+                             }
+                          }
+                          break;
+                          
+                        case Event::Timer :
+                              // Nothing to do, really.
+                            break;
+    
+                        case Event::Shutdown:
+                            cb.shutdown();
+                            break;
+    
+                        case Event::DoubleButtonPress :
+                          {
+                            ButtonPressEvent *bp = dynamic_cast<ButtonPressEvent *>(ev);
+                            transformMouseMotion( bp->window(), bp->mx(), bp->my() );
+                            _mbutton |= (1<<(bp->button()-1));
+    
+                            if( bp != NULL )
+                                cb.doubleButtonPress( _mx, _my, bp->button() );
+                          }
+                            break;
+    
+                        case Event::ButtonMotion :
+                            break;
+                            
                         }
-                        break;
-
-                    case Event::MouseMotion :
-                      {
-                        MouseMotionEvent *mm = dynamic_cast<MouseMotionEvent *>(ev);
-                        transformMouseMotion( mm->window(), mm->mx(), mm->my() );
-                        if( mm != NULL )
-                        {
-                            if( _mbutton )
-                                cb.mouseMotion(_mx, _my);
-                            else
-                                cb.passiveMouseMotion(_mx, _my);
-                         }
-                      }
-                      break;
-                      
-                    case Event::Timer :
-                          // Nothing to do, really.
-                        break;
-
-                    case Event::Shutdown:
-                        cb.shutdown();
-                        break;
-
-                    case Event::DoubleButtonPress :
-                    case Event::ButtonMotion :
-                        break;
-                        
                     }
                     q->pop_front();
                 }
@@ -717,17 +732,28 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
                     RenderSurface *rs = _inputArea->getRenderSurface(i);
                     EventQueue *q = rs->selectInput(mask);
                     qs.push_back(q);
+                    signals.push_back(q->getSignal());
                 }
             }
             else
             {
                 EventQueue *q = _rs->selectInput(mask);
                 qs.push_back(q);
+                signals.push_back(q->getSignal());
             }
-               
+
+            cancelSignal = CreateEvent(NULL,TRUE,FALSE,NULL);
+            signals.push_back(cancelSignal);
+
             _mbutton = 0;
         }
 
+        virtual void cancel()
+        {
+            SetEvent(cancelSignal);
+            KeyboardMouseImplementationBase::cancel();  
+        }
+
         void fini()
         {
             cancel();
@@ -737,13 +763,15 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
     protected:
 
         virtual ~KeyboardMouseImplementation() 
-        {}
+        { CloseHandle(cancelSignal); }
 
     private:
 
 
 
         std::vector< Producer::ref_ptr<EventQueue> > qs;
+        std::vector< HANDLE > signals;
+        HANDLE cancelSignal;
 };
 
 }
@@ -767,7 +795,6 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
 
         bool handleMouseEvent(KeyboardMouseCallback &cb, EventRef theEvent)
         {
-            
             // mouse down event                         
             Point wheresMyMouse;
             GetEventParameter (theEvent, kEventParamMouseLocation, typeQDPoint, NULL, sizeof(wheresMyMouse), NULL, &wheresMyMouse);
@@ -785,38 +812,117 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
                 EventMouseButton mouseButton;
                 GetEventParameter (theEvent, kEventParamMouseButton, typeMouseButton, NULL, sizeof(mouseButton), NULL, &mouseButton);
 
-                if (mouseButton==1)
-                {
-                    UInt32 modKey = GetCurrentKeyModifiers();
-                    if (modKey & 0x100)
+                // swap right and middle buttons so that middle button is 2, right button is 3.
+                if (mouseButton==3) mouseButton = 2;
+                else if (mouseButton==2) mouseButton = 3;
+                
+                // check tablet pointer device and map it to a musebutton
+                TabletProximityRec    theTabletRecord;    // The Tablet Proximity Record
+                // Extract the Tablet Proximity reccord from the event.
+                if(noErr == GetEventParameter(theEvent, kEventParamTabletProximityRec,
+                                              typeTabletProximityRec, NULL,
+                                              sizeof(TabletProximityRec),
+                                              NULL, (void *)&theTabletRecord)) {
+                    
+                    KeyboardMouseCallback::TabletPointerType pointerType;
+                    switch(theTabletRecord.pointerType)
                     {
-                        mouseButton = 3;
-                    }
-                    else if (modKey & 0x800)
-                    {
-                        mouseButton = 2;
+                        case 1: // pen
+                            pointerType = KeyboardMouseCallback::Pen;
+                            break;
+                            
+                        case 2: // puck
+                            pointerType = KeyboardMouseCallback::Puck;
+                            break;
+                            
+                        case 3: //eraser
+                            pointerType = KeyboardMouseCallback::Eraser;
+                            break;
+
+                        default:
+                           pointerType = KeyboardMouseCallback::Unknown;
+                           break;
                     }
+                    
+                    cb.penProximity(pointerType, (theTabletRecord.enterProximity != 0));
                 }
-                else
-                {
-                    if (mouseButton==2) mouseButton=3;
-                    else if (mouseButton==3) mouseButton=2; 
-                }
+
                 switch(GetEventKind(theEvent))
                 {
                     case kEventMouseDown:
-                        cb.buttonPress(_mx,_my, mouseButton);
+                        if (mouseButton==1) _actual_button1Pressed = true;
+                        if (mouseButton==2) _actual_button2Pressed = true;
+                        if (mouseButton==3) _actual_button3Pressed = true;
+                        updateMouseButtonState(cb);
                         break;
                     case kEventMouseUp:
-                        cb.buttonRelease(_mx,_my, mouseButton);
+                        if (mouseButton==1) _actual_button1Pressed = false;
+                        if (mouseButton==2) _actual_button2Pressed = false;
+                        if (mouseButton==3) _actual_button3Pressed = false;
+                        updateMouseButtonState(cb);
                         break;
                     case kEventMouseDragged:
+                        {
+                            // get pressure from the pen, only when mouse/pen is dragged
+                            TabletPointRec    theTabletRecord;
+                            if(noErr == GetEventParameter(theEvent,  kEventParamTabletPointRec, typeTabletPointRec, NULL, 
+                                            sizeof(TabletPointRec), NULL, (void *)&theTabletRecord)) {
+                            
+                                cb.penPressure(theTabletRecord.pressure / 65535.0f);
+                    
+                            }
+                        }
                         break;
+                        
                     case kEventMouseMoved:
                         break;
+                        
+                    // mouse with scroll-wheels
+                    case kEventMouseWheelMoved:
+                        {
+                            EventMouseWheelAxis axis;
+                            SInt32 delta;
+                            if (noErr == GetEventParameter( theEvent, kEventParamMouseWheelAxis, typeMouseWheelAxis, NULL, sizeof(axis), NULL, &axis )) {
+                                if (noErr == GetEventParameter( theEvent, kEventParamMouseWheelDelta, typeLongInteger, NULL, sizeof(delta), NULL, &delta )) {
+                                    switch (axis) {
+                                        case kEventMouseWheelAxisX:
+                                            cb.mouseScroll( (delta > 0) ? KeyboardMouseCallback::ScrollRight : KeyboardMouseCallback::ScrollLeft);
+                                            break;
+                                        case kEventMouseWheelAxisY:
+                                            cb.mouseScroll( (delta < 0) ? KeyboardMouseCallback::ScrollDown : KeyboardMouseCallback::ScrollUp);
+                                            break;
+                                    }
+                                }
+                            }
+                        }
+                        break;
+                    
+                    // new trackpads and mighty mouse, (not officially documented, see http://developer.apple.com/qa/qa2005/qa1453.html )
+                    case 11:
+                        {
+                            enum
+                                {
+                                kEventParamMouseWheelSmoothVerticalDelta       = 'saxy', // typeSInt32
+                                kEventParamMouseWheelSmoothHorizontalDelta     = 'saxx', // typeSInt32
+                                };
+                            
+                            SInt32 scroll_delta_x = 0;
+                            SInt32 scroll_delta_y = 0;
+                            OSErr err = noErr;
+                            err = GetEventParameter( theEvent, kEventParamMouseWheelSmoothVerticalDelta, typeLongInteger, NULL, sizeof(scroll_delta_y), NULL, &scroll_delta_y );
+                            err = GetEventParameter( theEvent, kEventParamMouseWheelSmoothHorizontalDelta, typeLongInteger, NULL, sizeof(scroll_delta_x), NULL, &scroll_delta_x );
+                            
+                            if ((scroll_delta_x != 0) || (scroll_delta_y != 0)) {
+                                cb.mouseScroll2D( scroll_delta_x, scroll_delta_y);
+                            }
+                        }
+                        break;
+                    
                     default:
                         return false;
                 }
+                
+                updateMouseButtonState(cb);
             
                 if( _ox != wheresMyMouse.h || _oy != wheresMyMouse.v )
                 {
@@ -836,11 +942,15 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
         
         bool handleKeyboardEvent(KeyboardMouseCallback &cb, EventRef theEvent)
         {
+            OSStatus status; 
+
             UInt32 modifierKeys;
             GetEventParameter (theEvent,kEventParamKeyModifiers,typeUInt32, NULL,sizeof(modifierKeys), NULL,&modifierKeys);
-            
+
             UInt32 dataSize;
-            GetEventParameter( theEvent, kEventParamKeyUnicodes, typeUnicodeText, NULL, 0, &dataSize, NULL );
+            /* jbw check return status so that we don't allocate a huge array */
+            status = GetEventParameter( theEvent, kEventParamKeyUnicodes, typeUnicodeText, NULL, 0, &dataSize, NULL );
+            if (status != noErr) return false;
             if (dataSize<=1) return false;
             
             UniChar* uniChars = new UniChar[dataSize+1];
@@ -867,7 +977,7 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
                      break;
                 }
                 case kEventRawKeyUp:
-                {
+                {                 
                      if( keychar & 0xFF00 )
                         cb.specialKeyRelease(keychar);
                      else
@@ -875,16 +985,20 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
                      break;
                 }
                 case kEventRawKeyModifiersChanged:
+                     updateMouseButtonState(cb);
                      break;
                 case kEventHotKeyPressed:
                      break;
                 case kEventHotKeyReleased:
                      break;
                 default:
-                    break;
+                     break;
             
             }
-           return true;
+
+            delete[] uniChars;
+
+            return true;
         }
 
         bool update( KeyboardMouseCallback &cb, bool block=false)
@@ -895,11 +1009,12 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
             {
                 switch(GetEventClass(theEvent))
                 {
+                    case kEventClassTablet:
                     case kEventClassMouse:
                         if (!handleMouseEvent(cb, theEvent)) {}
                             SendEventToEventTarget (theEvent, theTarget);
                         break;
-                        
+                   
                     case kEventClassKeyboard:
                         handleKeyboardEvent(cb, theEvent);
                         break;
@@ -920,7 +1035,19 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
 
         virtual void init()    
         {
-            _lastButtonPressed = 0;
+               
+            _actual_button1Pressed = false;
+            _actual_button2Pressed = false;
+            _actual_button3Pressed = false;
+
+            _computed_button1Pressed = false;
+            _computed_button2Pressed = false;
+            _computed_button3Pressed = false;
+
+            _previous_button1Pressed = false;
+            _previous_button2Pressed = false;
+            _previous_button3Pressed = false;
+
         }
 
         virtual void fini()
@@ -928,11 +1055,75 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
         }
 
 
+        void updateMouseButtonState(KeyboardMouseCallback &cb)
+        {
+            _computed_button2Pressed = _actual_button2Pressed;
+            _computed_button3Pressed = _actual_button3Pressed;
+
+            if (_actual_button1Pressed)
+            {
+                UInt32 modKey = GetCurrentKeyModifiers();
+                if (modKey & 0x100)
+                {
+                    _computed_button3Pressed = true;
+                    _computed_button1Pressed = false;
+                }
+                else if (modKey & 0x800)
+                {
+                    _computed_button2Pressed = true;
+                    _computed_button1Pressed = false;
+                }
+                else
+                {
+                    _computed_button1Pressed = true;
+                }
+            }
+            else
+            {
+                _computed_button1Pressed = false;
+            }
+            
+            if (_previous_button1Pressed != _computed_button1Pressed)
+            {
+                if (_computed_button1Pressed) cb.buttonPress(_mx,_my, 1);
+                else cb.buttonRelease(_mx,_my, 1);
+                
+                _previous_button1Pressed = _computed_button1Pressed;
+            }
+            
+            if (_previous_button2Pressed != _computed_button2Pressed)
+            {
+                if (_computed_button2Pressed) cb.buttonPress(_mx,_my, 2);
+                else cb.buttonRelease(_mx,_my, 2);
+                
+                _previous_button2Pressed = _computed_button2Pressed;
+            }
+            
+            if (_previous_button3Pressed != _computed_button3Pressed)
+            {
+                if (_computed_button3Pressed) cb.buttonPress(_mx,_my, 3);
+                else cb.buttonRelease(_mx,_my, 3);
+                
+                _previous_button3Pressed = _computed_button3Pressed;
+            }
+            
+        }
+
     private:
 
         short _ox, _oy;
-        int _lastButtonPressed;
+        
+        bool _actual_button1Pressed;
+        bool _actual_button2Pressed;
+        bool _actual_button3Pressed;
 
+        bool _computed_button1Pressed;
+        bool _computed_button2Pressed;
+        bool _computed_button3Pressed;
+
+        bool _previous_button1Pressed;
+        bool _previous_button2Pressed;
+        bool _previous_button3Pressed;
 };
 
 }
@@ -966,7 +1157,8 @@ KeyboardMouse::~KeyboardMouse()
             _implementation->cancel();
     }
     
-    cancel();
+    // This is the 
+    //cancel();
 
     while (isRunning())
     {
@@ -1121,8 +1313,6 @@ void KeyboardMouse::run()
         testCancel();
     }
 
-    // We never really get here.  Our thread is canceled before 
-    // this executes.  Thats... OK.
     _implementation->fini();
 }
 
