@@ -35,12 +35,12 @@ unsigned int RenderSurface::getNumberOfScreens(void) { return 1; }
 
 RenderSurface::Client::Client(unsigned long mask): _mask(mask)
 {
-        q = new EventQueue;
+    q = new EventQueue;
 }
 
 void RenderSurface::Client::queue( ref_ptr<Event> ev ) 
 { 
-        q->push_back( ev ); 
+    q->push_back( ev ); 
 }
 
 
@@ -361,9 +361,15 @@ LONG WINAPI RenderSurface::proc( Window hWnd, UINT uMsg, WPARAM wParam, LPARAM l
     return lRet;
 } 
 
-bool RenderSurface::realize( VisualChooser *vc, GLContext )
+bool RenderSurface::realize( VisualChooser *vc, GLContext sharedGLContext )
 {
     if( _realized ) return _realized;
+
+    if( sharedGLContext != 0L )
+        _sharedGLContext = sharedGLContext;
+    else
+        _sharedGLContext = _globallySharedGLContext;
+
     if( vc != NULL )
         _visualChooser = vc;
     if( _useConfigEventThread && _ownWindow && _drawableType == DrawableType_Window )
@@ -372,10 +378,18 @@ bool RenderSurface::realize( VisualChooser *vc, GLContext )
         _threadReady->block();
     }
     else
-        _init();
+    {
+      if ( !_init() )
+      {
+        return false;
+      }
+    }
         
     makeCurrent();
 
+    if (_sharedGLContext)
+        wglShareLists(_sharedGLContext, _glcontext);
+
     _realized = true;
 
     std::vector <Producer::ref_ptr<Callback> >::iterator p;
@@ -427,7 +441,7 @@ void RenderSurface::KillGLWindow()
         ShowCursor(TRUE);
     }
 
-    if (_glcontext && !wglDeleteContext(_glcontext))
+    if ( _glcontext && !wglDeleteContext(_glcontext))
     {
         doInternalError("Release Rendering Context Failed.");
     }
@@ -439,6 +453,11 @@ void RenderSurface::KillGLWindow()
     }
     _hdc = NULL;
 
+    if( _visualInfo && _ownVisualInfo ) {
+        delete _visualInfo;
+        _visualInfo = NULL;
+    }
+
     if (_ownWindow)
     {
         if (_win && !DestroyWindow(_win))
@@ -454,6 +473,98 @@ void RenderSurface::KillGLWindow()
     }
 }
 
+bool RenderSurface::_createVisualInfo()
+{
+    if( _visualID == NULL ) 
+    {
+        if( _glcontext != NULL && _hdc != NULL ) 
+        {
+            // _hdc presence means that drawable (Window or Pbuffer) exists as well
+            _visualID = GetPixelFormat( _hdc );
+            if( _visualID == NULL && _visualInfo != NULL && _ownVisualInfo == false ) 
+            {
+                // it might be null but suggested _visualInfo was passed from outside world
+                // I could leave it to the VisualChooser to select the visual info 
+                // but VisualInfo in VisualChooser gets reset and overwritten 
+                // by each addExtendedAttribute so its unlikely that it would work
+
+                _visualID = ChoosePixelFormat( _hdc, _visualInfo );
+            }
+        } else {
+
+            ref_ptr<VisualChooser> lvc;
+            if (_visualChooser.valid())
+            {
+                lvc = _visualChooser;
+            }
+            else
+            {
+                lvc = new VisualChooser;
+                lvc->setSimpleConfiguration(_drawableType != DrawableType_PBuffer);
+            }
+
+            if (_drawableType == DrawableType_PBuffer)
+            {
+                lvc->addExtendedAttribute(WGL_DRAW_TO_PBUFFER_ARB, true);
+                switch (_rtt_mode)
+                {
+                    case RenderToRGBTexture: 
+                        lvc->addExtendedAttribute(WGL_BIND_TO_TEXTURE_RGB_ARB, true);
+                    break;
+                    case RenderToRGBATexture: 
+                        lvc->addExtendedAttribute(WGL_BIND_TO_TEXTURE_RGBA_ARB, true);
+                    break;
+                    default: ;
+                }
+            }
+
+            if (_drawableType == DrawableType_Window)
+            {
+                lvc->addExtendedAttribute(WGL_DRAW_TO_WINDOW_ARB, true);
+            }
+
+            if ( lvc->choose(&_hdc, _screen,lvc->getStrictAdherence()) )
+            {
+                _visualID = lvc->getVisualID();    
+
+            }
+        }
+    }
+
+    if( _visualID == NULL )
+        return false;
+
+    if( _visualInfo == NULL ) 
+    {
+        // Try to find _visualInfo from _visualID
+        _visualInfo = new VisualInfo;
+        _ownVisualInfo = true;
+
+        HDC hdc = _hdc;
+
+        // HDC may be not set here but I hope that 
+        // description for root window will be correct
+
+        if( hdc == NULL ) 
+            hdc = GetDC( NULL ); 
+            
+        if( !DescribePixelFormat( hdc, _visualID, sizeof(PIXELFORMATDESCRIPTOR), _visualInfo ) ) {
+            delete _visualInfo;
+            _visualInfo = NULL;            
+        }
+
+        if( hdc != _hdc )
+            ReleaseDC( NULL, hdc );
+    }
+
+    // WHAT to do if we have valid _visualID but no _visualInfo ? 
+    // I would consider this a succes because only _visualID will be used later 
+    // If you decide otherwise ucomment line below
+    // if( _visualInfo ) return NULL;
+
+    return true;
+}
+
 bool RenderSurface::_init()
 {
     if (_drawableType == DrawableType_Window)
@@ -599,8 +710,13 @@ bool RenderSurface::_init()
                 rect.bottom = _screenHeight - _windowY;
                 rect.right = _windowWidth + _windowX;
 
-                // map from client area dimensions to outer window dimensions
-                AdjustWindowRectEx(&rect, dwStyle, FALSE, dwExStyle);
+                if(_decorations) {
+                  // map from client area dimensions to outer window dimensions
+                  AdjustWindowRectEx(&rect, dwStyle, FALSE, dwExStyle);
+                }
+                else {
+                  dwStyle=WS_POPUP;
+                }
 
                 x = rect.left;
                 y = rect.top;
@@ -626,21 +742,25 @@ bool RenderSurface::_init()
                 return FALSE;
             }
         }
-        else if( (_windowWidth == UnknownDimension) && (_windowHeight == UnknownDimension) )
+        else 
         {
-            POINT p;
+            if( (_windowWidth == UnknownDimension) && (_windowHeight == UnknownDimension) )
+            {
+                POINT p;
 
-            GetClientRect(_win, &rect);
-            _windowWidth = rect.right - rect.left;
-            _windowHeight = rect.bottom - rect.top;
+                GetClientRect(_win, &rect);
+                _windowWidth = rect.right - rect.left;
+                _windowHeight = rect.bottom - rect.top;
 
-            // map window origin to screen
-            p.x = rect.left;
-            p.y = rect.top;
-            ClientToScreen(_win, &p);
+                // map window origin to screen
+                p.x = rect.left;
+                p.y = rect.top;
+                ClientToScreen(_win, &p);
+
+                _windowX = p.x;
+                _windowY = _screenHeight - _windowHeight - p.y;
+            }
 
-            _windowX = p.x;
-            _windowY = _screenHeight - _windowHeight - p.y;
             _isFullScreen = false;
             _oldWndProc = (WNDPROC)SetWindowLong(_win, GWL_WNDPROC, (long)((WNDPROC)s_proc));
         }
@@ -672,47 +792,18 @@ bool RenderSurface::_init()
         }
     }
 
-    unsigned int visualId;
-
-    ref_ptr<VisualChooser> lvc;
-    if (_visualChooser.valid())
+    if( _visualID == 0 && !_createVisualInfo() ) 
     {
-        lvc = _visualChooser;
+        doInternalError("Can't assign Visual (PIXELFORMAT) to surface.");
+        if (_drawableType == DrawableType_Window)
+            KillGLWindow();
+        return FALSE;
     }
-    else
-    {
-        lvc = new VisualChooser;
-        bool doublebuffer = (_drawableType == DrawableType_Window);
-        lvc->setSimpleConfiguration(doublebuffer);
-    }
-
-    if (_drawableType == DrawableType_PBuffer)
-    {
-        lvc->addExtendedAttribute(WGL_DRAW_TO_PBUFFER_ARB, true);
-        switch (_rtt_mode)
-        {
-            case RenderToRGBTexture: 
-                lvc->addExtendedAttribute(WGL_BIND_TO_TEXTURE_RGB_ARB, true);
-                break;
-            case RenderToRGBATexture: 
-                lvc->addExtendedAttribute(WGL_BIND_TO_TEXTURE_RGBA_ARB, true);
-                break;
-            default: ;
-        }
-    }
-
-    if (_drawableType == DrawableType_Window)
-    {
-        lvc->addExtendedAttribute(WGL_DRAW_TO_WINDOW_ARB, true);
-    }
-
-    lvc->choose(&_hdc, _screen);
-    visualId = lvc->getVisualID();
 
     if (_drawableType == DrawableType_Window)
     {
         PIXELFORMATDESCRIPTOR pfd;
-        if (!SetPixelFormat(_hdc, visualId, &pfd))
+        if (!SetPixelFormat(_hdc, _visualID, &pfd))
         {
             doInternalError("Can't Set The PixelFormat.");
             KillGLWindow();
@@ -721,65 +812,78 @@ bool RenderSurface::_init()
     }
     else
     {
+        WGLExtensions *ext = WGLExtensions::instance();
+        if (!ext)
+        {
+            std::cerr << "Producer::RenderSurface: WGL extensions not available" << std::endl;
+            return false;
+        }
+
+        if (!ext->isSupported(WGLExtensions::ARB_pbuffer))
+        {
+            std::cerr << "Producer::RenderSurface: PBuffers are not supported" << std::endl;
+            return false;
+        }
+
         std::vector<int> pbattr(_user_pbattr);
 
         if (_rtt_mode != RenderToTextureMode_None)
         {
-            pbattr.push_back(WGL_TEXTURE_FORMAT_ARB);
-
-            switch (_rtt_mode)
+            if (!ext->isSupported(WGLExtensions::ARB_render_texture))
             {
-                case RenderToRGBTexture:
-                    pbattr.push_back(WGL_TEXTURE_RGB_ARB);
-                    break;
-                case RenderToRGBATexture:
-                    pbattr.push_back(WGL_TEXTURE_RGBA_ARB);
-                    break;
-                default: 
-                    pbattr.push_back(static_cast<int>(_rtt_mode));
+                std::cerr << "Producer::RenderSurface: Render-To-Texture is not supported" << std::endl;
             }
-
-            pbattr.push_back(WGL_TEXTURE_TARGET_ARB);
-
-            switch (_rtt_target)
+            else
             {
-                case TextureCUBE:
-                    pbattr.push_back(WGL_TEXTURE_CUBE_MAP_ARB);
-                    break;
-                case Texture1D:
-                    pbattr.push_back(WGL_TEXTURE_1D_ARB);
-                    break;
-                case Texture2D:
-                    pbattr.push_back(WGL_TEXTURE_2D_ARB);
-                    break;
-                default: 
-                    pbattr.push_back(static_cast<int>(_rtt_target));
-            }
+                pbattr.push_back(WGL_TEXTURE_FORMAT_ARB);
 
-            if (_rtt_options & RequestSpaceForMipMaps)
-            {
-                pbattr.push_back(WGL_MIPMAP_TEXTURE_ARB);
-                pbattr.push_back(1);
-            }
-                
-            if (_rtt_options & RequestLargestPBuffer)
-            {
-                pbattr.push_back(WGL_PBUFFER_LARGEST_ARB);
-                pbattr.push_back(1);
+                switch (_rtt_mode)
+                {
+                    case RenderToRGBTexture:
+                        pbattr.push_back(WGL_TEXTURE_RGB_ARB);
+                        break;
+                    case RenderToRGBATexture:
+                        pbattr.push_back(WGL_TEXTURE_RGBA_ARB);
+                        break;
+                    default: 
+                        pbattr.push_back(static_cast<int>(_rtt_mode));
+                }
+
+                pbattr.push_back(WGL_TEXTURE_TARGET_ARB);
+
+                switch (_rtt_target)
+                {
+                    case TextureCUBE:
+                        pbattr.push_back(WGL_TEXTURE_CUBE_MAP_ARB);
+                        break;
+                    case Texture1D:
+                        pbattr.push_back(WGL_TEXTURE_1D_ARB);
+                        break;
+                    case Texture2D:
+                        pbattr.push_back(WGL_TEXTURE_2D_ARB);
+                        break;
+                    default: 
+                        pbattr.push_back(static_cast<int>(_rtt_target));
+                }
+
+                if (_rtt_options & RequestSpaceForMipMaps)
+                {
+                    pbattr.push_back(WGL_MIPMAP_TEXTURE_ARB);
+                    pbattr.push_back(1);
+                }
+
+                if (_rtt_options & RequestLargestPBuffer)
+                {
+                    pbattr.push_back(WGL_PBUFFER_LARGEST_ARB);
+                    pbattr.push_back(1);
+                }
             }
         }
 
         // Terminate array
         pbattr.push_back(0);
 
-        WGLExtensions *ext = WGLExtensions::instance();
-        if (!ext)
-        {
-            doInternalError("Can't initialize WGL functions");
-            return false;
-        }
-
-        _win = reinterpret_cast<Window>(ext->wglCreatePBuffer(_hdc, visualId, _windowWidth, _windowHeight, &pbattr.front()));
+        _win = reinterpret_cast<Window>(ext->wglCreatePBuffer(_hdc, _visualID, _windowWidth, _windowHeight, &pbattr.front()));
         if (!_win)
         {
             doInternalError("Can't create the PBuffer");
@@ -801,11 +905,24 @@ bool RenderSurface::_init()
         _windowHeight = actual_height;
     }
 
-    if (!(_glcontext = wglCreateContext( _hdc )))
+    if( _glcontext == NULL )
     {
-        doInternalError("Can't Create A GL Rendering Context.");
-        KillGLWindow();
-        return FALSE;
+        if (!(_glcontext = wglCreateContext( _hdc )))
+        {
+            doInternalError("Can't Create A GL Rendering Context.");
+            KillGLWindow();
+            return FALSE;
+        }
+    }
+
+    // Initialize globallySharedGLContext if global sharing is requested and 
+    // the global variable is still NULL.
+    // Caveat is if the user attempted to share GL contexts already through realize()
+    if( _shareAllGLContexts == true && 
+        _globallySharedGLContext == 0L &&
+        _sharedGLContext == 0L )
+    {
+        _globallySharedGLContext = _glcontext;
     }
 
 // ?? WHY DOES THIS CRASH HERE ??  nhv
@@ -877,8 +994,10 @@ void RenderSurface::fullScreen( bool flag )
 
         /* Get rid of fullscreen mode, if it exists */
         if ( style & WS_POPUP ) {
-            style &= ~WS_POPUP;
-            style |= WS_OVERLAPPEDWINDOW;
+            if(_decorations) { // Only add border if we want decorations
+                style &= ~WS_POPUP;
+                style |= WS_OVERLAPPEDWINDOW;
+            }
             SetWindowLong(_win, GWL_STYLE, style);
             flags |= SWP_FRAMECHANGED;
         }
@@ -894,8 +1013,10 @@ void RenderSurface::fullScreen( bool flag )
         rect.bottom = _screenHeight - _windowY;
         rect.right = _windowWidth + _windowX;
 
-        // adjust from client dimensions to window outer dimentions
-        AdjustWindowRectEx(&rect, WS_OVERLAPPEDWINDOW, FALSE, WS_EX_APPWINDOW | WS_EX_WINDOWEDGE);
+        if(_decorations) {
+            // adjust from client dimensions to window outer dimentions
+            AdjustWindowRectEx(&rect, WS_OVERLAPPEDWINDOW, FALSE, WS_EX_APPWINDOW | WS_EX_WINDOWEDGE);
+        }
 
         int x = rect.left;
         int y = rect.top;
@@ -914,7 +1035,7 @@ bool RenderSurface::makeCurrent( void ) const
     if (_rtt_mode != RenderToTextureMode_None && _drawableType == DrawableType_PBuffer)    
     {        
         WGLExtensions *tempext = WGLExtensions::instance();
-        if (tempext)
+        if (tempext && tempext->isSupported(WGLExtensions::ARB_render_texture))
         {
             tempext->wglReleaseTexImage(reinterpret_cast<HPBUFFERARB>(_win), WGL_FRONT_LEFT_ARB);
             tempext->wglReleaseTexImage(reinterpret_cast<HPBUFFERARB>(_win), WGL_BACK_LEFT_ARB);
@@ -924,7 +1045,7 @@ bool RenderSurface::makeCurrent( void ) const
     if(_readDrawableRenderSurface && _readDrawableRenderSurface->isRealized())
     {
         WGLExtensions *tempext = WGLExtensions::instance();
-        if (tempext)
+        if (tempext && tempext->isSupported(WGLExtensions::ARB_make_current_read))
         {
             if (!tempext->wglMakeContextCurrent(_hdc, _readDrawableRenderSurface->_hdc, _glcontext))
                 return false;
@@ -947,18 +1068,24 @@ bool RenderSurface::makeCurrent( void ) const
 
     if (_rtt_dirty_mipmap && (_rtt_options & RequestSpaceForMipMaps) != 0)
     {
-        std::vector<int> pbattr;
-        pbattr.push_back(WGL_MIPMAP_LEVEL_ARB);
-        pbattr.push_back(_rtt_mipmap);
-        ext->wglSetPBufferAttrib(reinterpret_cast<HPBUFFERARB>(_win), &pbattr.front());
+        if (ext->isSupported(WGLExtensions::ARB_render_texture))
+        {
+            std::vector<int> pbattr;
+            pbattr.push_back(WGL_MIPMAP_LEVEL_ARB);
+            pbattr.push_back(_rtt_mipmap);
+            ext->wglSetPBufferAttrib(reinterpret_cast<HPBUFFERARB>(_win), &pbattr.front());
+        }
     }
 
     if (_rtt_dirty_face && _rtt_target == TextureCUBE)
     {
-        std::vector<int> pbattr;
-        pbattr.push_back(WGL_CUBE_MAP_FACE_ARB);
-        pbattr.push_back(0x207D + _rtt_face);
-        ext->wglSetPBufferAttrib(reinterpret_cast<HPBUFFERARB>(_win), &pbattr.front());
+        if (ext->isSupported(WGLExtensions::ARB_render_texture))
+        {
+            std::vector<int> pbattr;
+            pbattr.push_back(WGL_CUBE_MAP_FACE_ARB);
+            pbattr.push_back(0x207D + _rtt_face);
+            ext->wglSetPBufferAttrib(reinterpret_cast<HPBUFFERARB>(_win), &pbattr.front());
+        }
     }
 
     return true;
@@ -1082,60 +1209,60 @@ void RenderSurface::_resizeWindow()
 
 void RenderSurface::_setBorder( bool flag )
 {
-	//Flags that will be used for SetWindowPos
-	UINT flags = SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOSENDCHANGING |
+    //Flags that will be used for SetWindowPos
+    UINT flags = SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOSENDCHANGING |
                     SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED;
 
-	//Get current style of window
-	DWORD style = GetWindowLong(_win, GWL_STYLE);
+    //Get current style of window
+    DWORD style = GetWindowLong(_win, GWL_STYLE);
 
-	//Get current client dimensions
-	RECT rect;
-	GetClientRect( _win, &rect );
+    //Get current client dimensions
+    RECT rect;
+    GetClientRect( _win, &rect );
 
-	//Variables that will hold new size of window
-	int cx, cy;
+    //Variables that will hold new size of window
+    int cx, cy;
 
-	if(flag)
-	{
-		//Add border style to window
-		style &= ~WS_POPUP;
+    if(flag)
+    {
+        //Add border style to window
+        style &= ~WS_POPUP;
         style |= WS_OVERLAPPEDWINDOW;
 
-		//Calculate new size of window with border
-		rect.left = 0;
-		rect.top = 0;
-		rect.right = rect.right - rect.left;
-		rect.bottom = rect.bottom - rect.top;
-		
-		LONG styleEx = GetWindowLong(_win,GWL_EXSTYLE);
-		AdjustWindowRectEx(&rect, style, FALSE, styleEx);
+        //Calculate new size of window with border
+        rect.left = 0;
+        rect.top = 0;
+        rect.right = rect.right - rect.left;
+        rect.bottom = rect.bottom - rect.top;
+        
+        LONG styleEx = GetWindowLong(_win,GWL_EXSTYLE);
+        AdjustWindowRectEx(&rect, style, FALSE, styleEx);
 
-		cx = rect.right - rect.left;
-		cy = rect.bottom - rect.top;
-		flags &= ~(SWP_NOSIZE);
-	}
-	else
-	{
-		//Remove border style from window
-		style &= ~WS_OVERLAPPEDWINDOW;
-		style |= WS_POPUP;
+        cx = rect.right - rect.left;
+        cy = rect.bottom - rect.top;
+        flags &= ~(SWP_NOSIZE);
+    }
+    else
+    {
+        //Remove border style from window
+        style &= ~WS_OVERLAPPEDWINDOW;
+        style |= WS_POPUP;
 
-		//Calculate new size of window without border
-		cx = rect.right - rect.left;
-		cy = rect.bottom - rect.top;
-		flags &= ~(SWP_NOSIZE);
-	}
+        //Calculate new size of window without border
+        cx = rect.right - rect.left;
+        cy = rect.bottom - rect.top;
+        flags &= ~(SWP_NOSIZE);
+    }
 
-	//Apply new window style
-	SetWindowLong(_win, GWL_STYLE, style);
-	
-	//Need to call this to change window size and update frame style
-	SetWindowPos(_win, 
-					HWND_TOP,
-					0, 0, 
-					cx, cy, 
-					flags);
+    //Apply new window style
+    SetWindowLong(_win, GWL_STYLE, style);
+    
+    //Need to call this to change window size and update frame style
+    SetWindowPos(_win, 
+                    HWND_TOP,
+                    0, 0, 
+                    cx, cy, 
+                    flags);
 }
 
 void RenderSurface::_setWindowName( const std::string & name )
@@ -1159,6 +1286,12 @@ void RenderSurface::_setCursor(Cursor cursor)
     else
     {
         _currentCursor = cursor ? cursor : _nullCursor;
+
+        if ( _currentCursor == NULL )
+        {
+            _currentCursor = ::LoadCursor( NULL, IDC_ARROW );
+        }
+
         ::SetCursor(_currentCursor);
     }
 }
@@ -1236,6 +1369,12 @@ void RenderSurface::_fini()
 void RenderSurface::bindPBufferToTexture(BufferType buffer) const
 {
     WGLExtensions *ext = WGLExtensions::instance();
+    if (!ext || !ext->isSupported(WGLExtensions::ARB_render_texture))
+    {
+        // RTT is not supported, fall back to good old copy-to-texture.
+        glCopyTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, 0, 0, _windowWidth, _windowHeight);
+        return;
+    }
 
     switch (buffer)
     {
@@ -1250,11 +1389,6 @@ void RenderSurface::bindPBufferToTexture(BufferType buffer) const
     }        
 }
 
-void RenderSurface::_choosePixelFormat( VisualInfo *)
-{
-	//TODO
-}
-
 void RenderSurface::_useOverrideRedirect( bool )
 {
     // Ignored on Windows.  Flag value remains false.
@@ -1267,3 +1401,5 @@ void RenderSurface::_initThreads()
 
 
 #endif  // WIN32
+
+
