@@ -43,7 +43,7 @@ void (*glXGetProcAddressARB(const GLubyte *procName))(void)
 
 
 #if (defined(__APPLE__) || defined(macintosh)) || defined(_AIX)
-  #define USE_LOCAL_glXGetFBConfigFromVisual 
+      #define USE_LOCAL_glXGetFBConfigFromVisual 
 #endif
 
 #ifdef USE_LOCAL_glXGetFBConfigFromVisual
@@ -61,7 +61,7 @@ static GLXFBConfig _glXGetFBConfigFromVisual(::Display *dpy, XVisualInfo *visinf
                 return configs[i];
         }
     }
-	return NULL;
+    return NULL;
 }
 #endif
 
@@ -69,14 +69,14 @@ unsigned int RenderSurface::getNumberOfScreens()
 {
     if( _numScreens == UnknownAmount )
     {
-    	Display *dpy = XOpenDisplay("");
-    	if( dpy == NULL )
-    	{
-        	std::cerr << "Unable to open display \"" << XDisplayName( ":0.0") << "\".  Is the DISPLAY environmental variable set?\n";
-        	return 0;
-    	}
-    	_numScreens = ScreenCount(dpy); 
-    	XCloseDisplay(dpy);
+        Display *dpy = XOpenDisplay("");
+        if( dpy == NULL )
+        {
+            std::cerr << "Unable to open display \"" << XDisplayName( ":0.0") << "\".  Is the DISPLAY environmental variable set?\n";
+            return 0;
+        }
+        _numScreens = ScreenCount(dpy); 
+        XCloseDisplay(dpy);
     }
     return _numScreens;
 }
@@ -151,6 +151,42 @@ bool RenderSurface::realize( VisualChooser *vc, GLContext sharedGLContext )
 
     return _realized;
 }
+
+bool RenderSurface::_createVisualInfo()
+{
+// If Window was already defined, lets grab that visual
+    if( _win != 0 )
+    {
+        XWindowAttributes watt;
+        XGetWindowAttributes( _dpy, _win, &watt );
+        XVisualInfo temp;
+        temp.visualid = XVisualIDFromVisual(watt.visual);
+        int n;
+        _visualInfo = XGetVisualInfo( _dpy, VisualIDMask, &temp, &n );
+    }
+    else
+    {
+        if( !_visualChooser.valid() )
+        {
+            ref_ptr<VisualChooser> lvc = new VisualChooser;
+            lvc->setSimpleConfiguration(_drawableType != DrawableType_PBuffer);
+            _visualInfo = lvc->choose(_dpy, _screen);
+        }
+        else
+        {
+            _visualInfo = _visualChooser->choose(_dpy,_screen);
+        }
+    }
+
+    _visualID = 0;
+
+    if( _visualInfo == NULL )
+        return false;
+
+    _visualID = _visualInfo->visualid;
+
+    return true;
+}
  
 bool RenderSurface::_init()
 {
@@ -177,13 +213,13 @@ bool RenderSurface::_init()
     _screenWidth = DisplayWidth(_dpy, _screen );
     _screenHeight = DisplayHeight(_dpy, _screen );
 
-        // Query for GLX extension
+    // Query for GLX extension
     int erb, evb;
     if( glXQueryExtension( _dpy, &erb, &evb)  == False )
     {
         std::cerr << "RenderSurface::realize() " << 
-                    XDisplayName( dpyname) <<
-                    " has no GLX extension." << std::endl;
+                       XDisplayName( dpyname) <<
+                     " has no GLX extension." << std::endl;
          if( _dpy_created_locally )
          {
              XCloseDisplay( _dpy );
@@ -191,68 +227,53 @@ bool RenderSurface::_init()
          }
          return false;
     }
-    
-    if( _visualInfo == NULL )
+
+    if( _glcontext == NULL )
     {
-    // If Window was already defined, lets grab that visual
-        if( _win != 0 )
+        if( _visualInfo == NULL && _createVisualInfo() == false )
         {
-            XWindowAttributes watt;
-            XGetWindowAttributes( _dpy, _win, &watt );
-            XVisualInfo temp;
-            temp.visualid = XVisualIDFromVisual(watt.visual);
-            int n;
-            _visualInfo = XGetVisualInfo( _dpy, VisualIDMask, &temp, &n );
-        }
-        else
-        {
-            if( !_visualChooser.valid() )
+            if( _dpy_created_locally )
             {
-                ref_ptr<VisualChooser> lvc = new VisualChooser;
-                lvc->setSimpleConfiguration();
-                _visualInfo = lvc->choose(_dpy, _screen);
+                XCloseDisplay( _dpy );
+                _dpy = 0;
             }
-            else
+            return false;
+        }
+
+        if((_glcontext = glXCreateContext( _dpy, _visualInfo, _sharedGLContext, GL_TRUE ))==(GLXContext) 0L)
+        {
+            std::cerr << "RenderSurface:realize() - Can't create GLX Context." << std::endl;
+            if( _dpy_created_locally )
             {
-                _visualInfo = _visualChooser->choose(_dpy,_screen);
+                XCloseDisplay( _dpy );
+                _dpy = 0L;
             }
+            return false;
         }
     }
 
-    if( _visualInfo == NULL )
-    {
-        if( _dpy_created_locally )
-        {
-            XCloseDisplay( _dpy );
-            _dpy = 0;
-        }
-        return false;
-    }
-
-
-    if((_glcontext = glXCreateContext( _dpy, _visualInfo, _sharedGLContext, GL_TRUE ))==(GLXContext) 0L)
-    {
-        std::cerr << "RenderSurface:realize() - Can't create GLX Context." << std::endl;
-        if( _dpy_created_locally )
-        {
-            XCloseDisplay( _dpy );
-            _dpy = 0L;
-        }
-        return false;
-    }
-
     // Initialize globallySharedGLContext if global sharing is requested and 
     // the global variable is still NULL.
     // Caveat is if the user attempted to share GL contexts already through realize()
     if( _shareAllGLContexts == true && 
-            _globallySharedGLContext == 0L &&
-            _sharedGLContext == 0L )
+        _globallySharedGLContext == 0L &&
+        _sharedGLContext == 0L )
     {
         _globallySharedGLContext = _sharedGLContext = _glcontext;
     }
 
     if( _win == 0 )
     {
+        if( _visualInfo == NULL && _createVisualInfo() == false )
+        {
+            if( _dpy_created_locally )
+            {
+                XCloseDisplay( _dpy );
+                _dpy = 0;
+            }
+            return false;
+        }
+
         if( _drawableType == DrawableType_Window )
         {
             if( _parent == (Window)0L )
@@ -432,7 +453,6 @@ bool RenderSurface::_init()
 
 bool RenderSurface::makeCurrent( void ) const
 {
-
     if( _readDrawableRenderSurface != 0L && _readDrawableRenderSurface->isRealized())
     {
 #ifdef GLX_VERSION_1_3
@@ -702,7 +722,7 @@ void RenderSurface::_setBorder( bool flag )
     if( (atom = XInternAtom( _dpy, "_MOTIF_WM_HINTS", 0 )) != None )
     {
 // Hack for sending 64 bit atom to Xserver
-#if defined( _MIPS_SIM) && (_MIPS_SIM == _MIPS_SIM_ABI64) || defined ( __ia64 ) || defined (__amd64 )
+#if defined( _MIPS_SIM) && (_MIPS_SIM == _MIPS_SIM_ABI64) || defined ( __ia64 ) || defined (__amd64 ) || defined(__x86_64__)
         struct {
             CARD32 flags0;
             CARD32 flags1;
@@ -716,7 +736,7 @@ void RenderSurface::_setBorder( bool flag )
             CARD32 status1;
         } wmHints;
 
-#if defined( __ia64 ) || defined (__amd64)
+#if defined( __ia64 ) || defined (__amd64)  || defined (__x86_64__)
         wmHints.flags0       = (1L << 1);
         wmHints.functions0   = 0;
         wmHints.decorations0 = flag;
@@ -889,15 +909,9 @@ void RenderSurface::_fini()
 
 void RenderSurface::bindPBufferToTexture(BufferType buffer) const
 {
-    // Ok... a bit of a hack.
     glCopyTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, 0, 0, _windowWidth, _windowHeight);
 }
 
-void RenderSurface::_choosePixelFormat( VisualInfo *)
-{
-	//TODO
-}
-
 void RenderSurface::_useOverrideRedirect( bool flag)
 {
     if( _drawableType != DrawableType_Window )
@@ -914,14 +928,14 @@ void RenderSurface::_useOverrideRedirect( bool flag)
 
 void RenderSurface::_initThreads()
 {
-	// For multi-threaded X environment we must initialize the
-	// multi-threaded support before we do anything else -- LJE
-	static bool initialized = false;    // We shouldn't do this multiple times...
+    // For multi-threaded X environment we must initialize the
+    // multi-threaded support before we do anything else -- LJE
+    static bool initialized = false;    // We shouldn't do this multiple times...
 
     if( initialized )
         return;
 
-	initialized = true;
+    initialized = true;
 
 #ifdef GLX_SUN_init_threads
     Display *dpy = XOpenDisplay(":0.0");
@@ -952,7 +966,7 @@ void RenderSurface::_initThreads()
      //   lock in a toolkit or through explicit client programming), Xlib thread initialization is not required.
      //   It is recommended that single-threaded programs not call this function.
      //
-     // Since OSG enforces a policy of one-thread per client, this is not necessary and, potentially, detrimental
+     // Since Producer enforces a policy of one-thread per client, this is not necessary and, potentially, detrimental
      // Don Burns - May 3, 2005
 
 #if 0
@@ -973,3 +987,4 @@ void RenderSurface::_initThreads()
 
 #endif
 
+
